#Region "Header"
'      .NET Sample
'
'      Copyright (c) 2006 by Autodesk, Inc.
'
'      Permission to use, copy, modify, and distribute this software
'      for any purpose and without fee is hereby granted, provided
'      that the above copyright notice appears in all copies and
'      that both that copyright notice and the limited warranty and
'      restricted rights notice below appear in all supporting
'      documentation.
'
'      AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
'      AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
'      MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
'      DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
'      UNINTERRUPTED OR ERROR FREE.
'
'      Use, duplication, or disclosure by the U.S. Government is subject to
'      restrictions set forth in FAR 52.227-19 (Commercial Computer
'      Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
'      (Rights in Technical Data and Computer Software), as applicable.
#End Region

#Region "Namespaces"
Imports System
Imports System.Collections.Generic
Imports System.Text

Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.Aec.DatabaseServices
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.EditorInput
Imports Autodesk.Aec.Modeler
Imports Autodesk.Aec.Arch.DatabaseServices
Imports Autodesk.AutoCAD.Geometry
Imports System.Threading
#End Region

<Assembly: Autodesk.AutoCAD.Runtime.ExtensionApplication(GetType(AModelerSampleMgd))> 
<Assembly: Autodesk.AutoCAD.Runtime.CommandClass(GetType(AModelerSampleMgd))> 

Public Class AModelerSampleMgd
    Implements IExtensionApplication

#Region "IExtensionApplication"

    Public Sub Initialize() Implements IExtensionApplication.Initialize
        Dim ed As Autodesk.AutoCAD.EditorInput.Editor = Application.DocumentManager.MdiActiveDocument.Editor
        ed.WriteMessage("Loading AModeler Sample..." & vbCrLf)
        ed.WriteMessage("Input AModelerSample to start:" & vbCrLf)
    End Sub

    Public Sub Terminate() Implements IExtensionApplication.Terminate
    End Sub

#End Region

#Region "CommandMethod"

    <Autodesk.AutoCAD.Runtime.CommandMethod("AModelerSampleMgd", "AModelerSample", Autodesk.AutoCAD.Runtime.CommandFlags.Modal)> _
    Public Sub AModelerSample()
        Dim cmd As String = GetInputCommand()
        If commandExec(cmd) = True Then
            ZoomSample()
            Application.DocumentManager.MdiActiveDocument.Editor.Document.SendStringToExecute("AModelerSample" & vbCr, False, False, True)
        End If
    End Sub

#End Region

    Private Sub InitSample()
        Clear()
        Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("==========================" & vbCrLf)
        Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("Initializing AModeler .NET API sample by creating a base mass element entity at 0,0." & vbCrLf)
        Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("==========================" & vbCrLf)
        Init()
    End Sub

    Private Sub Clear()

        Dim db As Autodesk.AutoCAD.DatabaseServices.Database = HostApplicationServices.WorkingDatabase
        Dim tm As Autodesk.AutoCAD.DatabaseServices.TransactionManager = db.TransactionManager
        Dim t As Transaction = tm.StartTransaction()
        Dim bt As BlockTable = tm.GetObject(db.BlockTableId, OpenMode.ForRead, False)
        Dim btr As BlockTableRecord = tm.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForWrite, False)

        For Each id As Autodesk.AutoCAD.DatabaseServices.ObjectId In btr
            Dim met As MassElement = tm.GetObject(id, OpenMode.ForWrite)
            If met <> Nothing Then
                met.Erase()
            End If
        Next id
        t.Commit()
        t.Dispose()
    End Sub

    Private Function GetSelectedMassElement() As MassElement()
        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor
        Dim Opts As PromptSelectionOptions = New PromptSelectionOptions()
        Dim psr As PromptSelectionResult = ed.GetSelection(Opts)

        If psr.Status = PromptStatus.OK Then
            Dim ss As Autodesk.AutoCAD.EditorInput.SelectionSet = psr.Value
            Dim oids As Autodesk.AutoCAD.DatabaseServices.ObjectId() = ss.GetObjectIds()
            Dim count As Int32 = oids.Length
            Dim result(count) As MassElement
            Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database
            Dim tm As Autodesk.AutoCAD.DatabaseServices.TransactionManager = db.TransactionManager
            Dim trans As Transaction = tm.StartTransaction()
            Dim met As MassElement = New MassElement()
            For i As Int32 = 0 To count - 1
                Dim ety As MassElement = tm.GetObject(oids(i), OpenMode.ForWrite, True)
                result(i) = ety
            Next i
            trans.Commit()
            trans.Dispose()
            Return result
        End If
        Return Nothing
    End Function

    Private Function GetInputCommand() As String

        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor
        Dim pko As PromptKeywordOptions = New PromptKeywordOptions("Input the command:")
        pko.Keywords.Add("InitSample")
        pko.Keywords.Add("StretchBody")
        pko.Keywords.Add("AddBody")
        pko.Keywords.Add("ListBody")
        pko.Keywords.Add("SectionBody")
        pko.Keywords.Add("MultiplyBody")
        pko.Keywords.Add("SubBody")
        pko.Keywords.Add("SubBody")
        pko.Keywords.Add("Clear")
        pko.Keywords.Add("BooleanOp")
        pko.Keywords.Add("CombineBody")
        pko.Keywords.Add("TranslateBody")
        pko.Keywords.Add("RotateBody")
        pko.Keywords.Add("ScaleBody")
        pko.Keywords.Add("ListPickBody")
        pko.Keywords.Add("Quit")
        Dim result As PromptResult = ed.GetKeywords(pko)
        Return result.StringResult

    End Function

    Private Function commandExec(ByVal cmdString As String) As Boolean

        If (cmdString.ToLower() = "initsample") Then
            InitSample()
            Return True
        End If

        If (cmdString.ToLower() = "clear") Then
            Clear()
            Return True
        End If

        Dim db As Database = HostApplicationServices.WorkingDatabase
        Dim tm As Autodesk.AutoCAD.DatabaseServices.TransactionManager = db.TransactionManager
        Dim t As Transaction = tm.StartTransaction()
        Dim bt As BlockTable = tm.GetObject(db.BlockTableId, OpenMode.ForRead, False)
        Dim btr As BlockTableRecord = tm.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForWrite, False)
        Dim result As Boolean = True
        For Each id As Autodesk.AutoCAD.DatabaseServices.ObjectId In btr

            Dim met As MassElement = tm.GetObject(id, OpenMode.ForWrite)
            If met = Nothing Then
                Continue For
            End If
            Dim b As Autodesk.Aec.Modeler.Body = met.Body
            Dim b2 As Autodesk.Aec.Modeler.Body = Nothing

            If (cmdString.ToLower() = "subbody") Then
                Dim subB As Autodesk.Aec.Modeler.Body = Autodesk.Aec.Modeler.Body.Cone(New LineSegment3d(New Point3d(0, 0, 0), New Point3d(0, 0, 5)), 1, 1.5, 10)
                b2 = b - subB
                met.SetBody(b2, True)

            ElseIf (cmdString.ToLower() = "multiplybody") Then
                Dim bb As Autodesk.Aec.Modeler.Body = Autodesk.Aec.Modeler.Body.Cone(New LineSegment3d(New Point3d(0, 0, 0), New Point3d(0, 0, 5)), 1, 1.5, 10)
                b2 = b * bb
                met.SetBody(b2, True)

            ElseIf (cmdString.ToLower() = "sectionbody") Then
                b.Section(New Plane(New Point3d(0, 0, 0), New Vector3d(0, 1, 1)), True)
                met.SetBody(b, False)

            ElseIf (cmdString.ToLower() = "listbody") Then
                ListBody(met)

            ElseIf (cmdString.ToLower() = "addbody") Then
                Dim bb As Autodesk.Aec.Modeler.Body = Autodesk.Aec.Modeler.Body.Cylinder(New LineSegment3d(New Point3d(0, 0, 0), New Point3d(10, 0, 0)), 5, 20)
                b2 = b + bb
                met.SetBody(b2, True)

            ElseIf (cmdString.ToLower() = "stretchbody") Then
                b.Stretch(New LineSegment3d(New Point3d(0, 0, 0), New Point3d(0, 10, 0)))
                met.SetBody(b, False)

            ElseIf (cmdString.ToLower() = "booleanop") Then
                Dim bb As Autodesk.Aec.Modeler.Body = Autodesk.Aec.Modeler.Body.Cylinder(New LineSegment3d(New Point3d(0, 0, 0), New Point3d(10, 0, 0)), 5, 20)
                b2 = b.BooleanOperation(bb, BooleanOperatorType.Intersect)
                met.SetBody(b2, False)

            ElseIf (cmdString.ToLower() = "combinebody") Then
                Dim bb As Autodesk.Aec.Modeler.Body = Autodesk.Aec.Modeler.Body.Box(New Point3d(0, 0, 0), New Vector3d(11, 11, 11))
                b2 = bb.Combine(b)
                met.SetBody(b2, False)

            ElseIf (cmdString.ToLower() = "translatebody") Then
                b2 = b.Translate(New Vector3d(1, 1, 1))
                met.SetBody(b2, False)

            ElseIf (cmdString.ToLower() = "rotatebody") Then
                b2 = b.Rotate(New LineSegment3d(New Point3d(0, 0, 0), New Point3d(1, 1, 1)), 30)
                met.SetBody(b2, False)

            ElseIf (cmdString.ToLower() = "scalebody") Then
                b2 = b.Scale(New Point3d(0, 0, 0), 2)
                met.SetBody(b2, True)

            ElseIf (cmdString.ToLower() = "listpickbody") Then
                Dim ms() As MassElement = GetSelectedMassElement()
                For Each m As MassElement In ms
                    If (Nothing <> m) Then
                        ListBody(m)
                    End If
                Next m

            ElseIf (cmdString.ToLower() = "quit") Then
                result = False
            Else
                Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("Wrong command!" & vbCrLf)
            End If
        Next id

        t.Commit()
        t.Dispose()
        Return result
    End Function

    Public Sub Init()
        Dim db As Database = HostApplicationServices.WorkingDatabase
        Dim tm As Autodesk.AutoCAD.DatabaseServices.TransactionManager = db.TransactionManager
        Dim t As Transaction = tm.StartTransaction()
        Dim bt As BlockTable = tm.GetObject(db.BlockTableId, OpenMode.ForRead, False)
        Dim btr As BlockTableRecord = tm.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForWrite, False)
        Dim temp As Autodesk.Aec.DatabaseServices.MassElement = Autodesk.Aec.DatabaseServices.MassElement.Create(ShapeType.BoundaryRepresentation)
        Dim body As Autodesk.Aec.Modeler.Body = Autodesk.Aec.Modeler.Body.Cone(New LineSegment3d(New Point3d(0, 0, 0), New Point3d(0, 0, 10)), 3, 3, 10)
        temp.SetBody(body, False)
        btr.AppendEntity(temp)
        tm.AddNewlyCreatedDBObject(temp, True)
        t.Commit()
        t.Dispose()
    End Sub

    Private Sub ZoomSample()
        Try
            Application.DocumentManager.MdiActiveDocument.Editor.Document.SendStringToExecute("zoom" & vbCr, False, False, True)
            Application.DocumentManager.MdiActiveDocument.Editor.Document.SendStringToExecute("e" & vbCr, False, False, True)
            Application.DocumentManager.MdiActiveDocument.Editor.Document.SendStringToExecute("-view" & vbCr, False, False, True)
            Application.DocumentManager.MdiActiveDocument.Editor.Document.SendStringToExecute("_seiso" & vbCr, False, False, True)
        Catch
        End Try
    End Sub

    Private Sub ListBody(ByVal met As MassElement)

        Dim b As Autodesk.Aec.Modeler.Body = met.Body
        Dim faceNumber As Int32 = b.Faces.Count
        Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("There are " & faceNumber & " faces." & vbCrLf)
        Dim index As Int32 = 0

        For Each f As Autodesk.Aec.Modeler.Face In b.Faces
            Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("There are " & f.Edges.Count & " edges in face" & index & vbCrLf)
            index = index + 1
        Next f

        Dim surfaceNumber As Int32 = b.Surfaces.Count
        Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("There are " & surfaceNumber & " surfaces." & vbCrLf)
        index = 0
        For Each s As Autodesk.Aec.Modeler.Surface In b.Surfaces
            Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("Surface " & index & " is a " & s.Type & vbCrLf)
            index = index + 1
        Next s

        Dim curveNumber As Int32 = b.Curves.Count
        Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("There are " & curveNumber & " curves." & vbCrLf)
        index = 0
        For Each c As Autodesk.Aec.Modeler.Curve In b.Curves
            Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("Curve " & index & " is a " & c.Type & vbCrLf)
            index = index + 1
        Next c

        Dim vertexNumber As Int32 = b.Vertices.Count
        Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("There are " & vertexNumber & " vertices." & vbCrLf)
        index = 0
        For Each v As Autodesk.Aec.Modeler.Vertex In b.Vertices
            Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("Vertices " & index & " : " & v.Point.ToString() & vbCrLf)
            index = index + 1
        Next v


        Dim vMin As Autodesk.Aec.Modeler.Vertex = b.GetVertexMin(New Vector3d(0, 0, 1))
        Dim vMax As Autodesk.Aec.Modeler.Vertex = b.GetVertexMax(New Vector3d(0, 0, 1))
        Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("The Lowest Vertex is " & vMin.Point.ToString() & vbCrLf)
        Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("The Hightest Vertex is " & vMax.Point.ToString() & vbCrLf)
    End Sub

End Class




