#Region " Autodesk Copyright"
'      .NET Sample
'
'      Copyright (c) 2008 by Autodesk, Inc.
'
'      Permission to use, copy, modify, and distribute this software
'      for any purpose and without fee is hereby granted, provided
'      that the above copyright notice appears in all copies and
'      that both that copyright notice and the limited warranty and
'      restricted rights notice below appear in all supporting
'      documentation.
'
'      AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
'      AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
'      MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
'      DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
'      UNINTERRUPTED OR ERROR FREE.
'
'      Use, duplication, or disclosure by the U.S. Government is subject to
'      restrictions set forth in FAR 52.227-19 (Commercial Computer
'      Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
'      (Rights in Technical Data and Computer Software), as applicable.
'
#End Region
#Region "Namespaces References"

Imports System
Imports System.Collections.Generic
Imports System.Text
Imports System.Diagnostics
Imports System.Windows.Forms
Imports Microsoft.VisualBasic.Constants

Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.Geometry
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.EditorInput
Imports Autodesk.AutoCAD.GraphicsSystem
Imports Autodesk.Aec.DatabaseServices
Imports Autodesk.Aec.Building.ApplicationServices
Imports Autodesk.Aec.Building.DatabaseServices
Imports Autodesk.Aec.Building.Hvac.DatabaseServices
Imports Autodesk.Aec.Building.Piping.DatabaseServices
Imports Autodesk.Aec.Building.Hvac.ApplicationServices
Imports Autodesk.Aec.Building.Piping.ApplicationServices

'Create Aliases
'
Imports AcadApplication = Autodesk.AutoCAD.ApplicationServices.Application
Imports Entity = Autodesk.AutoCAD.DatabaseServices.Entity
Imports ObjectId = Autodesk.AutoCAD.DatabaseServices.ObjectId
Imports DictionaryProfileDefinition = Autodesk.Aec.Building.DatabaseServices.DictionaryProfileDefinition
Imports ObjectIdCollection = Autodesk.AutoCAD.DatabaseServices.ObjectIdCollection
Imports Type = Autodesk.Aec.Building.DatabaseServices.Type
Imports Utility = Autodesk.Aec.Building.DatabaseServices.Utility

#End Region

<Assembly: Autodesk.AutoCAD.Runtime.CommandClass(GetType(AddFlexSegments))> 
<Assembly: Autodesk.AutoCAD.Runtime.ExtensionApplication(GetType(AddFlexSegments))> 

Public Class AddFlexSegments
    Implements IExtensionApplication

#Region "Member Data"

    Private mEditor As Editor
    Private mDb As Database
    Private mDoc As Document
    Private mTransMgr As Autodesk.AutoCAD.DatabaseServices.TransactionManager

#End Region

#Region "IExtensionApplication"

    Public Sub Initialize() Implements IExtensionApplication.Initialize
        Dim ed As Editor = AcadApplication.DocumentManager.MdiActiveDocument.Editor
        ed.WriteMessage("Loading AddFlexSegments..." & vbCrLf)
        ed.WriteMessage("Type 'FlexDuctAdd' or 'FlexPipeAdd' to run the sample..." & vbCrLf)
    End Sub

    Public Sub Terminate() Implements IExtensionApplication.Terminate
    End Sub

#End Region

#Region "CmdAddFlexDuct"

    <Autodesk.AutoCAD.Runtime.CommandMethod("FlexSamples", "FlexDuctAdd", Autodesk.AutoCAD.Runtime.CommandFlags.Modal)> _
    Public Sub CmdAddFlexDuct()

        ' initialize variables
        '
        Setup()

        ' Create transaction
        '
        Dim trans As Transaction = mTransMgr.StartTransaction

        Try

            ' create a new flexSegment object
            '
            Dim flexSegment As DuctFlex = New DuctFlex

            ' make sure to call these before setting your geometry on the part
            '
            flexSegment.SetToStandard(mDb)
            flexSegment.SetDatabaseDefaults(mDb)

            ' Query parameter.
            '
            Dim domain As Domain = domain.DuctComponent
            Dim connectionType As ConnectionType = connectionType.Slip
            Dim diameter As Double = 200
            If IsImperial(trans) Then diameter = 12.0

            ' Now create the part
            '
            If CreatePartData(flexSegment, Context.ConnectionSizeDiameter, diameter, domain, connectionType) Then

                SetGeometry(flexSegment)
                SetConnectors(flexSegment, connectionType)
                SetFlexDuctGraphicPatterns(flexSegment)
                SetDuctSystem(trans, flexSegment, "Supply")
                AddToDatabase(trans, flexSegment)

            End If

        Catch ex As System.Exception
            mEditor.WriteMessage(ex.Message)

        Finally

            CType(trans, IDisposable).Dispose()
            CType(trans, IDisposable).Dispose()

        End Try
    End Sub

#End Region

#Region "CmdAddFlexPipe"

    <Autodesk.AutoCAD.Runtime.CommandMethod("FlexAdd", "FlexPipeAdd", Autodesk.AutoCAD.Runtime.CommandFlags.Modal)> _
    Public Sub CmdAddFlexPipe()
      
        Setup()

        ' Using 
        Dim trans As Transaction = mTransMgr.StartTransaction

        Try
            ' create a new flexSegment object
            '
            Dim flexSegment As PipeFlex = New PipeFlex

            ' make sure to call these before setting your geometry on the part
            '
            flexSegment.SetToStandard(mDb)
            flexSegment.SetDatabaseDefaults(mDb)

            ' Query parameter.
            '
            Dim domain As Domain = domain.PipeComponent
            Dim connectionType As ConnectionType = connectionType.UndefinedConnectionType
            Dim diameter As Double = 50
            If IsImperial(trans) Then diameter = 4.0

            ' Now create the part
            '
            If CreatePartData(flexSegment, Context.ConnectionSizeNominalDiameter, diameter, domain, connectionType) Then

                SetGeometry(flexSegment)
                SetConnectors(flexSegment, connectionType)
                SetFlexPipeGraphicPatterns(flexSegment)
                SetPipeSystem(trans, flexSegment, "Steam")
                AddToDatabase(trans, flexSegment)

            End If

        Catch ex As System.Exception
            mEditor.WriteMessage(ex.Message)

        Finally
            CType(trans, IDisposable).Dispose()
        End Try

    End Sub

#End Region


#Region "Setup"

    Protected Sub Setup()
        mEditor = AcadApplication.DocumentManager.MdiActiveDocument.Editor
        mDb = AcadApplication.DocumentManager.MdiActiveDocument.Database
        mDoc = AcadApplication.DocumentManager.MdiActiveDocument
        mTransMgr = mDb.TransactionManager
    End Sub

#End Region

#Region "IsImperial"

    Protected Function IsImperial(ByVal trans As Transaction) As Boolean

        Dim id As ObjectId = Autodesk.Aec.ApplicationServices.DrawingSetupVariables.GetInstance(mDb, True)
        Dim obj As Object = trans.GetObject(id, OpenMode.ForRead)
        Dim vars As Autodesk.Aec.ApplicationServices.DrawingSetupVariables = TryCast(obj, Autodesk.Aec.ApplicationServices.DrawingSetupVariables)

        If vars Is Nothing Then Return True

        Dim unitType As Autodesk.Aec.DatabaseServices.UnitType = New Autodesk.Aec.DatabaseServices.UnitType(vars.LinearUnit, 1)
        Return unitType.IsImperial

    End Function

#End Region

#Region "AddToDatabase"

    Private Sub AddToDatabase(ByVal trans As Transaction, ByVal flexSegment As MultiSegment)
        ' Use the system related layer.
        '
        flexSegment.SetDefaultLayer()

        ' add the flexSegment to the database
        '
        Dim bt As BlockTable = CType(mTransMgr.GetObject(mDb.BlockTableId, OpenMode.ForRead, False), BlockTable)
        Dim btr As BlockTableRecord = CType(mTransMgr.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForWrite, False), BlockTableRecord)
        btr.AppendEntity(flexSegment)
        mTransMgr.AddNewlyCreatedDBObject(flexSegment, True)
        trans.Commit()
    End Sub

#End Region

#Region "SetGeometry"

    Sub SetGeometry(ByVal flexSegment As MultiSegment)

        Dim pt1 As Point3d
        Dim pt2 As Point3d
        Dim pt3 As Point3d

        'Prompt for the 3 points to create 2 segments
        '
        Dim bPointAcquired As Boolean = False
        Dim ppr As PromptPointResult
        Dim op As PromptPointOptions = New PromptPointOptions("Select Start Point: ")

        Do Until bPointAcquired = True
            ppr = mEditor.GetPoint(op)

            If (ppr.Status.Equals(PromptStatus.Cancel)) Then Throw (New System.InvalidOperationException("User abort."))
            If (ppr.Status.Equals(PromptStatus.OK)) Then
                pt1 = ppr.Value

                Do Until bPointAcquired = True

                    op.Message = "Specify Second Point: "
                    op.UseBasePoint = True
                    op.BasePoint = pt1
                    ppr = mEditor.GetPoint(op)

                    If (ppr.Status.Equals(PromptStatus.Cancel)) Then Throw (New System.InvalidOperationException("User abort."))
                    If (ppr.Status.Equals(PromptStatus.OK)) Then

                        pt2 = ppr.Value
                        mEditor.DrawVector(pt1, pt2, 7, False)

                        Do Until bPointAcquired = True

                            op.Message = "Specify End Point: "
                            op.BasePoint = pt2
                            ppr = mEditor.GetPoint(op)

                            If (ppr.Status.Equals(PromptStatus.Cancel)) Then Throw (New System.InvalidOperationException("User abort."))
                            If (ppr.Status.Equals(PromptStatus.OK)) Then
                                pt3 = ppr.Value

                                bPointAcquired = True
                            End If

                        Loop

                    End If

                Loop

            End If

        Loop

        ' set the line and curve segments onto the flexSegment
        '
        Dim lineSeg As LineSegment3d = New LineSegment3d(pt1, pt2)
        flexSegment.SetBaseCurve(lineSeg, Vector3d.ZAxis, 0)

        ' NOTE:  We need to get the profile id from the style. Note the difference with flex segments vs other
        ' other parts is that we need to set the profile ourselves because there are more than one segment
        '
        Dim style As MultiSegmentStyle = CType(mTransMgr.GetObject(flexSegment.StyleId, OpenMode.ForRead, False), MultiSegmentStyle)

        'Get profileIds from the style
        '
        Dim profileId As ObjectId = style.ProfileId(0)
        Dim profileEndId As ObjectId = style.ProfileEndId(0)

        Dim index As Integer = 0

        'Set Start and End profile on the segment
        '
        flexSegment.SetProfileId(profileId, index)
        flexSegment.SetProfileIdEnd(profileEndId, index)

        index = 1

        'Create a new Line Segment
        '
        Dim lineSeg2 As LineSegment3d = New LineSegment3d(pt2, pt3)
        flexSegment.SetBaseCurve(lineSeg2, Vector3d.ZAxis, index)

        ' notice that we need to set the profile for each segment
        '
        flexSegment.SetProfileId(profileId, index)
        flexSegment.SetProfileIdEnd(profileEndId, index)

        flexSegment.UpdateCutPlanesAndNodes()
    End Sub

#End Region

#Region "SetFlexGraphicPatterns"

    Private Shared Sub SetFlexDuctGraphicPatterns(ByVal flexSegment As MultiSegment)
        Dim vars As HvacDBVariables = New HvacDBVariables

        ' set our patterns and spacing
        '
        flexSegment.BuiltInGraphicsPattern = vars.FlexDuctSingleLinePattern
        flexSegment.PatternSpacing = vars.FlexDuctSingleLineSpacing
        flexSegment.BuiltInGraphicsPattern2Line = vars.FlexDuctDoubleLinePattern
        flexSegment.PatternSpacing2Line = vars.FlexDuctDoubleLineSpacing
    End Sub

    Private Shared Sub SetFlexPipeGraphicPatterns(ByVal flexSegment As MultiSegment)
        Dim vars As PipeDBVariables = New PipeDBVariables

        ' set our patterns and spacing
        '
        flexSegment.BuiltInGraphicsPattern = vars.FlexPipeSingleLinePattern
        flexSegment.PatternSpacing = vars.FlexPipeSingleLineSpacing
        flexSegment.BuiltInGraphicsPattern2Line = vars.FlexPipeDoubleLinePattern
        flexSegment.PatternSpacing2Line = vars.FlexPipeDoubleLineSpacing
    End Sub

#End Region

#Region "SetSystem"

    Private Sub SetPipeSystem(ByVal trans As Transaction, ByVal flexSegment As MultiSegment, ByVal system As String)

        ' Sets the system and style id. This will not work until some segments have been assigned.
        '
        Dim sysDefDict As DictionaryPipeSystemDefinition = New DictionaryPipeSystemDefinition(mDb)

        If (sysDefDict.Has(system, trans)) Then
            flexSegment.SystemId = sysDefDict.GetAt(system)
        Else
            flexSegment.SystemId = DictionaryPipeSystemDefinition.GetStandardDefinition(mDb)
        End If

    End Sub

    Private Sub SetDuctSystem(ByVal trans As Transaction, ByVal flexSegment As MultiSegment, ByVal system As String)

        ' Sets the system and style id. This will not work until some segments have been assigned.
        '
        Dim sysDefDict As DictionaryDuctSystemDefinition = New DictionaryDuctSystemDefinition(mDb)

        If (sysDefDict.Has(system, trans)) Then
            flexSegment.SystemId = sysDefDict.GetAt(system)
        Else
            flexSegment.SystemId = DictionaryDuctSystemDefinition.GetStandardDefinition(mDb)
        End If

    End Sub

#End Region

#Region "SetConnectors"

    Private Sub SetConnectors(ByVal flexSegment As MultiSegment, ByVal connectionType As ConnectionType)

        ' Usually the PartManager setup the connector styles. Because the segments were not setup yet. It wasn't done.
        ' So we do it now.
        ' 
        Dim connectorStyleId As ObjectId = DictionaryConnectorStyle.GetHvacConnectorStyle(mDb)

        flexSegment.SetConnectionComponentStyle(connectorStyleId, flexSegment.Domain)

        ' The connection type is defined in the catalog data. But it wasn't aligned yet for the same reason.
        '
        For Each compMember As ConnectionComponentMember In flexSegment.ConnectionComponentMembers
            compMember.ConnectionType = connectionType
        Next

    End Sub

#End Region

#Region "CreatePartData"

    Private Function CreatePartData(ByVal flexSegment As MultiSegment, ByVal sizeContext As Context, ByVal diameter As Double, ByVal domain As Domain, ByVal connectionType As ConnectionType) As Boolean

        Dim returnValue As Boolean = True

        Try
            ' setup our query
            '
            Dim query As DataQuery = New DataQuery
            query.Domain = domain

            If domain = domain.DuctComponent Then
                query.Type = Type.DuctFlex
            Else
                query.Type = Type.PipeFlex
            End If

            ' We only want real sized parts.
            '
            query.PlaceholderFlag = QueryFlagState.FindOnlyFalse

            ' Look in the catalog for an existing flex part.
            '
            Dim catMgr As CatalogManager = New CatalogManager()
            Dim partsList As DataPartsList = catMgr.GetPartsList(query)

            If partsList.DataParts.Count > 0 Then query.PartGuid = partsList.DataParts(0).Guid

            query.AddSizeParameter(sizeContext, 1, diameter)

            Dim strValue As String = Utility.GetGlobalString(connectionType)
            query.AddSizeParameter(Context.ConnectionPortPhysicalType, 1, strValue)

            ' set the maximum number of records we would like to see
            '
            Dim maxRecords As Integer = 10

            ' Get the ExpandedTable from our query.
            ' NOTE: it will fail if:
            ' - we don't have any GUID.
            ' - diameter is not found.
            ' - connection type is not found.
            '
            Dim exTable As DataExpandedTable = PartManager.GetPartTable(query, maxRecords)

            If Not (exTable Is Nothing) Then

                ' get all the records from the table
                '
                Dim recordCol As DataRecordCollection = exTable.DataRecords

                If recordCol.Count > 0 Then
                    ' get the first record from the collection
                    '
                    Dim record As DataRecord = recordCol(0)

                    ' NOTE: for non MultiSegment objects, this also sets the profile 
                    ' on the object, we will need to do this ourselves. See the 
                    ' SetGeometry method
                    '
                    PartManager.CreatePartViaRecord(exTable, record, flexSegment)
                Else
                    mEditor.WriteMessage("" & Microsoft.VisualBasic.Chr(10) & "No records found.")
                    returnValue = False
                End If
            End If

        Catch be As Autodesk.Aec.Building.DatabaseServices.BuildingException
            If be.ErrorCode = ErrorCode.QueryResultNull Then
                MessageBox.Show("The query did not find any parts.")
            End If
            returnValue = False

        Catch ex As System.Exception
            returnValue = False
            Debug.Write(String.Format("" & Microsoft.VisualBasic.Chr(10) & "Caught the following Exception:{0}", ex.ToString))

        End Try

        Return returnValue

    End Function

#End Region

End Class

