#Region "Header"
'      .NET Sample
'
'      Copyright (c) 2006 by Autodesk, Inc.
'
'      Permission to use, copy, modify, and distribute this software
'      for any purpose and without fee is hereby granted, provided
'      that the above copyright notice appears in all copies and
'      that both that copyright notice and the limited warranty and
'      restricted rights notice below appear in all supporting
'      documentation.
'
'      AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
'      AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
'      MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
'      DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
'      UNINTERRUPTED OR ERROR FREE.
'
'      Use, duplication, or disclosure by the U.S. Government is subject to
'      restrictions set forth in FAR 52.227-19 (Commercial Computer
'      Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
'      (Rights in Technical Data and Computer Software), as applicable.
#End Region

#Region "Namespaces"

Imports System

Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.EditorInput
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.Geometry

Imports Autodesk.Aec.DatabaseServices
Imports Autodesk.Aec.Arch.DatabaseServices
Imports Autodesk.Aec.Structural.DatabaseServices
Imports Autodesk.Aec.Geometry

Imports TransactionManager = Autodesk.AutoCAD.DatabaseServices.TransactionManager
Imports DBObject = Autodesk.AutoCAD.DatabaseServices.DBObject
Imports Entity = Autodesk.AutoCAD.DatabaseServices.Entity
Imports ObjectId = Autodesk.AutoCAD.DatabaseServices.ObjectId
Imports ObjectIdCollection = Autodesk.AutoCAD.DatabaseServices.ObjectIdCollection

#End Region

<Assembly: Autodesk.AutoCAD.Runtime.ExtensionApplication(GetType(Class1))> 
<Assembly: Autodesk.AutoCAD.Runtime.CommandClass(GetType(Class1))> 

Public Class Class1
    Implements IExtensionApplication

#Region "IExtensionApplication"

    Public Sub Initialize() Implements IExtensionApplication.Initialize
        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor
        ed.WriteMessage("Loading AecHardwiredStylesMgd..." + vbCrLf)
    End Sub

    Public Sub Terminate() Implements IExtensionApplication.Terminate
    End Sub

#End Region

#Region "CommandMethod"
    <Autodesk.AutoCAD.Runtime.CommandMethod("AecHardwiredStylesMgd", "Hardwired", Autodesk.AutoCAD.Runtime.CommandFlags.Modal)> _
    Public Sub Hardwired()

        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

        Dim options As PromptKeywordOptions = New PromptKeywordOptions("Option [Door/Member/Roofslab/SLab/STair/WAll/WIndow/All]", _
                                                                       "Door Member Roofslab SLab STair WAll WIndow All")
        Dim res1 As PromptResult = ed.GetKeywords(options)
        Dim opt As String = res1.StringResult

        If (opt = "Door" Or opt = "All") Then
            CreateDoorStyles()
        End If
        If (opt = "Member" Or opt = "All") Then
            CreateMemberStyles()
        End If
        If (opt = "Roofslab" Or opt = "All") Then
            CreateRoofSlabStyles()
        End If
        If (opt = "SLab" Or opt = "All") Then
            CreateSlabStyles()
        End If
        If (opt = "STair" Or opt = "All") Then
            CreateStairStyles()
        End If
        If (opt = "WAll" Or opt = "All") Then
            CreateWallStyles()
        End If
        If (opt = "WIndow" Or opt = "All") Then
            CreateWindowStyles()
        End If

    End Sub
#End Region

#Region "CreateDoorStyles"
    Private Sub CreateDoorStyles()

        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

        Dim db As Database = HostApplicationServices.WorkingDatabase
        Dim tm As TransactionManager = db.TransactionManager
        Dim trans As Transaction = tm.StartTransaction()

        Dim dict As DictionaryDoorStyle = New DictionaryDoorStyle(db)

        Try
            Dim stylename As String = "InteriorDouble"
            Dim description As String = "Interior Double, 2" + Chr(34) & " Thick, Wood As String"

            If Not (dict.Has(stylename, trans)) Then
                ed.WriteMessage("Creating " + description + vbCrLf)

                Dim style As DoorStyle = New DoorStyle()
                style.Description = description

                style.StopWidth = 2.0
                style.StopDepth = 2.5
                style.FrameWidth = 2.0
                style.FrameDepth = 7.0
                style.DoorThickness = 2.0
                style.DoorType = DoorType.Double
                style.DoorShape = OpenShapeType.Rectangular

                dict.AddNewRecord(stylename, style)
                tm.AddNewlyCreatedDBObject(style, True)

                Dim list As StandardSizeOpeningList = New StandardSizeOpeningList()
                Dim opening As StandardSizeOpening = New StandardSizeOpening()
                opening.Set(32.0, 80.0, 0.0, 0.0)
                list.Openings.Add(opening)

                opening = New StandardSizeOpening()
                opening.Set(34.0, 80.0, 0.0, 0.0)
                list.Openings.Add(opening)

                opening = New StandardSizeOpening()
                opening.Set(36.0, 80.0, 0.0, 0.0)
                list.Openings.Add(opening)

                opening = New StandardSizeOpening()
                opening.Set(32.0, 84.0, 0.0, 0.0)
                list.Openings.Add(opening)

                opening = New StandardSizeOpening()
                opening.Set(34.0, 84.0, 0.0, 0.0)
                list.Openings.Add(opening)

                opening = New StandardSizeOpening()
                opening.Set(36.0, 84.0, 0.0, 0.0)
                list.Openings.Add(opening)

                opening = New StandardSizeOpening()
                opening.Set(32.0, 88.0, 0.0, 0.0)
                list.Openings.Add(opening)

                opening = New StandardSizeOpening()
                opening.Set(34.0, 88.0, 0.0, 0.0)
                list.Openings.Add(opening)

                opening = New StandardSizeOpening()
                opening.Set(36.0, 88.0, 0.0, 0.0)
                list.Openings.Add(opening)

                opening = New StandardSizeOpening()
                opening.Set(42.0, 80.0, 0.0, 0.0)
                list.Openings.Add(opening)

                opening = New StandardSizeOpening()
                opening.Set(50.0, 88.0, 0.0, 0.0)
                list.Openings.Add(opening)

                ' create the extension dictionary
                style.CreateExtensionDictionary()
                Dim extDictId As ObjectId = style.ExtensionDictionary
                Dim obj As DBObject = tm.GetObject(extDictId, OpenMode.ForWrite)
                Dim dbDict As DBDictionary = obj

                Dim standardSizesDict As DBDictionary = New DBDictionary()
                dbDict.SetAt(StandardSizeOpeningList.ExtensionDictionaryName, standardSizesDict)
                Dim idStandardSizesDict As ObjectId = dbDict.GetAt(StandardSizeOpeningList.ExtensionDictionaryName)

                tm.AddNewlyCreatedDBObject(standardSizesDict, True)

                ' get the extension dictionary and add the list
                obj = tm.GetObject(idStandardSizesDict, OpenMode.ForWrite)
                dbDict = obj
                dbDict.SetAt(StandardSizeOpeningList.ExtensionDictionaryKeyName, list)

                tm.AddNewlyCreatedDBObject(list, True)
            End If

            stylename = "ClosetBifold"
            description = "Closet Bifold, 2" + Chr(34) & " Thick, Masonite Door Style"

            If Not (dict.Has(stylename, trans)) Then
                ed.WriteMessage("Creating " + description + vbCrLf)

                Dim style As DoorStyle = New DoorStyle()
                style.Description = description

                style.FrameWidth = 3.0
                style.FrameDepth = 7.0
                style.DoorThickness = 2.0
                style.DoorType = DoorType.Bifold
                style.DoorShape = OpenShapeType.Rectangular

                dict.AddNewRecord(stylename, style)
                tm.AddNewlyCreatedDBObject(style, True)

                Dim list As StandardSizeOpeningList = New StandardSizeOpeningList()
                Dim opening As StandardSizeOpening = New StandardSizeOpening()
                opening.Set(28.0, 80.0, 0.0, 0.0)
                list.Openings.Add(opening)

                opening = New StandardSizeOpening()
                opening.Set(36.0, 80.0, 0.0, 0.0)
                list.Openings.Add(opening)

                opening = New StandardSizeOpening()
                opening.Set(48.0, 84.0, 0.0, 0.0)
                list.Openings.Add(opening)

                ' create the extension dictionary
                style.CreateExtensionDictionary()
                Dim extDictId As ObjectId = style.ExtensionDictionary
                Dim obj As DBObject = tm.GetObject(extDictId, OpenMode.ForWrite)
                Dim dbDict As DBDictionary = obj

                Dim standardSizesDict As DBDictionary = New DBDictionary()
                dbDict.SetAt(StandardSizeOpeningList.ExtensionDictionaryName, standardSizesDict)
                Dim idStandardSizesDict As ObjectId = dbDict.GetAt(StandardSizeOpeningList.ExtensionDictionaryName)

                tm.AddNewlyCreatedDBObject(standardSizesDict, True)

                ' get the extension dictionary and add the list
                obj = tm.GetObject(idStandardSizesDict, OpenMode.ForWrite)
                dbDict = obj
                dbDict.SetAt(StandardSizeOpeningList.ExtensionDictionaryKeyName, list)

                tm.AddNewlyCreatedDBObject(list, True)
            End If

            trans.Commit()
        Catch
            trans.Abort()
        Finally
            trans.Dispose()
        End Try

    End Sub
#End Region

#Region "CreateMemberStyles"
    Private Sub CreateMemberStyles()
        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

        Dim db As Database = HostApplicationServices.WorkingDatabase
        Dim tm As TransactionManager = db.TransactionManager
        Dim trans As Transaction = tm.StartTransaction()

        Dim dict As DictionaryMemberStyle = New DictionaryMemberStyle(db)
        Dim shapeDict As DictionaryMemberNodeShape = New DictionaryMemberNodeShape(db)

        Try
            ' create the shape definitions

            Dim shapename As String = "W12x20"
            Dim shapeIdW12x20 As ObjectId = ObjectId.Null

            If shapeDict.Has(shapename, trans) Then
                shapeIdW12x20 = shapeDict.GetAt(shapename)
            Else
                Dim shapeDef As MemberNodeShape = New MemberNodeShape()
                Dim i As Integer
                For i = 0 To 3 - 1 Step i + 1
                    Dim curves As Curve2d() = Nothing
                    Dim profile As Profile = Nothing
                    IShapeForDesign(i, curves, profile, 12, 0.1, 0.1, 4)

                    If Not curves Is Nothing Then
                        Dim j As Integer
                        For j = 0 To curves.Length - 1 Step j + 1
                            Dim curve As Curve2d = curves(j)
                            shapeDef.AddCurve(i, curve)
                        Next
                    End If

                    If Not profile Is Nothing Then
                        shapeDef.SetProfile(i, profile)
                    End If
                Next
                shapeDict.AddNewRecord(shapename, shapeDef)
                tm.AddNewlyCreatedDBObject(shapeDef, True)
                shapeIdW12x20 = shapeDef.ObjectId
            End If

            shapename = "W36x100"
            Dim shapeIdW36x100 As ObjectId = ObjectId.Null

            If shapeDict.Has(shapename, trans) Then
                shapeIdW36x100 = shapeDict.GetAt(shapename)
            Else
                Dim shapeDef As MemberNodeShape = New MemberNodeShape()
                Dim i As Integer
                For i = 0 To 3 - 1 Step i + 1
                    Dim curves As Curve2d() = Nothing
                    Dim profile As Profile = Nothing
                    IShapeForDesign(i, curves, profile, 36, 0.1, 0.1, 12)

                    If Not curves Is Nothing Then
                        Dim j As Integer
                        For j = 0 To curves.Length - 1 Step j + 1
                            Dim curve As Curve2d = curves(j)
                            shapeDef.AddCurve(i, curve)
                        Next
                    End If

                    If Not profile Is Nothing Then
                        shapeDef.SetProfile(i, profile)
                    End If
                Next
                shapeDict.AddNewRecord(shapename, shapeDef)
                tm.AddNewlyCreatedDBObject(shapeDef, True)
                shapeIdW36x100 = shapeDef.ObjectId
            End If

            shapename = "W36x210"
            Dim shapeIdW36x210 As ObjectId = ObjectId.Null

            If shapeDict.Has(shapename, trans) Then
                shapeIdW36x100 = shapeDict.GetAt(shapename)
            Else
                Dim shapeDef As MemberNodeShape = New MemberNodeShape()
                Dim i As Integer
                For i = 0 To 3 - 1 Step i + 1
                    Dim curves As Curve2d() = Nothing
                    Dim profile As Profile = Nothing
                    IShapeForDesign(i, curves, profile, 36.75, 0.8125, 1.375, 12.125)

                    If Not curves Is Nothing Then
                        Dim j As Integer
                        For j = 0 To curves.Length - 1 Step j + 1
                            Dim curve As Curve2d = curves(j)
                            shapeDef.AddCurve(i, curve)
                        Next
                    End If

                    If Not profile Is Nothing Then
                        shapeDef.SetProfile(i, profile)
                    End If
                Next
                shapeDict.AddNewRecord(shapename, shapeDef)
                tm.AddNewlyCreatedDBObject(shapeDef, True)
                shapeIdW36x210 = shapeDef.ObjectId
            End If

            shapename = "W6x15"
            Dim shapeIdW6x15 As ObjectId = ObjectId.Null

            If shapeDict.Has(shapename, trans) Then
                shapeIdW6x15 = shapeDict.GetAt(shapename)
            Else
                Dim shapeDef As MemberNodeShape = New MemberNodeShape()
                Dim i As Integer
                For i = 0 To 3 - 1 Step i + 1
                    Dim curves As Curve2d() = Nothing
                    Dim profile As Profile = Nothing
                    IShapeForDesign(i, curves, profile, 6, 0.25, 0.25, 6)

                    If Not curves Is Nothing Then
                        Dim j As Integer
                        For j = 0 To curves.Length - 1 Step j + 1
                            Dim curve As Curve2d = curves(j)
                            shapeDef.AddCurve(i, curve)
                        Next
                    End If

                    If Not profile Is Nothing Then
                        shapeDef.SetProfile(i, profile)
                    End If
                Next
                shapeDict.AddNewRecord(shapename, shapeDef)
                tm.AddNewlyCreatedDBObject(shapeDef, True)
                shapeIdW6x15 = shapeDef.ObjectId
            End If

            shapename = "WT12x31"
            Dim shapeIdWT12x31 As ObjectId = ObjectId.Null

            If shapeDict.Has(shapename, trans) Then
                shapeIdWT12x31 = shapeDict.GetAt(shapename)
            Else
                Dim shapeDef As MemberNodeShape = New MemberNodeShape()
                Dim i As Integer
                For i = 0 To 3 - 1 Step i + 1
                    Dim curves As Curve2d() = Nothing
                    Dim profile As Profile = Nothing
                    TShapeForDesign(i, curves, profile, 11.87, 0.43, 0.59, 7.04)

                    If Not curves Is Nothing Then
                        Dim j As Integer
                        For j = 0 To curves.Length - 1 Step j + 1
                            Dim curve As Curve2d = curves(j)
                            shapeDef.AddCurve(i, curve)
                        Next
                    End If

                    If Not profile Is Nothing Then
                        shapeDef.SetProfile(i, profile)
                    End If
                Next
                shapeDict.AddNewRecord(shapename, shapeDef)
                tm.AddNewlyCreatedDBObject(shapeDef, True)
                shapeIdWT12x31 = shapeDef.ObjectId
            End If

            shapename = "C6x13"
            Dim shapeIdC6x13 As ObjectId = ObjectId.Null

            If shapeDict.Has(shapename, trans) Then
                shapeIdC6x13 = shapeDict.GetAt(shapename)
            Else
                Dim shapeDef As MemberNodeShape = New MemberNodeShape()
                Dim i As Integer
                For i = 0 To 3 - 1 Step i + 1
                    Dim curves As Curve2d() = Nothing
                    Dim profile As Profile = Nothing
                    AmericanStandardChannelsForDesign(i, curves, profile, 6, 0.4375, 0.3125, 2.18)

                    If Not curves Is Nothing Then
                        Dim j As Integer
                        For j = 0 To curves.Length - 1 Step j + 1
                            Dim curve As Curve2d = curves(j)
                            shapeDef.AddCurve(i, curve)
                        Next
                    End If

                    If Not profile Is Nothing Then
                        shapeDef.SetProfile(i, profile)
                    End If
                Next
                shapeDict.AddNewRecord(shapename, shapeDef)
                tm.AddNewlyCreatedDBObject(shapeDef, True)
                shapeIdC6x13 = shapeDef.ObjectId
            End If

            shapename = "ROD2"
            Dim shapeIdROD2 As ObjectId = ObjectId.Null

            If shapeDict.Has(shapename, trans) Then
                shapeIdROD2 = shapeDict.GetAt(shapename)
            Else
                Dim shapeDef As MemberNodeShape = New MemberNodeShape()
                Dim i As Integer
                For i = 0 To 3 - 1 Step i + 1
                    Dim profile As Profile = Autodesk.Aec.Geometry.Profile.CreateCircle(2, New Vector2d(0, 0))
                    shapeDef.SetProfile(i, profile)
                Next
                shapeDict.AddNewRecord(shapename, shapeDef)
                tm.AddNewlyCreatedDBObject(shapeDef, True)
                shapeIdROD2 = shapeDef.ObjectId
            End If

            shapename = "Tube"
            Dim shapeIdTube As ObjectId = ObjectId.Null

            If shapeDict.Has(shapename, trans) Then
                shapeIdTube = shapeDict.GetAt(shapename)
            Else
                Dim shapeDef As MemberNodeShape = New MemberNodeShape()
                Dim i As Integer
                For i = 0 To 3 - 1 Step i + 1
                    Dim profile As Profile = New Profile()
                    Dim circleShape As Profile = Autodesk.Aec.Geometry.Profile.CreateCircle(10.0, New Vector2d(0, 0))
                    profile.AddRingsFrom(circleShape)
                    circleShape = Autodesk.Aec.Geometry.Profile.CreateCircle(9.0, New Vector2d(0, 0))
                    profile.AddRingsFrom(circleShape)
                    profile.Rings(0).IsVoid = False
                    profile.Rings(1).IsVoid = True

                    shapeDef.SetProfile(i, profile)
                Next
                shapeDict.AddNewRecord(shapename, shapeDef)
                tm.AddNewlyCreatedDBObject(shapeDef, True)
                shapeIdTube = shapeDef.ObjectId
            End If

            ' create the member styles

            Dim stylename As String = "Taper"
            Dim description As String = "Tapered Member Style"

            If Not (dict.Has(stylename, trans)) Then
                ed.WriteMessage("Creating " + description + vbCrLf)

                Dim style As MemberStyle = New MemberStyle()
                style.Description = description

                Dim startNode As MemberNodeComponent = New MemberNodeComponent()
                Dim endNode As MemberNodeComponent = New MemberNodeComponent()
                startNode.ShapeId = shapeIdW12x20
                startNode.ReferenceNodeId = New MemberNodeId(0, RelativeEnd.Start)
                startNode.Rotation = Math.PI / 2
                endNode.ShapeId = shapeIdW36x100
                endNode.ReferenceNodeId = New MemberNodeId(1, RelativeEnd.Start)
                endNode.Rotation = Math.PI / 2
                Dim comp As MemberComponent = New MemberComponent()
                comp.StartNode = startNode
                comp.EndNode = endNode
                style.Components.Add(comp)

                dict.AddNewRecord(stylename, style)
                tm.AddNewlyCreatedDBObject(style, True)
            End If

            stylename = "W12x20"
            description = "W12x20 Member Style"

            If Not (dict.Has(stylename, trans)) Then
                ed.WriteMessage("Creating " + description + vbCrLf)

                Dim style As MemberStyle = New MemberStyle()
                style.Description = description

                Dim startNode As MemberNodeComponent = New MemberNodeComponent()
                Dim endNode As MemberNodeComponent = New MemberNodeComponent()
                startNode.ShapeId = shapeIdW12x20
                startNode.ReferenceNodeId = New MemberNodeId(0, RelativeEnd.Start)
                endNode.ShapeId = shapeIdW12x20
                endNode.ReferenceNodeId = New MemberNodeId(0, RelativeEnd.End)
                Dim comp As MemberComponent = New MemberComponent()
                comp.StartNode = startNode
                comp.EndNode = endNode
                style.Components.Add(comp)

                dict.AddNewRecord(stylename, style)
                tm.AddNewlyCreatedDBObject(style, True)
            End If

            stylename = "Rod"
            description = "Rebar Member Style"

            If Not (dict.Has(stylename, trans)) Then
                ed.WriteMessage("Creating " + description + vbCrLf)

                Dim style As MemberStyle = New MemberStyle()
                style.Description = description

                Dim startNode As MemberNodeComponent = New MemberNodeComponent()
                startNode.ReferenceNodeId = New MemberNodeId(0, RelativeEnd.Start)
                startNode.ShapeId = shapeIdROD2
                Dim comp As MemberComponent = New MemberComponent()
                comp.StartNode = startNode
                style.Components.Add(comp)

                dict.AddNewRecord(stylename, style)
                tm.AddNewlyCreatedDBObject(style, True)
            End If

            stylename = "W36x210"
            description = "W36x210 Member Style"

            If Not (dict.Has(stylename, trans)) Then
                ed.WriteMessage("Creating " + description + vbCrLf)

                Dim style As MemberStyle = New MemberStyle()
                style.Description = description

                Dim startNode As MemberNodeComponent = New MemberNodeComponent()
                Dim endNode As MemberNodeComponent = New MemberNodeComponent()
                startNode.ShapeId = shapeIdW36x210
                startNode.ReferenceNodeId = New MemberNodeId(0, RelativeEnd.Start)
                endNode.ShapeId = shapeIdW36x210
                endNode.ReferenceNodeId = New MemberNodeId(0, RelativeEnd.End)
                Dim comp As MemberComponent = New MemberComponent()
                comp.StartNode = startNode
                comp.EndNode = endNode
                style.Components.Add(comp)

                dict.AddNewRecord(stylename, style)
                tm.AddNewlyCreatedDBObject(style, True)
            End If

            stylename = "W6x15"
            description = "W6x15 Member Style"

            If Not (dict.Has(stylename, trans)) Then
                ed.WriteMessage("Creating " + description + vbCrLf)

                Dim style As MemberStyle = New MemberStyle()
                style.Description = description

                Dim startNode As MemberNodeComponent = New MemberNodeComponent()
                Dim endNode As MemberNodeComponent = New MemberNodeComponent()
                startNode.ShapeId = shapeIdW6x15
                startNode.ReferenceNodeId = New MemberNodeId(0, RelativeEnd.Start)
                endNode.ShapeId = shapeIdW6x15
                endNode.ReferenceNodeId = New MemberNodeId(0, RelativeEnd.End)
                Dim comp As MemberComponent = New MemberComponent()
                comp.StartNode = startNode
                comp.EndNode = endNode
                style.Components.Add(comp)

                dict.AddNewRecord(stylename, style)
                tm.AddNewlyCreatedDBObject(style, True)
            End If

            stylename = "WT12x31"
            description = "WT12x31 Member Style"

            If Not (dict.Has(stylename, trans)) Then
                ed.WriteMessage("Creating " + description + vbCrLf)

                Dim style As MemberStyle = New MemberStyle()
                style.Description = description

                Dim startNode As MemberNodeComponent = New MemberNodeComponent()
                Dim endNode As MemberNodeComponent = New MemberNodeComponent()
                startNode.ShapeId = shapeIdWT12x31
                startNode.ReferenceNodeId = New MemberNodeId(0, RelativeEnd.Start)
                endNode.ShapeId = shapeIdWT12x31
                endNode.ReferenceNodeId = New MemberNodeId(0, RelativeEnd.End)
                Dim comp As MemberComponent = New MemberComponent()
                comp.StartNode = startNode
                comp.EndNode = endNode
                style.Components.Add(comp)

                dict.AddNewRecord(stylename, style)
                tm.AddNewlyCreatedDBObject(style, True)
            End If

            stylename = "C6x13"
            description = "C6x13 Member Style"

            If Not (dict.Has(stylename, trans)) Then
                ed.WriteMessage("Creating " + description + vbCrLf)

                Dim style As MemberStyle = New MemberStyle()
                style.Description = description

                Dim startNode As MemberNodeComponent = New MemberNodeComponent()
                Dim endNode As MemberNodeComponent = New MemberNodeComponent()
                startNode.ShapeId = shapeIdC6x13
                startNode.ReferenceNodeId = New MemberNodeId(0, RelativeEnd.Start)
                endNode.ShapeId = shapeIdC6x13
                endNode.ReferenceNodeId = New MemberNodeId(0, RelativeEnd.End)
                Dim comp As MemberComponent = New MemberComponent()
                comp.StartNode = startNode
                comp.EndNode = endNode
                style.Components.Add(comp)

                dict.AddNewRecord(stylename, style)
                tm.AddNewlyCreatedDBObject(style, True)
            End If

            stylename = "Tube10x9"
            description = "Tube Member Style"

            If Not (dict.Has(stylename, trans)) Then
                ed.WriteMessage("Creating " + description + vbCrLf)

                Dim style As MemberStyle = New MemberStyle()
                style.Description = description

                Dim startNode As MemberNodeComponent = New MemberNodeComponent()
                Dim endNode As MemberNodeComponent = New MemberNodeComponent()
                startNode.ShapeId = shapeIdTube
                startNode.ReferenceNodeId = New MemberNodeId(0, RelativeEnd.Start)
                endNode.ShapeId = shapeIdTube
                endNode.ReferenceNodeId = New MemberNodeId(0, RelativeEnd.End)
                Dim comp As MemberComponent = New MemberComponent()
                comp.StartNode = startNode
                comp.EndNode = endNode
                style.Components.Add(comp)

                dict.AddNewRecord(stylename, style)
                tm.AddNewlyCreatedDBObject(style, True)
            End If

            trans.Commit()
        Catch
            trans.Abort()
        Finally
            trans.Dispose()
        End Try
    End Sub
#End Region

#Region "CreateRoofSlabStyles"
    Private Sub CreateRoofSlabStyles()

        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

        Dim db As Database = HostApplicationServices.WorkingDatabase
        Dim tm As TransactionManager = db.TransactionManager
        Dim trans As Transaction = tm.StartTransaction()

        Dim dict As DictionaryRoofSlabStyle = New DictionaryRoofSlabStyle(db)

        Try
            Dim stylename As String = "Flat Roof"
            Dim description As String = "Flat Roof Slab Style"

            If Not (dict.Has(stylename, trans)) Then
                ed.WriteMessage("Creating " + description + vbCrLf)

                Dim style As RoofSlabStyle = New RoofSlabStyle()
                style.Description = description

                dict.AddNewRecord(stylename, style)
                tm.AddNewlyCreatedDBObject(style, True)

                ' a. Gravel Ballast
                Dim comp As SlabStyleComponent = New SlabStyleComponent()
                comp.SubSetDatabaseDefaults(db)
                comp.Name = "Gravel Ballast"
                comp.Position.SetFixedThickness(2.0)
                comp.Position.Offset.BaseValue = 6.0
                comp.Position.Offset.OperatorType = InstanceBasedValueOperatorType.Plus
                comp.Position.Offset.Operand = 0.0
                comp.Position.Offset.UseInstanceValue = True

                style.Components.Add(comp)

                ' b. Roofing membrane
                comp = New SlabStyleComponent()
                comp.SubSetDatabaseDefaults(db)
                comp.Name = "Roofing Membrane"
                comp.Position.SetFixedThickness(0.0)
                comp.Position.Offset.BaseValue = 6.0
                comp.Position.Offset.OperatorType = InstanceBasedValueOperatorType.Plus
                comp.Position.Offset.Operand = 0.0
                comp.Position.Offset.UseInstanceValue = True

                style.Components.Add(comp)

                ' c. Insulation
                comp = New SlabStyleComponent()
                comp.SubSetDatabaseDefaults(db)
                comp.Name = "Rigid Insulation"
                comp.Position.SetFixedOffset(6.0)
                comp.Position.Thickness.BaseValue = 0.0
                comp.Position.Thickness.OperatorType = InstanceBasedValueOperatorType.Plus
                comp.Position.Thickness.Operand = 0.0
                comp.Position.Thickness.UseInstanceValue = True

                style.Components.Add(comp)

                ' d. Concrete
                comp = New SlabStyleComponent()
                comp.SubSetDatabaseDefaults(db)
                comp.Name = "Concrete"
                comp.Position.SetFixedOffset(2.0)
                comp.Position.SetFixedThickness(4.0)

                style.Components.Add(comp)

                ' e. Metal decking
                comp = New SlabStyleComponent()
                comp.SubSetDatabaseDefaults(db)
                comp.Name = "Metal Decking"
                comp.Position.SetFixedOffset(0.0)
                comp.Position.SetFixedThickness(2.0)

                style.Components.Add(comp)
            End If

            stylename = "Wood Frame Floor"
            description = "Sloped Standing Seam Metal Roof Slab Style"

            If Not (dict.Has(stylename, trans)) Then
                ed.WriteMessage("Creating " + description + vbCrLf)

                Dim style As RoofSlabStyle = New RoofSlabStyle()
                style.Description = description

                dict.AddNewRecord(stylename, style)
                tm.AddNewlyCreatedDBObject(style, True)

                ' a. Standing Seam Metal Roof Panel
                Dim comp As SlabStyleComponent = New SlabStyleComponent()
                comp.SubSetDatabaseDefaults(db)
                comp.Name = "Standing Seam Metal Roof Panel"
                comp.Position.SetFixedOffset(6.0)
                comp.Position.SetFixedThickness(2.0)

                style.Components.Add(comp)

                ' b. Insulation
                comp = New SlabStyleComponent()
                comp.SubSetDatabaseDefaults(db)
                comp.Name = "Insulation"
                comp.Position.SetFixedOffset(4.0)
                comp.Position.SetFixedThickness(2.0)

                style.Components.Add(comp)

                'c. Structural Roof Frame
                comp = New SlabStyleComponent()
                comp.SubSetDatabaseDefaults(db)
                comp.Name = "Structural Roof Frame"
                comp.Position.SetFixedOffset(0.0)
                comp.Position.SetFixedThickness(4.0)

                style.Components.Add(comp)
            End If

            trans.Commit()

        Catch
            trans.Abort()
        Finally
            trans.Dispose()
        End Try

    End Sub
#End Region

#Region "CreateSlabStyles"
    Private Sub CreateSlabStyles()

        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

        Dim db As Database = HostApplicationServices.WorkingDatabase
        Dim tm As TransactionManager = db.TransactionManager
        Dim trans As Transaction = tm.StartTransaction()

        Dim dict As DictionarySlabStyle = New DictionarySlabStyle(db)

        Try
            Dim stylename As String = "6in Concrete on Grade"
            Dim description As String = "6in Concrete on Grade Slab Style"

            If Not (dict.Has(stylename, trans)) Then
                ed.WriteMessage("Creating " + description + vbCrLf)

                Dim style As SlabStyle = New SlabStyle()
                style.Description = description

                dict.AddNewRecord(stylename, style)
                tm.AddNewlyCreatedDBObject(style, True)

                ' a. Concrete
                Dim comp As SlabStyleComponent = New SlabStyleComponent()
                comp.SubSetDatabaseDefaults(db)
                comp.Name = "Concrete"
                comp.Position.SetFixedOffset(0.0)
                comp.Position.SetFixedThickness(6.0)
                style.Components.Add(comp)

                ' b. Polythene sheet
                comp = New SlabStyleComponent()
                comp.SubSetDatabaseDefaults(db)
                comp.Name = "Polyethylene Sheet"
                comp.Position.SetFixedOffset(-0.125)
                comp.Position.SetFixedThickness(0.125)
                style.Components.Add(comp)

                ' c. Gravel base
                comp = New SlabStyleComponent()
                comp.SubSetDatabaseDefaults(db)
                comp.Name = "Gravel"
                comp.Position.SetFixedOffset(-4.125)
                comp.Position.SetFixedThickness(4.0)
                style.Components.Add(comp)
            End If

            stylename = "Wood Frame Floor"
            description = "Wood Frame Floor Slab Style"

            If Not (dict.Has(stylename, trans)) Then
                ed.WriteMessage("Creating " + description + vbCrLf)

                Dim style As SlabStyle = New SlabStyle()
                style.Description = description

                dict.AddNewRecord(stylename, style)
                tm.AddNewlyCreatedDBObject(style, True)

                ' a. Finish Floor
                Dim comp As SlabStyleComponent = New SlabStyleComponent()
                comp.SubSetDatabaseDefaults(db)
                comp.Name = "Finish Floor"
                comp.Position.SetFixedThickness(0.5)
                comp.Position.Offset.BaseValue = 3.0 / 4.0
                comp.Position.Offset.OperatorType = InstanceBasedValueOperatorType.Plus
                comp.Position.Offset.Operand = 0.0
                comp.Position.Offset.UseInstanceValue = True
                style.Components.Add(comp)

                ' b. Plywood subfloor
                comp = New SlabStyleComponent()
                comp.SubSetDatabaseDefaults(db)
                comp.Name = "Plywood Subfloor"
                comp.Position.SetFixedThickness(3.0 / 4.0)
                comp.Position.Offset.BaseValue = 0.0
                comp.Position.Offset.OperatorType = InstanceBasedValueOperatorType.Plus
                comp.Position.Offset.Operand = 0.0
                comp.Position.Offset.UseInstanceValue = True
                style.Components.Add(comp)

                ' c. Floor Joists
                comp = New SlabStyleComponent()
                comp.SubSetDatabaseDefaults(db)
                comp.Name = "Floor Joists"
                comp.Position.SetFixedOffset(0.0)
                comp.Position.Thickness.BaseValue = 0.0
                comp.Position.Thickness.OperatorType = InstanceBasedValueOperatorType.Plus
                comp.Position.Thickness.Operand = 0.0
                comp.Position.Thickness.UseInstanceValue = True
                style.Components.Add(comp)
            End If

            trans.Commit()
        Catch
            trans.Abort()
        Finally
            trans.Dispose()
        End Try

    End Sub
#End Region

#Region "CreateStairStyles"
    Private Sub CreateStairStyles()
        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

        Dim db As Database = HostApplicationServices.WorkingDatabase
        Dim tm As TransactionManager = db.TransactionManager
        Dim trans As Transaction = tm.StartTransaction()

        Dim dict As DictionaryStairStyle = New DictionaryStairStyle(db)

        Try
            Dim stylename As String = "Wood1"
            Dim description As String = "Wood (Saddle) Stair Style"

            If Not (dict.Has(stylename, trans)) Then
                ed.WriteMessage("Creating " + description + vbCrLf)

                Dim style As StairStyle = New StairStyle()
                style.SetToStandard(db)
                style.Description = description
                style.Flags = style.Flags Or StairStyleFlags.HideRiser ' true
                style.Nosing = 2.0

                Dim stringer As StairStringerComponentDefinition = New StairStringerComponentDefinition()
                stringer.SubSetDatabaseDefaults(db)
                stringer.Type = StairStringerType.Saddled
                stringer.AnchorType = StairStringerAnchorType.AlignLeft
                stringer.Width = 5.0
                stringer.WaistAtFlight = 7.0
                stringer.WaistAtLanding = 7.0
                stringer.Offset = 2.0
                style.ComponentDefinitions.Add(stringer)

                stringer = New StairStringerComponentDefinition()
                stringer.SubSetDatabaseDefaults(db)
                stringer.Type = StairStringerType.Saddled
                stringer.AnchorType = StairStringerAnchorType.AlignRight
                stringer.Width = 5.0
                stringer.WaistAtFlight = 7.0
                stringer.WaistAtLanding = 7.0
                stringer.Offset = 2.0
                style.ComponentDefinitions.Add(stringer)

                dict.AddNewRecord(stylename, style)
                tm.AddNewlyCreatedDBObject(style, True)
            End If

            stylename = "Wood2"
            description = "Wood (Housed) Stair Style"

            If Not (dict.Has(stylename, trans)) Then
                ed.WriteMessage("Creating " + description + vbCrLf)

                Dim style As StairStyle = New StairStyle()
                style.SetToStandard(db)
                style.Description = description
                style.Flags = style.Flags Or StairStyleFlags.HideRiser ' true

                Dim stringer As StairStringerComponentDefinition = New StairStringerComponentDefinition()
                stringer.SubSetDatabaseDefaults(db)
                stringer.Type = StairStringerType.Housed
                stringer.AnchorType = StairStringerAnchorType.AlignLeft
                stringer.Width = 5.0
                stringer.WaistAtFlight = 7.0
                stringer.WaistAtLanding = 7.0
                stringer.ThicknessTotalAtFlight = 12.0
                stringer.ThicknessTotalAtLanding = 12.0
                style.ComponentDefinitions.Add(stringer)

                stringer = New StairStringerComponentDefinition()
                stringer.SubSetDatabaseDefaults(db)
                stringer.Type = StairStringerType.Housed
                stringer.AnchorType = StairStringerAnchorType.AlignRight
                stringer.Width = 5.0
                stringer.WaistAtFlight = 7.0
                stringer.WaistAtLanding = 7.0
                stringer.ThicknessTotalAtFlight = 12.0
                stringer.ThicknessTotalAtLanding = 12.0
                style.ComponentDefinitions.Add(stringer)

                dict.AddNewRecord(stylename, style)
                tm.AddNewlyCreatedDBObject(style, True)
            End If

            stylename = "Steel1"
            description = "Steel Pan Stair Style"

            If Not (dict.Has(stylename, trans)) Then
                ed.WriteMessage("Creating " + description + vbCrLf)

                Dim style As StairStyle = New StairStyle()
                style.SetToStandard(db)
                style.Description = description
                style.Flags = style.Flags Or StairStyleFlags.HideRiser ' true

                Dim stringer As StairStringerComponentDefinition = New StairStringerComponentDefinition()
                stringer.SubSetDatabaseDefaults(db)
                stringer.Type = StairStringerType.Saddled
                stringer.AnchorType = StairStringerAnchorType.AlignLeft
                stringer.Width = 5.0
                stringer.WaistAtFlight = 6.0
                stringer.WaistAtLanding = 6.0
                style.ComponentDefinitions.Add(stringer)

                stringer = New StairStringerComponentDefinition()
                stringer.SubSetDatabaseDefaults(db)
                stringer.Type = StairStringerType.Saddled
                stringer.AnchorType = StairStringerAnchorType.AlignRight
                stringer.Width = 5.0
                stringer.WaistAtFlight = 6.0
                stringer.WaistAtLanding = 6.0
                style.ComponentDefinitions.Add(stringer)

                dict.AddNewRecord(stylename, style)
                tm.AddNewlyCreatedDBObject(style, True)
            End If

            stylename = "Steel2"
            description = "Steel Stair Style"

            If Not (dict.Has(stylename, trans)) Then
                ed.WriteMessage("Creating " + description + vbCrLf)

                Dim style As StairStyle = New StairStyle()
                style.SetToStandard(db)
                style.Description = description
                style.Flags = style.Flags And Not StairStyleFlags.HideRiser 'false

                Dim stringer As StairStringerComponentDefinition = New StairStringerComponentDefinition()
                stringer.SubSetDatabaseDefaults(db)
                stringer.Type = StairStringerType.Saddled
                stringer.AnchorType = StairStringerAnchorType.AlignLeft
                stringer.Width = 5.0
                stringer.WaistAtFlight = 6.0
                stringer.WaistAtLanding = 6.0
                style.ComponentDefinitions.Add(stringer)

                stringer = New StairStringerComponentDefinition()
                stringer.SubSetDatabaseDefaults(db)
                stringer.Type = StairStringerType.Saddled
                stringer.AnchorType = StairStringerAnchorType.AlignRight
                stringer.Width = 5.0
                stringer.WaistAtFlight = 6.0
                stringer.WaistAtLanding = 6.0
                style.ComponentDefinitions.Add(stringer)

                dict.AddNewRecord(stylename, style)
                tm.AddNewlyCreatedDBObject(style, True)
            End If

            stylename = "Concrete"
            description = "Concrete Slab Stair Style"

            If Not (dict.Has(stylename, trans)) Then
                ed.WriteMessage("Creating " + description + vbCrLf)

                Dim style As StairStyle = New StairStyle()
                style.SetToStandard(db)
                style.Description = description
                style.Flags = style.Flags And Not StairStyleFlags.HideRiser 'false

                Dim stringer As StairStringerComponentDefinition = New StairStringerComponentDefinition()
                stringer.SubSetDatabaseDefaults(db)
                stringer.Type = StairStringerType.Slab
                stringer.AnchorType = StairStringerAnchorType.FullWidth
                stringer.Width = 1.0
                stringer.WaistAtFlight = 7.0
                stringer.WaistAtLanding = 7.0
                style.ComponentDefinitions.Add(stringer)

                dict.AddNewRecord(stylename, style)
                tm.AddNewlyCreatedDBObject(style, True)
            End If

            stylename = "Cantilever"
            description = "Double Cantilever Stair Style"

            If Not (dict.Has(stylename, trans)) Then
                ed.WriteMessage("Creating " + description + vbCrLf)

                Dim style As StairStyle = New StairStyle()
                style.SetToStandard(db)
                style.Description = description
                style.Flags = style.Flags And Not StairStyleFlags.HideRiser 'false

                Dim stringer As StairStringerComponentDefinition = New StairStringerComponentDefinition()
                stringer.SubSetDatabaseDefaults(db)
                stringer.Type = StairStringerType.Saddled
                stringer.AnchorType = StairStringerAnchorType.Center
                stringer.Width = 10.0
                stringer.WaistAtFlight = 8.0
                stringer.WaistAtLanding = 8.0
                style.ComponentDefinitions.Add(stringer)

                dict.AddNewRecord(stylename, style)
                tm.AddNewlyCreatedDBObject(style, True)
            End If

            trans.Commit()
        Catch
            trans.Abort()
        Finally
            trans.Dispose()
        End Try
    End Sub
#End Region

#Region "CreateWallStyles"
    Private Sub CreateWallStyles()

        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

        Dim db As Database = HostApplicationServices.WorkingDatabase
        Dim tm As TransactionManager = db.TransactionManager
        Dim trans As Transaction = tm.StartTransaction()

        Dim dict As DictionaryWallStyle = New DictionaryWallStyle(db)

        Try
            Dim stylename As String = "Cavity"
            Dim description As String = "Cavity Wall Style"

            If Not (dict.Has(stylename, trans)) Then
                ed.WriteMessage("Creating " + description + vbCrLf)

                Dim style As WallStyle = New WallStyle()
                style.Description = description

                dict.AddNewRecord(stylename, style)
                tm.AddNewlyCreatedDBObject(style, True)

                Dim comp As WallStyleComponent = New WallStyleComponent()
                comp.SubSetDatabaseDefaults(db)
                comp.Name = "facer"
                comp.Position.SetFixedEdgeOffset(-6)
                comp.Position.SetFixedWidth(4.0)
                comp.Priority = 1
                style.Components.Add(comp)

                comp = New WallStyleComponent()
                comp.SubSetDatabaseDefaults(db)
                comp.Name = "facel"
                comp.Position.SetFixedEdgeOffset(2)
                comp.Position.SetFixedWidth(4.0)
                comp.Priority = 8
                style.Components.Add(comp)
            End If

            stylename = "Mason"
            description = "Mason Wall Style"

            If Not (dict.Has(stylename, trans)) Then
                ed.WriteMessage("Creating " + description + vbCrLf)

                Dim style As WallStyle = New WallStyle()
                style.Description = description

                dict.AddNewRecord(stylename, style)
                tm.AddNewlyCreatedDBObject(style, True)

                Dim comp As WallStyleComponent = New WallStyleComponent()
                comp.SubSetDatabaseDefaults(db)
                comp.Name = "mainwall"
                comp.Position.SetFixedEdgeOffset(-3)
                comp.Position.SetFixedWidth(3.0)
                comp.Priority = 7
                style.Components.Add(comp)

                comp = New WallStyleComponent()
                comp.SubSetDatabaseDefaults(db)
                comp.Name = "someline"
                comp.Position.SetFixedEdgeOffset(0)
                comp.Position.SetFixedWidth(3.0)
                comp.Priority = 8
                style.Components.Add(comp)
            End If

            stylename = "ExtMason"
            description = "ExtMason Wall Style"

            If Not (dict.Has(stylename, trans)) Then
                ed.WriteMessage("Creating " + description + vbCrLf)

                Dim style As WallStyle = New WallStyle()
                style.Description = description

                dict.AddNewRecord(stylename, style)
                tm.AddNewlyCreatedDBObject(style, True)

                Dim comp As WallStyleComponent = New WallStyleComponent()
                comp.SubSetDatabaseDefaults(db)
                comp.Name = "mainwall"
                comp.Position.SetFixedEdgeOffset(-3)
                comp.Position.SetFixedWidth(3.0)
                comp.Position.StartElevation.ElevationType = WallElevationType.FromWallBottom
                comp.Position.EndElevation.ElevationType = WallElevationType.FromWallTop
                comp.Position.StartElevation.ElevationOffset = 0.0
                comp.Position.EndElevation.ElevationOffset = 0.0
                style.Components.Add(comp)

                comp = New WallStyleComponent()
                comp.SubSetDatabaseDefaults(db)
                comp.Name = "someline"
                comp.Position.SetFixedEdgeOffset(0)
                comp.Position.SetFixedWidth(3.0)
                comp.Position.StartElevation.ElevationType = WallElevationType.FromBaseline
                comp.Position.EndElevation.ElevationType = WallElevationType.FromBaseHeight
                comp.Position.StartElevation.ElevationOffset = 0.0
                comp.Position.EndElevation.ElevationOffset = 0.0
                style.Components.Add(comp)
            End If

            stylename = "Connor"
            description = "Connor Wall Style"

            If Not (dict.Has(stylename, trans)) Then
                ed.WriteMessage("Creating " + description + vbCrLf)

                Dim style As WallStyle = New WallStyle()
                style.Description = description

                dict.AddNewRecord(stylename, style)
                tm.AddNewlyCreatedDBObject(style, True)

                Dim comp As WallStyleComponent = New WallStyleComponent()
                comp.SubSetDatabaseDefaults(db)
                comp.Name = "comp2"
                comp.Position.SetFixedEdgeOffset(-3.5)
                comp.Position.SetFixedWidth(7.0)
                comp.Priority = 3
                comp.Position.StartElevation.ElevationType = WallElevationType.FromWallBottom
                comp.Position.EndElevation.ElevationType = WallElevationType.FromBaseline
                comp.Position.StartElevation.ElevationOffset = 0.0
                comp.Position.EndElevation.ElevationOffset = 36.0
                style.Components.Add(comp)

                comp = New WallStyleComponent()
                comp.SubSetDatabaseDefaults(db)
                comp.Name = "comp1"
                comp.Position.SetFixedEdgeOffset(-2)
                comp.Position.SetFixedWidth(4.0)
                comp.Priority = 3
                comp.Position.StartElevation.ElevationType = WallElevationType.FromBaseline
                comp.Position.EndElevation.ElevationType = WallElevationType.FromWallTop
                comp.Position.StartElevation.ElevationOffset = 36.0
                comp.Position.EndElevation.ElevationOffset = 0.0
                style.Components.Add(comp)
            End If

            stylename = "CementBottom"
            description = "CementBottom Wall Style"

            If Not (dict.Has(stylename, trans)) Then
                ed.WriteMessage("Creating " + description + vbCrLf)

                Dim style As WallStyle = New WallStyle()
                style.Description = description

                dict.AddNewRecord(stylename, style)
                tm.AddNewlyCreatedDBObject(style, True)

                Dim comp As WallStyleComponent = New WallStyleComponent()
                comp.SubSetDatabaseDefaults(db)
                comp.Name = "comp1"
                comp.Position.SetFixedEdgeOffset(-4)
                comp.Position.SetFixedWidth(8.0)
                comp.Position.StartElevation.ElevationType = WallElevationType.FromWallBottom
                comp.Position.EndElevation.ElevationType = WallElevationType.FromBaseline
                comp.Position.StartElevation.ElevationOffset = 0.0
                comp.Position.EndElevation.ElevationOffset = 36.0
                comp.Priority = 1
                style.Components.Add(comp)

                comp = New WallStyleComponent()
                comp.SubSetDatabaseDefaults(db)
                comp.Name = "comp2"
                comp.Position.SetFixedEdgeOffset(-4)
                comp.Position.SetFixedWidth(2.0)
                comp.Position.StartElevation.ElevationType = WallElevationType.FromBaseline
                comp.Position.EndElevation.ElevationType = WallElevationType.FromWallTop
                comp.Position.StartElevation.ElevationOffset = 36.0
                comp.Position.EndElevation.ElevationOffset = 0.0
                comp.Priority = 12
                style.Components.Add(comp)

                comp = New WallStyleComponent()
                comp.SubSetDatabaseDefaults(db)
                comp.Name = "comp3"
                comp.Position.SetFixedEdgeOffset(1)
                comp.Position.SetFixedWidth(3.0)
                comp.Position.StartElevation.ElevationType = WallElevationType.FromBaseline
                comp.Position.EndElevation.ElevationType = WallElevationType.FromBaseHeight
                comp.Position.StartElevation.ElevationOffset = 36.0
                comp.Position.EndElevation.ElevationOffset = 0.0
                comp.Priority = 15
                style.Components.Add(comp)
            End If

            stylename = "SimpleVariable"
            description = "SimpleVariable Wall Style"

            If Not (dict.Has(stylename, trans)) Then
                ed.WriteMessage("Creating " + description + vbCrLf)

                Dim style As WallStyle = New WallStyle()
                style.Description = description

                dict.AddNewRecord(stylename, style)
                tm.AddNewlyCreatedDBObject(style, True)

                Dim comp As WallStyleComponent = New WallStyleComponent()
                comp.SubSetDatabaseDefaults(db)
                comp.Name = "comp2"
                comp.Position.SetFixedEdgeOffset(-3)
                comp.Position.SetFixedWidth(6.0)
                comp.Priority = 3
                comp.Position.InstanceWidth.OperatorType = InstanceBasedValueOperatorType.Plus
                comp.Position.InstanceWidth.Operand = 0.0
                comp.Position.InstanceWidth.UseInstanceValue = True
                style.Components.Add(comp)
            End If

            stylename = "ComplexVariable"
            description = "ComplexVariable Wall Style"

            If Not (dict.Has(stylename, trans)) Then
                ed.WriteMessage("Creating " + description + vbCrLf)

                Dim style As WallStyle = New WallStyle()
                style.Description = description

                dict.AddNewRecord(stylename, style)
                tm.AddNewlyCreatedDBObject(style, True)

                Dim comp As WallStyleComponent = New WallStyleComponent()
                comp.SubSetDatabaseDefaults(db)
                comp.Name = "comp1"
                comp.Position.SetFixedEdgeOffset(-3)
                comp.Position.SetFixedWidth(1.0)
                comp.Priority = 3
                style.Components.Add(comp)

                comp = New WallStyleComponent()
                comp.SubSetDatabaseDefaults(db)
                comp.Name = "comp2"
                comp.Position.SetFixedEdgeOffset(-2)
                comp.Position.SetFixedWidth(0)
                comp.Priority = 4
                comp.Position.InstanceWidth.OperatorType = InstanceBasedValueOperatorType.Plus
                comp.Position.InstanceWidth.Operand = 0.0
                comp.Position.InstanceWidth.UseInstanceValue = True
                style.Components.Add(comp)

                comp = New WallStyleComponent()
                comp.SubSetDatabaseDefaults(db)
                comp.Name = "comp3"
                comp.Position.SetFixedEdgeOffset(-2)
                comp.Position.SetFixedWidth(1.0)
                comp.Priority = 3
                comp.Position.InstanceWidth.OperatorType = InstanceBasedValueOperatorType.Plus
                comp.Position.InstanceWidth.Operand = 0.0
                comp.Position.InstanceWidth.UseInstanceValue = True
                style.Components.Add(comp)
            End If

            trans.Commit()
        Catch
            trans.Abort()
        Finally
            trans.Dispose()
        End Try
    End Sub
#End Region

#Region "CreateWindowStyles"
    Private Sub CreateWindowStyles()
        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

        Dim db As Database = HostApplicationServices.WorkingDatabase
        Dim tm As TransactionManager = db.TransactionManager
        Dim trans As Transaction = tm.StartTransaction()

        Dim dict As DictionaryWindowStyle = New DictionaryWindowStyle(db)

        Try
            Dim stylename As String = "Sophie"
            Dim description As String = "Sophie Window Style"

            If Not (dict.Has(stylename, trans)) Then
                ed.WriteMessage("Creating " + description + vbCrLf)

                Dim style As WindowStyle = New WindowStyle()
                style.Description = description

                style.SashWidth = 1.5
                style.SashDepth = 2.5
                style.FrameWidth = 3.2
                style.FrameDepth = 6.0
                style.WindowType = WindowType.Picture
                style.WindowShape = OpenShapeType.Oval

                dict.AddNewRecord(stylename, style)
                tm.AddNewlyCreatedDBObject(style, True)

                Dim list As StandardSizeOpeningList = New StandardSizeOpeningList()
                Dim opening As StandardSizeOpening = New StandardSizeOpening()
                opening.Set(12.0, 20.0, 0.0, 0.0)
                list.Openings.Add(opening)

                opening = New StandardSizeOpening()
                opening.Set(18.0, 24.0, 0.0, 0.0)
                list.Openings.Add(opening)

                opening = New StandardSizeOpening()
                opening.Set(24.0, 36.0, 0.0, 0.0)
                list.Openings.Add(opening)

                opening = New StandardSizeOpening()
                opening.Set(24.0, 12.0, 0.0, 0.0)
                list.Openings.Add(opening)

                opening = New StandardSizeOpening()
                opening.Set(36.0, 18.0, 0.0, 0.0)
                list.Openings.Add(opening)

                ' create the extension dictionary
                style.CreateExtensionDictionary()
                Dim extDictId As ObjectId = style.ExtensionDictionary
                Dim obj As DBObject = tm.GetObject(extDictId, OpenMode.ForWrite)
                Dim dbDict As DBDictionary = obj

                Dim standardSizesDict As DBDictionary = New DBDictionary()
                dbDict.SetAt(StandardSizeOpeningList.ExtensionDictionaryName, standardSizesDict)
                Dim idStandardSizesDict As ObjectId = dbDict.GetAt(StandardSizeOpeningList.ExtensionDictionaryName)

                tm.AddNewlyCreatedDBObject(standardSizesDict, True)

                ' get the extension dictionary and add the list
                obj = tm.GetObject(idStandardSizesDict, OpenMode.ForWrite)
                dbDict = obj
                dbDict.SetAt(StandardSizeOpeningList.ExtensionDictionaryKeyName, list)

                tm.AddNewlyCreatedDBObject(list, True)
            End If

            stylename = "Kori"
            description = "Kori Window Style"

            If Not (dict.Has(stylename, trans)) Then
                ed.WriteMessage("Creating " + description + vbCrLf)

                Dim style As WindowStyle = New WindowStyle()
                style.Description = description
                style.FrameWidth = 2.0
                style.FrameDepth = 4.0
                style.WindowType = WindowType.DoubleHung
                style.WindowShape = OpenShapeType.Rectangular

                dict.AddNewRecord(stylename, style)
                tm.AddNewlyCreatedDBObject(style, True)

                Dim list As StandardSizeOpeningList = New StandardSizeOpeningList()
                Dim opening As StandardSizeOpening = New StandardSizeOpening()
                opening.Set(24.0, 24.0, 0.0, 0.0)
                list.Openings.Add(opening)

                opening = New StandardSizeOpening()
                opening.Set(28.0, 30.0, 0.0, 0.0)
                list.Openings.Add(opening)

                opening = New StandardSizeOpening()
                opening.Set(30.0, 32.0, 0.0, 0.0)
                list.Openings.Add(opening)

                opening = New StandardSizeOpening()
                opening.Set(30.0, 34.0, 0.0, 0.0)
                list.Openings.Add(opening)

                opening = New StandardSizeOpening()
                opening.Set(32.0, 36.0, 0.0, 0.0)
                list.Openings.Add(opening)

                ' create the extension dictionary
                style.CreateExtensionDictionary()
                Dim extDictId As ObjectId = style.ExtensionDictionary
                Dim obj As DBObject = tm.GetObject(extDictId, OpenMode.ForWrite)
                Dim dbDict As DBDictionary = obj

                Dim standardSizesDict As DBDictionary = New DBDictionary()
                dbDict.SetAt(StandardSizeOpeningList.ExtensionDictionaryName, standardSizesDict)
                Dim idStandardSizesDict As ObjectId = dbDict.GetAt(StandardSizeOpeningList.ExtensionDictionaryName)

                tm.AddNewlyCreatedDBObject(standardSizesDict, True)

                ' get the extension dictionary and add the list
                obj = tm.GetObject(idStandardSizesDict, OpenMode.ForWrite)
                dbDict = obj
                dbDict.SetAt(StandardSizeOpeningList.ExtensionDictionaryKeyName, list)

                tm.AddNewlyCreatedDBObject(list, True)
            End If

            stylename = "Susanne"
            description = "Susanne Window Style"

            If Not (dict.Has(stylename, trans)) Then
                ed.WriteMessage("Creating " + description + vbCrLf)

                Dim style As WindowStyle = New WindowStyle()
                style.Description = description
                style.FrameWidth = 2.0
                style.FrameDepth = 4.0
                style.WindowType = WindowType.Casement
                style.WindowShape = OpenShapeType.Hexagon

                dict.AddNewRecord(stylename, style)
                tm.AddNewlyCreatedDBObject(style, True)

                Dim list As StandardSizeOpeningList = New StandardSizeOpeningList()
                Dim opening As StandardSizeOpening = New StandardSizeOpening()
                opening.Set(24.0, 24.0, 0.0, 0.0)
                list.Openings.Add(opening)

                opening = New StandardSizeOpening()
                opening.Set(28.0, 30.0, 0.0, 0.0)
                list.Openings.Add(opening)

                opening = New StandardSizeOpening()
                opening.Set(30.0, 32.0, 0.0, 0.0)
                list.Openings.Add(opening)

                opening = New StandardSizeOpening()
                opening.Set(30.0, 34.0, 0.0, 0.0)
                list.Openings.Add(opening)

                opening = New StandardSizeOpening()
                opening.Set(32.0, 36.0, 0.0, 0.0)
                list.Openings.Add(opening)

                ' create the extension dictionary
                style.CreateExtensionDictionary()
                Dim extDictId As ObjectId = style.ExtensionDictionary
                Dim obj As DBObject = tm.GetObject(extDictId, OpenMode.ForWrite)
                Dim dbDict As DBDictionary = obj

                Dim standardSizesDict As DBDictionary = New DBDictionary()
                dbDict.SetAt(StandardSizeOpeningList.ExtensionDictionaryName, standardSizesDict)
                Dim idStandardSizesDict As ObjectId = dbDict.GetAt(StandardSizeOpeningList.ExtensionDictionaryName)

                tm.AddNewlyCreatedDBObject(standardSizesDict, True)

                ' get the extension dictionary and add the list
                obj = tm.GetObject(idStandardSizesDict, OpenMode.ForWrite)
                dbDict = obj
                dbDict.SetAt(StandardSizeOpeningList.ExtensionDictionaryKeyName, list)

                tm.AddNewlyCreatedDBObject(list, True)
            End If

            trans.Commit()
        Catch
            trans.Abort()
        Finally
            trans.Dispose()
        End Try
    End Sub
#End Region

#Region "IShapeForDesign"
    Private Sub IShapeForDesign(ByVal detailLevel As Autodesk.Aec.Structural.DatabaseServices.DetailLevel, _
                                ByRef curves As Curve2d(), ByRef profile As Profile, _
                                ByVal d As Double, ByVal tw As Double, ByVal tf As Double, ByVal bf As Double)

        curves = Nothing
        profile = Nothing

        Dim hd As Double = d * 0.5
        Dim htw As Double = tw * 0.5
        Dim hbf As Double = bf * 0.5

        If detailLevel = Autodesk.Aec.Structural.DatabaseServices.DetailLevel.Low Then
            ReDim curves(0 To 2)

            Dim line As LineSegment2d = New LineSegment2d(New Point2d(-hbf, hd), New Point2d(hbf, hd))
            curves(0) = line

            line = New LineSegment2d(New Point2d(0.0, hd), New Point2d(0.0, -hd))
            curves(1) = line

            line = New LineSegment2d(New Point2d(-hbf, -hd), New Point2d(hbf, -hd))
            curves(2) = line
        ElseIf detailLevel = Autodesk.Aec.Structural.DatabaseServices.DetailLevel.Medium Then
            Dim pnts As Point2dCollection = New Point2dCollection()

            pnts.Add(New Point2d(-hbf, -hd))
            pnts.Add(New Point2d(hbf, -hd))
            pnts.Add(New Point2d(hbf, -hd + tf))
            pnts.Add(New Point2d(htw, -hd + tf))
            pnts.Add(New Point2d(htw, hd - tf))
            pnts.Add(New Point2d(hbf, hd - tf))
            pnts.Add(New Point2d(hbf, hd))
            pnts.Add(New Point2d(-hbf, hd))
            pnts.Add(New Point2d(-hbf, hd - tf))
            pnts.Add(New Point2d(-htw, hd - tf))
            pnts.Add(New Point2d(-htw, -hd + tf))
            pnts.Add(New Point2d(-hbf, -hd + tf))

            Dim ring As Ring = New Ring(pnts)

            profile = New Profile()
            profile.AddRing(ring)
        Else ' Autodesk.Aec.Structural.DatabaseServices.DetailLevel.Medium (Detail) Then 
            ' this is a quarter arc
            Dim qBulge As Double = 0.41421356
            Dim startWidth As Double = -1
            Dim endWidth As Double = -1

            Dim pline As Polyline = New Polyline()
            pline.AddVertexAt(0, New Point2d(-hbf, -hd), 0, startWidth, endWidth)
            pline.AddVertexAt(1, New Point2d(hbf, -hd), qBulge, startWidth, endWidth)
            pline.AddVertexAt(2, New Point2d(hbf - tf, -hd + tf), 0, startWidth, endWidth)
            pline.AddVertexAt(3, New Point2d(htw + tf, -hd + tf), -qBulge, startWidth, endWidth)
            pline.AddVertexAt(4, New Point2d(htw, -hd + 2 * tf), 0, startWidth, endWidth)
            pline.AddVertexAt(5, New Point2d(htw, hd - 2 * tf), -qBulge, startWidth, endWidth)
            pline.AddVertexAt(6, New Point2d(htw + tf, hd - tf), 0, startWidth, endWidth)
            pline.AddVertexAt(7, New Point2d(hbf - tf, hd - tf), qBulge, startWidth, endWidth)
            pline.AddVertexAt(8, New Point2d(hbf, hd), 0, startWidth, endWidth)
            pline.AddVertexAt(9, New Point2d(-hbf, hd), qBulge, startWidth, endWidth)
            pline.AddVertexAt(10, New Point2d(-hbf + tf, hd - tf), 0, startWidth, endWidth)
            pline.AddVertexAt(11, New Point2d(-htw - tf, hd - tf), -qBulge, startWidth, endWidth)
            pline.AddVertexAt(12, New Point2d(-htw, hd - 2 * tf), 0, startWidth, endWidth)
            pline.AddVertexAt(13, New Point2d(-htw, -hd + 2 * tf), -qBulge, startWidth, endWidth)
            pline.AddVertexAt(14, New Point2d(-htw - tf, -hd + tf), 0, startWidth, endWidth)
            pline.AddVertexAt(15, New Point2d(-hbf + tf, -hd + tf), qBulge, startWidth, endWidth)
            pline.AddVertexAt(16, New Point2d(-hbf, -hd), 0, startWidth, endWidth)

            Dim ring As Ring = New Ring()
            ring.AddSegmentsFrom(pline, True)

            profile = New Profile()
            profile.AddRing(ring)
        End If
    End Sub
#End Region

#Region "TShapeForDesign"
    Private Sub TShapeForDesign(ByVal detailLevel As Autodesk.Aec.Structural.DatabaseServices.DetailLevel, _
                                ByRef curves As Curve2d(), ByRef profile As Profile, _
                                ByVal d As Double, ByVal tw As Double, ByVal tf As Double, ByVal bf As Double)

        curves = Nothing
        profile = Nothing

        Dim hd As Double = d * 0.5
        Dim htw As Double = tw * 0.5
        Dim hbf As Double = bf * 0.5

        If detailLevel = Autodesk.Aec.Structural.DatabaseServices.DetailLevel.Low Then
            ReDim curves(0 To 1)

            Dim line As LineSegment2d = New LineSegment2d(New Point2d(-hbf, hd), New Point2d(hbf, hd))
            curves(0) = line

            line = New LineSegment2d(New Point2d(0.0, -hd), New Point2d(0.0, hd))
            curves(1) = line
        ElseIf detailLevel = Autodesk.Aec.Structural.DatabaseServices.DetailLevel.Medium Then
            Dim pnts As Point2dCollection = New Point2dCollection()

            pnts.Add(New Point2d(-htw, -hd))
            pnts.Add(New Point2d(htw, -hd))
            pnts.Add(New Point2d(htw, hd - tf))
            pnts.Add(New Point2d(hbf, hd - tf))
            pnts.Add(New Point2d(hbf, hd))
            pnts.Add(New Point2d(-hbf, hd))
            pnts.Add(New Point2d(-hbf, hd - tf))
            pnts.Add(New Point2d(-htw, hd - tf))

            Dim ring As Ring = New Ring(pnts)

            profile = New Profile()
            profile.AddRing(ring)
        Else ' Autodesk.Aec.Structural.DatabaseServices.DetailLevel.Medium (Detail) Then 
            ' this is a quarter arc
            Dim qBulge As Double = 0.41421356
            Dim startWidth As Double = -1
            Dim endWidth As Double = -1

            Dim pline As Polyline = New Polyline()
            pline.AddVertexAt(0, New Point2d(-htw, -hd), 0, startWidth, endWidth)
            pline.AddVertexAt(1, New Point2d(htw, -hd), 0, startWidth, endWidth)
            pline.AddVertexAt(2, New Point2d(htw, hd - 2 * tf), -qBulge, startWidth, endWidth)
            pline.AddVertexAt(3, New Point2d(htw + tf, hd - tf), 0, startWidth, endWidth)
            pline.AddVertexAt(4, New Point2d(hbf - tf, hd - tf), qBulge, startWidth, endWidth)
            pline.AddVertexAt(5, New Point2d(hbf, hd), 0, startWidth, endWidth)
            pline.AddVertexAt(6, New Point2d(-hbf, hd), qBulge, startWidth, endWidth)
            pline.AddVertexAt(7, New Point2d(-hbf + tf, hd - tf), 0, startWidth, endWidth)
            pline.AddVertexAt(8, New Point2d(-htw - tf, hd - tf), -qBulge, startWidth, endWidth)
            pline.AddVertexAt(9, New Point2d(-htw, hd - 2 * tf), 0, startWidth, endWidth)
            pline.AddVertexAt(10, New Point2d(-htw, -hd), 0, startWidth, endWidth)

            Dim ring As Ring = New Ring()
            ring.AddSegmentsFrom(pline, True)

            profile = New Profile()
            profile.AddRing(ring)
        End If
    End Sub
#End Region

#Region "ASCForDesign"
    Private Sub AmericanStandardChannelsForDesign(ByVal detailLevel As Autodesk.Aec.Structural.DatabaseServices.DetailLevel, _
                                                  ByRef curves As Curve2d(), ByRef profile As Profile, _
                                                  ByVal d As Double, ByVal tw As Double, ByVal tf As Double, ByVal bf As Double)

        curves = Nothing
        profile = Nothing

        Dim hd As Double = d * 0.5
        Dim htw As Double = tw * 0.5
        Dim hbf As Double = bf * 0.5

        If detailLevel = Autodesk.Aec.Structural.DatabaseServices.DetailLevel.Low Then
            ReDim curves(0 To 2)

            Dim line As LineSegment2d = New LineSegment2d(New Point2d(0.0, hd), New Point2d(hbf, hd))
            curves(0) = line

            line = New LineSegment2d(New Point2d(0.0, hd), New Point2d(0.0, -hd))
            curves(1) = line

            line = New LineSegment2d(New Point2d(0.0, -hd), New Point2d(hbf, -hd))
            curves(2) = line
        ElseIf detailLevel = Autodesk.Aec.Structural.DatabaseServices.DetailLevel.Medium Then
            Dim pnts As Point2dCollection = New Point2dCollection()

            pnts.Add(New Point2d(0.0, -hd))
            pnts.Add(New Point2d(hbf, -hd))
            pnts.Add(New Point2d(hbf, -hd + tf))
            pnts.Add(New Point2d(htw, -hd + tf))
            pnts.Add(New Point2d(htw, hd - tf))
            pnts.Add(New Point2d(hbf, hd - tf))
            pnts.Add(New Point2d(hbf, hd))
            pnts.Add(New Point2d(0.0, hd))

            Dim ring As Ring = New Ring(pnts)

            profile = New Profile()
            profile.AddRing(ring)
        Else ' Autodesk.Aec.Structural.DatabaseServices.DetailLevel.Medium (Detail) Then 
            ' this is a quarter arc
            Dim qBulge As Double = 0.41421356
            Dim startWidth As Double = -1
            Dim endWidth As Double = -1

            Dim pline As Polyline = New Polyline()
            pline.AddVertexAt(0, New Point2d(0.0, -hd), 0, startWidth, endWidth)
            pline.AddVertexAt(1, New Point2d(hbf, -hd), qBulge, startWidth, endWidth)
            pline.AddVertexAt(2, New Point2d(hbf - tf, -hd + tf), 0, startWidth, endWidth)
            pline.AddVertexAt(3, New Point2d(htw + tf, -hd + tf), -qBulge, startWidth, endWidth)
            pline.AddVertexAt(4, New Point2d(htw, -hd + 2 * tf), 0, startWidth, endWidth)
            pline.AddVertexAt(5, New Point2d(htw, hd - 2 * tf), -qBulge, startWidth, endWidth)
            pline.AddVertexAt(6, New Point2d(htw + tf, hd - tf), 0, startWidth, endWidth)
            pline.AddVertexAt(7, New Point2d(hbf - tf, hd - tf), qBulge, startWidth, endWidth)
            pline.AddVertexAt(8, New Point2d(hbf, hd), 0, startWidth, endWidth)
            pline.AddVertexAt(9, New Point2d(0.0, hd), 0, startWidth, endWidth)
            pline.AddVertexAt(10, New Point2d(0.0, -hd), 0, startWidth, endWidth)

            Dim ring As Ring = New Ring()
            ring.AddSegmentsFrom(pline, True)

            profile = New Profile()
            profile.AddRing(ring)
        End If
    End Sub

#End Region

End Class
