#Region "Copyright"
'      .NET Property Definition Formula Sample
'
'      Copyright (c) 2007 by Autodesk, Inc.
'
'      Permission to use, copy, modify, and distribute this software
'      for any purpose and without fee is hereby granted, provided
'      that the above copyright notice appears in all copies and
'      that both that copyright notice and the limited warranty and
'      restricted rights notice below appear in all supporting
'      documentation.
'
'      AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
'      AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
'      MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
'      DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
'      UNINTERRUPTED OR ERROR FREE.
'
'      Use, duplication, or disclosure by the U.S. Government is subject to
'      restrictions set forth in FAR 52.227-19 (Commercial Computer
'      Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
'      (Rights in Technical Data and Computer Software), as applicable.
#End Region

#Region "Imports"
Imports System
Imports System.Collections
Imports System.Collections.Generic
Imports System.Collections.Specialized

Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.Geometry
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.EditorInput

Imports Autodesk.Aec.Geometry
Imports Autodesk.Aec.DatabaseServices
Imports Autodesk.Aec.Arch.DatabaseServices

Imports Autodesk.Aec.PropertyData
Imports Autodesk.Aec.PropertyData.DatabaseServices

Imports DBTransactionManager = Autodesk.AutoCAD.DatabaseServices.TransactionManager
Imports Dictionary = Autodesk.Aec.DatabaseServices.Dictionary

Imports ObjectId = Autodesk.AutoCAD.DatabaseServices.ObjectId
Imports ObjectIdCollection = Autodesk.AutoCAD.DatabaseServices.ObjectIdCollection

Imports Entity = Autodesk.AutoCAD.DatabaseServices.Entity
Imports DBObject = Autodesk.AutoCAD.DatabaseServices.DBObject
Imports AecEntity = Autodesk.Aec.DatabaseServices.Entity
Imports AMEntity = Autodesk.Aec.Modeler.Entity
Imports Microsoft.VisualBasic
#End Region

<Assembly: ExtensionApplication(GetType(AecPropertyDefinitionFormulaSampleMgd.PropertyDefinitionFormulaSample))> 
<Assembly: CommandClass(GetType(AecPropertyDefinitionFormulaSampleMgd.PropertyDefinitionFormulaSample))> 

Namespace AecPropertyDefinitionFormulaSampleMgd

    Public Class PropertyDefinitionFormulaSample
        Implements IExtensionApplication

#Region "IExtensionApplication Members"

        ''' <summary>
        ''' Initialization.
        ''' </summary>
        Public Sub Initialize() Implements IExtensionApplication.Initialize

            Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor
            ed.WriteMessage("Type ""PropertyDefinitionFormulaSample"" to run the sample:" & vbCrLf)
        End Sub


        ''' <summary>
        ''' Termination.
        ''' </summary>
        Public Sub Terminate() Implements IExtensionApplication.Terminate
        End Sub


#End Region

#Region "CommandMethod"

        ''' <summary>
        ''' Command implementation.
        ''' </summary>
        <Autodesk.AutoCAD.Runtime.CommandMethod("PropertyDefinitionFormulaSample", "PropertyDefinitionFormulaSample", Autodesk.AutoCAD.Runtime.CommandFlags.Modal)> _
        Public Sub ShowSample()

            Dim db As Database = GetDatabase()
            Dim ed As Editor = GetEditor()
            ed.WriteMessage("Adding a new property set definition contains a formula property to calculate wall volume." & vbCrLf)
            ''' we need to add all the automatic properties prior to the formula property
            Dim psd As PropertySetDefinition = CreateWallPropertySetDefinition()
            ''' then we add the property set definition to the dictionary to make formula property work properly
            Dim dict As DictionaryPropertySetDefinitions = New DictionaryPropertySetDefinitions(db)
            Using trans As Transaction = db.TransactionManager.StartTransaction()
                dict.AddNewRecord("SampleWallPropertySetDefinition", psd)
                trans.AddNewlyCreatedDBObject(psd, True)
                ''' now we can create the formula property
                Dim formula As PropertyDefinitionFormula = New PropertyDefinitionFormula()
                formula.SetToStandard(db)
                formula.SubSetDatabaseDefaults(db)
                formula.Name = "Wall Volume"
                formula.UseFormulaForDescription = True
                ''' before setting formula string to the formula property, we need to make sure
                ''' that the property definition is added to the property set definition (which has an object id)
                psd.Definitions.Add(formula)
                ''' so we can set the formula string now
                formula.SetFormulaString("[Length]*[Height]*[Width]")
                ''' and here we change the sample values of the referenced properties
                formula.DataItems(0).Sample = 1
                formula.DataItems(1).Sample = 2
                formula.DataItems(2).Sample = 3
                trans.Commit()
            End Using

            ed.WriteMessage("A new property set definition ""SampleWallPropertySetDefinition"" is created." & vbCrLf)
            ed.WriteMessage("It contains a formula definition named ""Wall Volume""." & vbCrLf)
        End Sub


#End Region

#Region "HelperFunction"
        Public Function GetDatabase() As Database

            Return Application.DocumentManager.MdiActiveDocument.Database
        End Function


        Public Function GetEditor() As Editor

            Return Application.DocumentManager.MdiActiveDocument.Editor
        End Function



        Public Function CreateWallPropertySetDefinition() As PropertySetDefinition
            Dim db As Database = GetDatabase()
            Dim def As PropertySetDefinition = New PropertySetDefinition
            ''' we want the property set to apply to walls
            Dim appliesTo As StringCollection = New StringCollection
            Dim wallClassName As String = RXClass.GetClass(GetType(Wall)).Name
            appliesTo.Add(wallClassName)
            def.SetToStandard(db)
            def.SubSetDatabaseDefaults(db)
            def.AlternateName = "SampleWallPropertySet"
            def.IsLocked = False
            def.IsVisible = True
            def.IsWriteable = True

            Dim properties As StringCollection = New StringCollection
            properties.Add("Width")
            properties.Add("Length")
            properties.Add("Height")

            For Each propName As String In properties
                Dim propDef As PropertyDefinition = New PropertyDefinition
                propDef.SetToStandard(db)
                propDef.SubSetDatabaseDefaults(db)
                propDef.Automatic = True
                propDef.Name = propName
                propDef.Description = propName
                propDef.IsVisible = True
                propDef.IsReadOnly = True
                propDef.SetAutomaticData(wallClassName, propName)
                def.Definitions.Add(propDef)
            Next

            Return def
        End Function

#End Region

    End Class

End Namespace
