#Region "Copyright"
'      .NET Sample
'
'      Copyright (c) 2005 by Autodesk, Inc.
'
'      Permission to use, copy, modify, and distribute this software
'      for any purpose and without fee is hereby granted, provided
'      that the above copyright notice appears in all copies and
'      that both that copyright notice and the limited warranty and
'      restricted rights notice below appear in all supporting
'      documentation.
'
'      AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
'      AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
'      MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
'      DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
'      UNINTERRUPTED OR ERROR FREE.
'
'      Use, duplication, or disclosure by the U.S. Government is subject to
'      restrictions set forth in FAR 52.227-19 (Commercial Computer
'      Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
'      (Rights in Technical Data and Computer Software), as applicable.
'
#End Region

#Region "Namespaces"
Imports System.Windows
Imports System.ComponentModel
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.EditorInput
Imports Autodesk.AutoCAD.Runtime

Imports DBTransactionManager = Autodesk.AutoCAD.DatabaseServices.TransactionManager
#End Region


<Assembly: Autodesk.AutoCAD.Runtime.ExtensionApplication(GetType(Class1))> 
<Assembly: Autodesk.AutoCAD.Runtime.CommandClass(GetType(Class1))> 

Public Class Class1
    Implements IExtensionApplication

#Region "IExtensionApplication"
    Public Sub Initialize() Implements IExtensionApplication.Initialize
        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor
        ed.WriteMessage("Loading AecPropertyGridMgd..." & vbCrLf)
    End Sub

    Public Sub Terminate() Implements IExtensionApplication.Terminate
    End Sub
#End Region

#Region "Command"
    <Autodesk.AutoCAD.Runtime.CommandMethod("AecPropertyGridMgd", "PropertyGrid", Autodesk.AutoCAD.Runtime.CommandFlags.Modal)> _
    Public Sub PropertyGrid()

        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

        Dim options As PromptSelectionOptions = New PromptSelectionOptions()
        options.SingleOnly = True
        options.MessageForAdding = "Pick an entity: "

        Dim res1 As PromptSelectionResult = ed.GetSelection(options)

        If res1.Status <> PromptStatus.OK Then
            Exit Sub
        End If

        Dim selSet As ObjectId() = res1.Value.GetObjectIds()

        Dim db As Database
        db = Application.DocumentManager.MdiActiveDocument.Database

        Dim tm As DBTransactionManager
        tm = db.TransactionManager

        Dim trans As Transaction
        trans = tm.StartTransaction()

        Dim res2 As Windows.Forms.DialogResult

        Try

            Dim obj As Object
            obj = tm.GetObject(selSet(0), OpenMode.ForWrite, False, False)

            Dim styleId As ObjectId
            Try
                styleId = obj.StyleId
            Catch
                styleId = ObjectId.Null
            End Try

            If Not styleId.IsNull Then
                Dim res3 As PromptResult = ed.GetString("Option [Object/Style]: ")
                If res3.StringResult.StartsWith("S") Or res3.StringResult.StartsWith("s") Then
                    obj = tm.GetObject(styleId, OpenMode.ForWrite, False, False)
                End If
            End If

#If DEBUG_CM Then
            ' check for Aec properties that are not categorized
            CheckCategories(obj)
#End If

            Dim form As New Form1()
            form.SetObjects(obj)

            res2 = form.ShowDialog()

            ' free up the object owned by the property grid
            form.ResetObjects()

        Catch

            trans.Abort()
            trans.Dispose()
            Exit Try

        Finally

            If res2 = Windows.Forms.DialogResult.OK Then
                trans.Commit()
            End If
            trans.Dispose()

        End Try

    End Sub
#End Region

#Region "CheckCategories"
    Private Sub CheckCategories(ByRef obj As Object)

        Dim results As String = ""
        Dim prop As PropertyDescriptor
        Dim props As PropertyDescriptorCollection
        props = TypeDescriptor.GetProperties(obj)
        For Each prop In props
            If prop.Category = "Misc" Then
                If prop.ComponentType.Namespace.StartsWith("Autodesk.Aec") Then
                    results = results & prop.Name & " (" & prop.ComponentType.Name & ")" & vbCrLf
                End If
            End If
        Next
        If results <> "" Then
            Forms.MessageBox.Show(results, "Category missing!", Windows.Forms.MessageBoxButtons.OK, Windows.Forms.MessageBoxIcon.Information)
        End If

    End Sub
#End Region

End Class
