#Region "Header"
'      .NET Sample
'
'      Copyright (c) 2007 by Autodesk, Inc.
'
'      Permission to use, copy, modify, and distribute this software
'      for any purpose and without fee is hereby granted, provided
'      that the above copyright notice appears in all copies and
'      that both that copyright notice and the limited warranty and
'      restricted rights notice below appear in all supporting
'      documentation.
'
'      AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
'      AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
'      MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
'      DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
'      UNINTERRUPTED OR ERROR FREE.
'
'      Use, duplication, or disclosure by the U.S. Government is subject to
'      restrictions set forth in FAR 52.227-19 (Commercial Computer
'      Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
'      (Rights in Technical Data and Computer Software), as applicable.
#End Region

#Region "Namespaces"

Imports System

Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.EditorInput
Imports Autodesk.AutoCAD.Runtime

Imports Autodesk.Aec.DatabaseServices
Imports Autodesk.Aec.PropertyData.DatabaseServices

Imports DBTransactionManager = Autodesk.AutoCAD.DatabaseServices.TransactionManager
Imports AcadDb = Autodesk.AutoCAD.DatabaseServices
Imports AecDb = Autodesk.Aec.DatabaseServices
Imports AecPropDb = Autodesk.Aec.PropertyData.DatabaseServices

Imports ObjectId = Autodesk.AutoCAD.DatabaseServices.ObjectId
Imports ObjectIdCollection = Autodesk.AutoCAD.DatabaseServices.ObjectIdCollection

#End Region

<Assembly: Autodesk.AutoCAD.Runtime.ExtensionApplication(GetType(Class1))> 
<Assembly: Autodesk.AutoCAD.Runtime.CommandClass(GetType(Class1))> 

Public Class Class1
    Implements IExtensionApplication

#Region "IExtensionApplication"

    Public Sub Initialize() Implements IExtensionApplication.Initialize
        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor
        ed.WriteMessage("Loading AecPropertySampleMgd..." & vbCrLf)
    End Sub

    Public Sub Terminate() Implements IExtensionApplication.Terminate
    End Sub

#End Region

#Region "CommandMethods"

#Region "Command_PropertyQuery"
    <Autodesk.AutoCAD.Runtime.CommandMethod("AecPropertyQueryMgd", "PropertyQuery", Autodesk.AutoCAD.Runtime.CommandFlags.Modal)> _
    Public Sub PropertyQuery()

        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

        ' prompt for the property name and value of interest
        Dim res1 As PromptResult = ed.GetString("Property name: ")
        Dim res2 As PromptResult = ed.GetString("Property value: ")

        Dim propName As String = res1.StringResult
        Dim propValue As String = res2.StringResult

        Dim db As Database = HostApplicationServices.WorkingDatabase
        Dim tm As DBTransactionManager = db.TransactionManager
        Dim trans As Transaction = tm.StartTransaction()

        Try

            Dim bt As BlockTable = tm.GetObject(db.BlockTableId, OpenMode.ForRead, False)
            Dim btr As BlockTableRecord = tm.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForRead, False)

            Dim id As ObjectId
            For Each id In btr

                Dim found As Boolean = False
                Dim foundFromStyle As Boolean = False

                ' get the property sets
                Dim setIds As ObjectIdCollection = Nothing
                GetPropertySets(id, setIds)

                If setIds.Count > 0 Then
                    found = FindPropertyValue(id, setIds, propName, propValue)
                End If

                ' get the property sets from style
                Dim setIdsFromStyle As ObjectIdCollection = Nothing
                GetPropertySetsFromStyle(id, setIdsFromStyle)

                If setIdsFromStyle.Count > 0 Then
                    foundFromStyle = FindPropertyValue(id, setIdsFromStyle, propName, propValue)
                End If

                ' hilight if found
                If found Or foundFromStyle Then

                    Dim obj As Object = tm.GetObject(id, OpenMode.ForRead, False, False)
                    If Not TypeOf obj Is AcadDb.Entity Then
                        Exit Sub
                    End If
                    Dim ent As AcadDb.Entity = obj

                    Dim ids() As ObjectId = {ObjectId.Null}
                    Dim index As SubentityId = New SubentityId(SubentityType.Null, 0)
                    Dim path As FullSubentityPath = New FullSubentityPath(ids, index)
                    ent.Highlight(path, True)

                End If
            Next

        Catch
        Finally
            trans.Dispose()
        End Try

    End Sub
    ' End Command_PropertyQuery
#End Region

#Region "Command_ListPropertySetDefByName"
    ' <summary>
    ' Command implementation for ListPropertySetDefByName.
    ' </summary>
    <Autodesk.AutoCAD.Runtime.CommandMethod("AecPropertySampleMgd", "ListPropertySetDefByName", Autodesk.AutoCAD.Runtime.CommandFlags.Modal)> _
    Public Sub Command_ListPropSetDefByName()
        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

        ' prompt for the property set definition name to list
        Dim res1 As PromptResult = ed.GetString(vbCrLf + "Enter Property Set Definition name: ")

        If res1.Status <> PromptStatus.OK Then
            Return
        End If

        Dim psdName As String = res1.StringResult

        ' Find and get the property set definition objectId by it's name. 
        ' Once we have the objectId we can open and inspect it.
        Dim psdId As ObjectId = GetPropertySetDefinitionIdByName(psdName)

        If psdId = ObjectId.Null Then
            ed.WriteMessage(vbCrLf + "The property set definition named: " + psdName + " does not exit in this drawing.")
            Return
        End If

        Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database
        Dim tm As Autodesk.AutoCAD.DatabaseServices.TransactionManager = db.TransactionManager
        Dim trans As Transaction = tm.StartTransaction()

        Dim psd As AecPropDb.PropertySetDefinition = trans.GetObject(psdId, OpenMode.ForRead)
        ed.WriteMessage(vbCrLf + vbCrLf + "Property Set Definition Name: " + psd.Name)
        ed.WriteMessage(vbCrLf + "Display Name: " + psd.DisplayName)
        ed.WriteMessage(vbCrLf + "Description: " + psd.Description)
        ed.WriteMessage(vbCrLf + "Is Visible: " + psd.IsVisible.ToString())
        Dim appliesto As System.Collections.Specialized.StringCollection = psd.AppliesToFilter
        Dim s As String
        For Each s In appliesto
            ed.WriteMessage(vbCrLf + "  Applies To: " + s)
        Next
        Dim propdefs As AecPropDb.PropertyDefinitionCollection = psd.Definitions
        ed.WriteMessage(vbCrLf + "Definitions Count: " + propdefs.Count.ToString() + vbCrLf)

        Dim propdef As AecPropDb.PropertyDefinition
        For Each propdef In propdefs
            ed.WriteMessage(vbCrLf + "  PropDef Name: " + propdef.Name)
            ed.WriteMessage(vbCrLf + "    Description: " + propdef.Description)
            ed.WriteMessage(vbCrLf + "    IsVisible: " + propdef.IsVisible.ToString())
            ed.WriteMessage(vbCrLf + "    IsAutomatic: " + propdef.Automatic.ToString())
            Try
                ed.WriteMessage(vbCrLf + "    UnitType: " + propdef.UnitType.ToString())
            Catch

            End Try

            ed.WriteMessage(vbCrLf + "    DataType: " + propdef.DataType.ToString())
            ed.WriteMessage(vbCrLf + "    DefaultData: " + propdef.DefaultData.ToString())
        Next
    End Sub
#End Region

#Region "Command_CreatePropertySetDef"
    ' <summary>
    ' Command implementation for CreatePropertySetDef.
    ' </summary>
    <Autodesk.AutoCAD.Runtime.CommandMethod("AecPropertySampleMgd", "CreatePropertySetDef", Autodesk.AutoCAD.Runtime.CommandFlags.Modal)> _
    Public Sub Command_CreatePropertySetDef()

        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

        ' prompt for the property set definition name to create
        Dim res1 As PromptResult = ed.GetString(vbCrLf + "Enter Property Set Definition name: ")

        If res1.Status <> PromptStatus.OK Then
            Return
        End If

        Dim psdname As String = res1.StringResult

        Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database

        ' Setup an array of objects that this property set definition will apply to
        Dim appliesto As System.Collections.Specialized.StringCollection = New System.Collections.Specialized.StringCollection()
        appliesto.Add("AecDbWall")
        appliesto.Add("AecDbWindow")
        appliesto.Add("AcDbLine")
        appliesto.Add("AcDbCircle")

        ' Create a list of property definitions that will reside in the property set definition
        Dim defs As System.Collections.ArrayList = New System.Collections.ArrayList()
        Dim def As AecPropDb.PropertyDefinition
        Dim tm As AcadDb.TransactionManager = db.TransactionManager

        Dim i As Integer
        For i = 0 To 9 Step i + 1
            def = New AecPropDb.PropertyDefinition()
            def.SetToStandard(db)
            def.SubSetDatabaseDefaults(db)
            def.Name = "Test" + i.ToString()
            def.DataType = Autodesk.Aec.PropertyData.DataType.Integer
            def.DefaultData = i
            defs.Add(def)
        Next

        ' Creates the property set and provides back the objectId.
        Dim id As ObjectId = CreatePropertySetDefinition(psdname, appliesto, defs)
    End Sub
#End Region

#Region "Command_FindPropertySetByNameOnObject"
    ' <summary>
    ' Command implementation for FindPropertySetByNameOnObject.
    ' </summary>
    <Autodesk.AutoCAD.Runtime.CommandMethod("AecPropertySampleMgd", "FindPropertySetByNameOnObject", Autodesk.AutoCAD.Runtime.CommandFlags.Modal)> _
    Public Sub Command_FindPropertySetByNameOnObject()
        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

        Try
            ' prompt for the property set name
            Dim res1 As PromptResult = ed.GetString(vbCrLf + "Enter Property Set name: ")

            If res1.Status <> PromptStatus.OK Then
                Return
            End If

            Dim psname As String = res1.StringResult

            Dim entopts As PromptEntityOptions = New PromptEntityOptions(vbCrLf + "Select an entity to find property set ")
            Dim ent As PromptEntityResult = Nothing
            Try
                ent = ed.GetEntity(entopts)
            Catch
                ed.WriteMessage(vbCrLf + "You did not select a valid entity")
                Return
            End Try

            If ent.Status = PromptStatus.OK Then
                Dim entid As ObjectId = ent.ObjectId
                Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database
                Dim tm As Autodesk.AutoCAD.DatabaseServices.TransactionManager = db.TransactionManager
                Dim myT As Transaction = tm.StartTransaction()

                Dim entity As Autodesk.AutoCAD.DatabaseServices.Entity = tm.GetObject(entid, OpenMode.ForRead, True)
                Dim wasFound As Boolean = GetPropSetByNameFromObject(entity, psname)

                If wasFound Then
                    ed.WriteMessage(vbCrLf + " Found Property Set named: " + psname + " on entity selected")
                    ' See Also Autodesk.Aec.PropertyData.DatabaseServices.PropertyDataServices.GetPropertySets(ObjectId, ObjectId, ObjectIdCollection)
                    ' that one handles xrefs via the blockref path mechanism.
                    Dim ids As ObjectIdCollection = AecPropDb.PropertyDataServices.GetPropertySets(entity)
                    ed.WriteMessage(vbCrLf + " Found a total of " + ids.Count.ToString() + " property sets on entity selected")
                Else
                    ed.WriteMessage(vbCrLf + " DID NOT Find Property Set named " + psname + " on entity selected")
                End If

                myT.Commit()
                myT.Dispose()
            End If
        Catch e As SystemException
            ed.WriteMessage("Exception: " + e.Message)
        Finally

        End Try

    End Sub
#End Region

#Region "Command_GetPropSetDefIdFromPropertyName"
    ' <summary>
    ' Command implementation for GetPropSetDefIdFromPropertyName.
    ' </summary>

    <Autodesk.AutoCAD.Runtime.CommandMethod("AecPropertySampleMgd", "GetPropSetDefIdFromPropertyName", Autodesk.AutoCAD.Runtime.CommandFlags.Modal)> _
    Public Sub Command_GetPropSetDefIdFromPropertyName()
        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

        ' prompt for the property name
        Dim res1 As PromptResult = ed.GetString(vbCrLf + "Enter Property name: ")

        If res1.Status <> PromptStatus.OK Then
            Return
        End If

        Dim pname As String = res1.StringResult

        Dim db As Database = HostApplicationServices.WorkingDatabase
        Dim tm As AcadDb.TransactionManager = db.TransactionManager

        Dim trans As Transaction = tm.StartTransaction()

        Dim bt As BlockTable = CType(tm.GetObject(db.BlockTableId, OpenMode.ForRead, False), BlockTable)
        Dim btr As BlockTableRecord = CType(tm.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForRead, False), BlockTableRecord)

        Dim id As ObjectId
        For Each id In btr

            ' get the property sets
            Dim setIds As ObjectIdCollection
            Dim dbobj As AcadDb.DBObject = tm.GetObject(id, OpenMode.ForRead, False, False)
            setIds = AecPropDb.PropertyDataServices.GetPropertySets(dbobj)

            Dim psId As ObjectId
            For Each psId In setIds
                Dim pset As AecPropDb.PropertySet = tm.GetObject(psId, OpenMode.ForRead, False, False)

                ' First see if the property with given name is in this set. if not, go to next set.
                Dim pid As Integer
                Try
                    pid = pset.PropertyNameToId(pname)
                Catch
                    ' most likely eKeyNotfound skip
                    Continue For
                End Try

                Dim psdefId As ObjectId = pset.PropertySetDefinition
                ed.WriteMessage(vbCrLf + " Property Set with id: " + psId.ToString() + " contains a property with name: " + pname)
                ed.WriteMessage(vbCrLf + " and refernces the property set definition with id: " + psdefId.ToString())

                ' Lets also output some property set information about the property
                Dim value_unit As AecPropDb.PropertyValueUnitPair = pset.GetValueAndUnitAt(pid)

                Dim val As Object = value_unit.Value

                ed.WriteMessage(vbCrLf + " UnitType = " + value_unit.UnitType.InternalName + ", IsImperial = " + value_unit.UnitType.IsImperial.ToString() + ", Type = " + value_unit.UnitType.Type.ToString())
                ed.WriteMessage(vbCrLf + " DataType = " + val.GetType().ToString())
                ed.WriteMessage(vbCrLf + " Value = " + val.ToString())

            Next

        Next
        trans.Commit()
        trans.Dispose()

    End Sub

#End Region

#Region "Command_GetPropertyDataByName"
    ' <summary>
    ' Command implementation for GetPropertyDataByName.
    ' </summary>

    <Autodesk.AutoCAD.Runtime.CommandMethod("AecPropertySampleMgd", "GetPropertyDataByName", Autodesk.AutoCAD.Runtime.CommandFlags.Modal)> _
    Public Sub Command_GetPropertyDataByName()
        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

        ' prompt for the property name
        Dim res1 As PromptResult = ed.GetString(vbCrLf + "Enter Property name: ")

        If res1.Status <> PromptStatus.OK Then
            Return
        End If

        Dim pname As String = res1.StringResult

        Dim db As Database = HostApplicationServices.WorkingDatabase
        Dim tm As AcadDb.TransactionManager = db.TransactionManager

        Dim dbobj As AcadDb.DBObject

        Dim trans As Transaction = tm.StartTransaction()

        Dim bt As BlockTable = tm.GetObject(db.BlockTableId, OpenMode.ForRead, False)
        Dim btr As BlockTableRecord = tm.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForRead, False)

        Dim id As ObjectId
        For Each id In btr

            dbobj = tm.GetObject(id, OpenMode.ForRead, False, False)

            Dim values As System.Collections.ArrayList = GetValuesFromPropertySetByName(pname, dbobj)
            Dim value_unit As AecPropDb.PropertyValueUnitPair
            For Each value_unit In values
                ed.WriteMessage(vbCrLf + "Property with name = " + pname + " Entity ObjectId = " + id.ToString())
                ed.WriteMessage(vbCrLf + "Unit Type = " + value_unit.UnitType.InternalName + ", IsImperial = " + value_unit.UnitType.IsImperial.ToString() + ", Type = " + value_unit.UnitType.Type.ToString())

                Dim val As Object = value_unit.Value
                If Not val Is Nothing Then
                    ed.WriteMessage(vbCrLf + "DataType = " + val.GetType().ToString())
                    ed.WriteMessage(vbCrLf + "Value = " + val.ToString())
                End If

            Next
        Next
        trans.Commit()
        trans.Dispose()
    End Sub
#End Region

#Region "Command_SetPropertyDataByName"
    ' <summary>
    ' Command implementation for SetPropertyDataByName.
    ' </summary>
    <Autodesk.AutoCAD.Runtime.CommandMethod("AecPropertySampleMgd", "SetPropertyDataByName", Autodesk.AutoCAD.Runtime.CommandFlags.Modal)> _
    Public Sub Command_SetPropertyDataByName()
        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

        ' prompt for the property name
        Dim res1 As PromptResult = ed.GetString(vbCrLf + "Enter Property name (must be a floating point type for this sample): ")

        If res1.Status <> PromptStatus.OK Then
            Return
        End If

        Dim pname As String = res1.StringResult


        Dim res2 As PromptDoubleResult = ed.GetDouble(vbCrLf + "Enter new Property value (floating point number): ")

        If res2.Status <> PromptStatus.OK Then
            Return
        End If

        Dim NewValue As Double = res2.Value


        Dim db As Database = HostApplicationServices.WorkingDatabase
        Dim tm As AcadDb.TransactionManager = db.TransactionManager

        Dim dbobj As AcadDb.DBObject

        Dim trans As Transaction = tm.StartTransaction()

        Dim bt As BlockTable = tm.GetObject(db.BlockTableId, OpenMode.ForRead, False)
        Dim btr As BlockTableRecord = tm.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForRead, False)

        Dim id As ObjectId
        For Each id In btr
            dbobj = tm.GetObject(id, OpenMode.ForRead, False, False)
            Dim values As System.Collections.ArrayList = GetValuesFromPropertySetByName(pname, dbobj)
            Dim value_unit As AecPropDb.PropertyValueUnitPair
            For Each value_unit In values
                Dim currentValue As Object = value_unit.Value
                If Not currentValue Is Nothing Then
                    If (currentValue.GetType() Is GetType(Double)) Then
                        Dim trans2 As Transaction = tm.StartTransaction()

                        dbobj.UpgradeOpen()
                        Dim WasChanged As Boolean = SetValuesFromPropertySetByName(pname, dbobj, NewValue)
                        If WasChanged Then
                            ed.WriteMessage(vbCrLf + "Succesfully changed value for objectId = " + id.ToString())
                            ed.WriteMessage(vbCrLf + "Current Value = " + currentValue.ToString() + " New Value = " + NewValue.ToString())
                        Else
                            ed.WriteMessage(vbCrLf + "Failed to change value for objectId = " + id.ToString())
                        End If
                        trans2.Commit()
                        trans2.Dispose()
                    End If
                End If
            Next
        Next
        trans.Commit()
        trans.Dispose()
    End Sub
#End Region

#Region "Command_CreatePropSetOnDBObject"
    ' <summary>
    ' Command implementation for CreatePropSetOnDBObject.
    ' </summary>

    <Autodesk.AutoCAD.Runtime.CommandMethod("AecPropertySampleMgd", "CreatePropSetOnDBObject", Autodesk.AutoCAD.Runtime.CommandFlags.Modal)> _
    Public Sub Command_CreatePropSetOnDBObject()
        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor
        Try
            Dim entopts As PromptEntityOptions = New PromptEntityOptions("Select an entity to add a property set to: ")
            Dim ent As PromptEntityResult = Nothing
            Try
                ent = ed.GetEntity(entopts)
            Catch
                ed.WriteMessage("You did not select a valid entity")
                Return
            End Try

            If (ent.Status <> PromptStatus.OK) Then
                Return
            End If


            ' prompt for the property name
            Dim res1 As PromptResult = ed.GetString(vbCrLf + "Enter Property set definition name: ")

            If (res1.Status <> PromptStatus.OK) Then
                Return
            End If


            Dim pname As String = res1.StringResult


            Dim propsetId As ObjectId = GetPropertySetDefinitionIdByName(pname)
            If (propsetId.IsNull) Then
                ed.WriteMessage(vbCrLf + " There are no property set definitions by that name")
            End If


            Dim entid As ObjectId = ent.ObjectId
            Dim WasCreated As Boolean = CreatePropSetOnDBObject(entid, propsetId)

            If (WasCreated) Then
                ed.WriteMessage(vbCrLf + " Successfully created property set named " + pname + " on selected entity")
            Else
                ed.WriteMessage(vbCrLf + " Failed to create property set on entity.")
            End If


        Catch

        End Try
    End Sub
#End Region

#Region "Command_CreatePropSetDefintionWithAutomaticData"
    ' <summary>
    ' Command implementation for CreatePropSetDefAuto.
    ' </summary>

    <Autodesk.AutoCAD.Runtime.CommandMethod("AecPropertySampleMgd", "CreatePropSetDefAuto", Autodesk.AutoCAD.Runtime.CommandFlags.Modal)> _
    Public Sub Command_CreatePropSetDefintionWithAutomaticData()

        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor
        Try

            Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database

            ' prompt for the property name
            Dim result As PromptResult = ed.GetString(vbCrLf + "Enter new property set definition name: ")

            If (result.Status <> PromptStatus.OK) Then
                ed.WriteMessage(vbCrLf + "Error with input...")
                Return
            End If

            Dim pname As String = result.StringResult

            Dim dictDefs As DictionaryPropertySetDefinitions = New DictionaryPropertySetDefinitions(db)
            Dim trans As Transaction = db.TransactionManager.StartTransaction()
            If (dictDefs.Has(pname, trans) = True) Then
                ed.WriteMessage(vbCrLf + "Property set with that name already exists. Try again, please...")
                trans.Commit()
                trans.Dispose()
                Return
            End If
            trans.Commit()
            trans.Dispose()

            result = ed.GetString(vbCrLf + "Enter class name to work with: ")

            If (result.Status <> PromptStatus.OK) Then
                ed.WriteMessage(vbCrLf + "Error with input...")
                Return
            End If

            Dim className As String = result.StringResult

            Dim runtime As Autodesk.AutoCAD.Runtime.Dictionary = Autodesk.AutoCAD.Runtime.SystemObjects.ClassDictionary
            If (runtime.Contains(className) = False) Then
                ed.WriteMessage(vbCrLf + "Objects of that class type are not present in the system. For example, for wall it will be AecDbWall.")
                Return
            End If

            Dim WasCreated As Boolean = CreatePropertySetDefinitionWithAutoData(pname, className)

            If (WasCreated) Then
                ed.WriteMessage(vbCrLf + " Successfully created property set definition named " + pname + " with all available automatic data.")
            Else
                ed.WriteMessage(vbCrLf + " Failed to create property set definition.")
            End If
        Catch

        End Try
    End Sub

#End Region

#Region "Command_CreatePropSetDefintionWithAutomaticDataMultiClass"
    ' <summary>
    ' Command implementation for CreatePropSetDefAutoDup.
    ' </summary>

    <Autodesk.AutoCAD.Runtime.CommandMethod("AecPropertySampleMgd", "CreatePropSetDefAutoDup", Autodesk.AutoCAD.Runtime.CommandFlags.Modal)> _
    Public Sub Command_CreatePropSetDefintionWithAutomaticDataMultiClass()

        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor
        Try

            Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database

            ' prompt for the property name
            Dim result As PromptResult = ed.GetString(vbCrLf + "Enter new property set definition name: ")

            If (result.Status <> PromptStatus.OK) Then
                ed.WriteMessage(vbCrLf + "Error with input...")
                Return
            End If


            Dim pname As String = result.StringResult

            Dim dictDefs As DictionaryPropertySetDefinitions = New DictionaryPropertySetDefinitions(db)
            Dim trans As Transaction = db.TransactionManager.StartTransaction()

            If (dictDefs.Has(pname, trans) = True) Then
                ed.WriteMessage(vbCrLf + "Property set with that name already exists. Try again, please...")
                trans.Dispose()
                trans.Commit()
                Return
            End If
            trans.Commit()
            trans.Dispose()


            Dim classNames As System.Collections.Specialized.StringCollection = New System.Collections.Specialized.StringCollection()
            Dim runtime As Autodesk.AutoCAD.Runtime.Dictionary = Autodesk.AutoCAD.Runtime.SystemObjects.ClassDictionary

            result = ed.GetString(vbCrLf + "Enter first class name to work with: ")

            If (result.Status <> PromptStatus.OK) Then
                ed.WriteMessage(vbCrLf + "Error with input...")
                Return
            End If

            Dim className As String = result.StringResult
            If (runtime.Contains(className) = True) Then
                classNames.Add(className)
            Else
                ed.WriteMessage(vbCrLf + "Objects of that class type are not present in the system. For example, for wall it will be in the format: AecDbWall.")
                Return
            End If

            result = ed.GetString(vbCrLf + "Enter second class name to work with: ")

            If (result.Status <> PromptStatus.OK) Then
                ed.WriteMessage(vbCrLf + "Error with input...")
                Return
            End If

            className = result.StringResult
            If (runtime.Contains(className) = True) Then
                classNames.Add(result.StringResult)
            Else
                ed.WriteMessage(vbCrLf + "Objects of that class type are not present in the system. For example, for wall it will be in the format: AecDbWall.")
                Return
            End If

            Dim WasCreated As Boolean = CreatePropertySetDefinitionWithAutoDataDup(pname, classNames)

            If (WasCreated) Then
                ed.WriteMessage(vbCrLf + " Successfully created property set definition named " + pname + " with all available automatic data.")
            Else
                ed.WriteMessage(vbCrLf + " Failed to create property set definition.")
            End If
        Catch

        End Try
    End Sub
#End Region

#Region "Command_GetPropertyXRef"
    ' <summary>
    ' Command implementation for Get property X ref.
    ' </summary>

    <Autodesk.AutoCAD.Runtime.CommandMethod("AecPropertySampleMgd", "GetPropertyXRef", Autodesk.AutoCAD.Runtime.CommandFlags.Modal)> _
    Public Sub Command_GetPropertyXRef()
        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor
        Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database
        Dim tm As DBTransactionManager = db.TransactionManager
        Dim trans As Transaction = tm.StartTransaction()

        Try
            Dim pso As PromptSelectionOptions = New PromptSelectionOptions()
            pso.MessageForAdding = "Select an Object from X reference:"
            pso.SingleOnly = True
            Dim result As PromptSelectionResult = ed.GetSelection(pso)
            If result.Status <> PromptStatus.OK Then
                ed.WriteMessage("Error with input...")
                Return
            End If


            Dim pso2 As PromptStringOptions = New PromptStringOptions("Input the Property Set Name:")
            pso2.DefaultValue = "WallObjects"
            Dim testSetName As String = ed.GetString(pso2).StringResult

            Dim pso3 As PromptStringOptions = New PromptStringOptions("Input the Property Name:")
            pso3.DefaultValue = "Width"
            Dim testPropName As String = ed.GetString(pso3).StringResult

            Dim idRefPaths As ObjectIdAndBlockReferencePathCollection = New ObjectIdAndBlockReferencePathCollection()
            For Each selectedObj As SelectedObject In result.Value
                If selectedObj.ObjectId.ObjectClass = RXObject.GetClass(GetType(Autodesk.AutoCAD.DatabaseServices.BlockReference)) Then
                    'If selectedObj.ObjectId Is Autodesk.AutoCAD.DatabaseServices.BlockReference Then

                    Dim br As Autodesk.AutoCAD.DatabaseServices.BlockReference = trans.GetObject(selectedObj.ObjectId, OpenMode.ForWrite)
                    Dim btr As BlockTableRecord = trans.GetObject(br.BlockTableRecord, OpenMode.ForWrite)
                    For Each id As ObjectId In btr
                        Dim index As Int32 = idRefPaths.Add(New ObjectIdAndBlockReferencePath(id))
                        ShowPropertyData(id, idRefPaths(index).BlockReferencePath, testSetName, testPropName)
                    Next
                End If
            Next
            trans.Commit()
        Catch e As System.Exception
            ed.WriteMessage(e.Message)
            trans.Abort()
        Finally
            trans.Dispose()
        End Try
    End Sub
#End Region

#End Region

#Region "ImplementationMethods"

#Region "GetPropertySets"
    Private Sub GetPropertySets(ByVal id As ObjectId, ByRef setIds As ObjectIdCollection)

        setIds = New ObjectIdCollection()

        Dim db As Database = HostApplicationServices.WorkingDatabase
        Dim tm As DBTransactionManager = db.TransactionManager
        Dim trans As Transaction = tm.StartTransaction()

        Try

            Dim obj As Object = tm.GetObject(id, OpenMode.ForRead, False, False)
            If Not TypeOf obj Is AecDb.Entity Then
                Exit Sub
            End If
            Dim ent As AecDb.Entity = obj

            ' get the property sets on the object
            setIds = PropertyDataServices.GetPropertySets(ent)

        Catch
        Finally
            trans.Dispose()
        End Try

    End Sub
#End Region

#Region "GetPropertySetsFromStyle"
    Private Sub GetPropertySetsFromStyle(ByVal id As ObjectId, ByRef setIds As ObjectIdCollection)

        setIds = New ObjectIdCollection()

        Dim db As Database = HostApplicationServices.WorkingDatabase
        Dim tm As DBTransactionManager = db.TransactionManager
        Dim trans As Transaction = tm.StartTransaction()

        Try

            Dim obj As Object = tm.GetObject(id, OpenMode.ForRead, False, False)
            If Not TypeOf obj Is AecDb.Entity Then
                Exit Sub
            End If
            Dim ent As AecDb.Entity = obj

            ' use late binding to see if the entity has a StyleId property
            obj = ent.GetType().InvokeMember("StyleId", System.Reflection.BindingFlags.GetProperty, Nothing, ent, Nothing)
            If Not TypeOf obj Is ObjectId Then
                Exit Sub
            End If

            Dim styleId As ObjectId = obj
            If styleId.IsNull Then
                Exit Sub
            End If

            obj = tm.GetObject(styleId, OpenMode.ForRead, False, False)
            If Not TypeOf obj Is AecDb.DBObject Then
                Exit Sub
            End If
            Dim style As AecDb.DBObject = obj

            ' get the property sets from style
            setIds = PropertyDataServices.GetPropertySets(style)

        Catch
        Finally
            trans.Dispose()
        End Try

    End Sub
#End Region

#Region "FindPropertyValue"
    Private Function FindPropertyValue(ByVal id As ObjectId, ByVal setIds As ObjectIdCollection, ByVal propName As String, ByVal propValue As String) As Boolean

        Dim result As Boolean = False

        Dim db As Database = HostApplicationServices.WorkingDatabase
        Dim tm As DBTransactionManager = db.TransactionManager
        Dim trans As Transaction = tm.StartTransaction()

        Try

            Dim setId As ObjectId
            For Each setId In setIds

                ' open the prop set
                Dim obj As Object = tm.GetObject(setId, OpenMode.ForRead, False, False)
                If Not TypeOf obj Is PropertySet Then
                    GoTo [CONTINUE]
                End If
                Dim propSet As PropertySet = obj

                Dim index As Integer = -1
                Dim data As Object

                Try
                    index = propSet.PropertyNameToId(propName)
                Catch
                End Try

                ' see if we have a manual property
                If index > -1 Then

                    data = propSet.GetAt(index)

                    ' check for a match
                    If data.ToString() = propValue Then

                        result = True
                        Exit For
                    End If

                End If

                ' go through the prop set def for automatic properties
                Dim idPropSetDef As ObjectId = propSet.PropertySetDefinition
                If idPropSetDef.IsNull Then
                    GoTo [CONTINUE]
                End If

                obj = tm.GetObject(idPropSetDef, OpenMode.ForRead, False, False)
                If Not TypeOf obj Is PropertySetDefinition Then
                    GoTo [CONTINUE]
                End If
                Dim propSetDef As PropertySetDefinition = obj

                ' find the automatic property
                Dim bFound As Boolean = False
                Dim propDef As PropertyDefinition
                For Each propDef In propSetDef.Definitions
                    If propDef.Name = propName Then
                        bFound = True
                    End If
                Next

                If Not bFound Then
                    GoTo [CONTINUE]
                End If

                ' get the data
                Dim idBlockRefPaths As ObjectIdCollection = New ObjectIdCollection()
                data = propSetDef.GetValue(index, id, idBlockRefPaths)

                ' check for a match
                If data.ToString() = propValue Then
                    result = True
                    Exit For
                End If

[CONTINUE]:
            Next

        Catch

        Finally
            trans.Dispose()
        End Try

        FindPropertyValue = result

    End Function
#End Region

#Region "GetPropertySetDefinitionIdByName"
    ' <summary>
    ' Finds the property set definition objectId with the given name.
    ' </summary>
    ' <param name="psdName">The property set definition name.</param>
    ' <returns> true, if a property with the specified value is found.</returns>
    Private Function GetPropertySetDefinitionIdByName(ByVal psdName As String) As ObjectId
        Dim psdId As ObjectId = ObjectId.Null
        Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database
        Dim tm As Autodesk.AutoCAD.DatabaseServices.TransactionManager = db.TransactionManager
        Dim trans As Transaction = tm.StartTransaction()

        Dim psdDict As AecPropDb.DictionaryPropertySetDefinitions = New AecPropDb.DictionaryPropertySetDefinitions(db)
        If psdDict.Has(psdName, trans) Then
            psdId = psdDict.GetAt(psdName)
        End If

        trans.Commit()
        trans.Dispose()
        Return psdId
    End Function
#End Region

#Region "CreatePropertySetDefinition"
    ' <summary>
    ' Creates a property set definition with given name, applies to filter, and property definitions.
    ' </summary>
    ' <param name="psdName">The property set definition name.</param>
    ' <param name="appliesto">An array of strings that represent the objects this property set will apply to.</param>
    ' <param name="propdefs">An array of property definitions that will be contained by this property set definition.</param>
    ' <returns> The objectId of the eixtsing or newly created property set definiton.</returns>
    Private Function CreatePropertySetDefinition(ByVal psdname As String, _
                                                 ByVal appliesto As System.Collections.Specialized.StringCollection, _
                                                 ByVal propdefs As System.Collections.ArrayList) As ObjectId
        Dim psdId As ObjectId

        ' first let's see if it exists... If so just return its ObjectId.
        psdId = GetPropertySetDefinitionIdByName(psdname)
        If psdId <> ObjectId.Null Then
            Return psdId
        End If

        Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database

        ' Create the new property set definition;
        Dim tm As Autodesk.AutoCAD.DatabaseServices.TransactionManager = db.TransactionManager
        Dim psd As AecPropDb.PropertySetDefinition = New AecPropDb.PropertySetDefinition()
        psd.SetToStandard(db)
        psd.SubSetDatabaseDefaults(db)
        psd.AlternateName = psdname
        psd.IsLocked = False
        psd.IsVisible = True
        psd.IsWriteable = True

        psd.SetAppliesToFilter(appliesto, False)

        ' Place the property definitions into the new property set by
        ' first getting the collection of property definitions (in this
        ' case the collection is empty because it is new), and the new 
        ' property defintions to the collection.
        Dim psdefs As AecPropDb.PropertyDefinitionCollection = psd.Definitions
        Dim propdef As AecPropDb.PropertyDefinition
        For Each propdef In propdefs
            psdefs.Add(propdef)
        Next

        ' Add the property set definition to the dictionary
        Dim trans As Transaction = tm.StartTransaction()

        Dim psdDict As AecPropDb.DictionaryPropertySetDefinitions = New AecPropDb.DictionaryPropertySetDefinitions(db)
        psdDict.AddNewRecord(psdname, psd)
        tm.AddNewlyCreatedDBObject(psd, True)

        psdId = psd.ObjectId
        trans.Commit()
        trans.Dispose()

        Return psdId
    End Function

#End Region

#Region "GetPropSetByNameFromObject"
    ' <summary>
    ' Finds the property set by its name on a given object.
    ' </summary>
    ' <param name="dbobj">The property set definition name.</param>
    ' <param name="propname">The property name to find on the object.</param>
    ' <returns> true if the property set with the given name was found, or false otherwise. </returns>
    Private Function GetPropSetByNameFromObject(ByVal dbobj As AcadDb.DBObject, ByVal propname As String) As Boolean
        Dim definitionId As ObjectId = GetPropertySetDefinitionIdByName(propname)
        Dim propSetId As ObjectId = ObjectId.Null
        If Not definitionId.IsNull Then
            Try
                propSetId = AecPropDb.PropertyDataServices.GetPropertySet(dbobj, definitionId)
            Catch
                ' More than likely eKeyNotFound, so this is a more specific
                ' place to handle a "failed to find" condition.
            End Try
        End If

        If Not propSetId.IsNull Then
            Return True ' got an ID so we found it
        End If

        Return False ' didn't find it by name on this object.
    End Function
#End Region

#Region "GetValuesFromPropertySetByName"
    ' <summary>
    ' Returns the values (PropertyValueUnitPair) of a property by name on a given object.
    ' </summary>
    ' <param name="pname">The property name to find on the object.</param>
    ' <param name="dbobj">The object to find the property on. </param>
    ' <returns> An array of the values </returns>
    Public Function GetValuesFromPropertySetByName(ByVal pname As String, ByVal dbobj As AcadDb.DBObject) As System.Collections.ArrayList

        Dim setIds As ObjectIdCollection = AecPropDb.PropertyDataServices.GetPropertySets(dbobj)

        Dim values As System.Collections.ArrayList = New System.Collections.ArrayList()

        If setIds.Count = 0 Then
            Return values ' just return emtpy collection...
        End If


        Dim db As Database = HostApplicationServices.WorkingDatabase
        Dim tm As AcadDb.TransactionManager = db.TransactionManager
        Dim psId As ObjectId
        For Each psId In setIds
            Dim pset As AecPropDb.PropertySet = tm.GetObject(psId, OpenMode.ForRead, False, False) 'As AecPropDb.PropertySet
            Dim pid As Integer
            Try
                pid = pset.PropertyNameToId(pname)
                values.Add(pset.GetValueAndUnitAt(pid))
            Catch e As Autodesk.AutoCAD.Runtime.Exception
                ' most likely eKeyNotfound.

            End Try

        Next

        Return values
    End Function
#End Region

#Region "SetValuesFromPropertySetByName"
    ' <summary>
    ' Sets the values (PropertyValueUnitPair) of a property by name on a given object.
    ' </summary>
    ' <param name="pname">The property name to find on the object.</param>
    ' <param name="dbobj">The object to set the property on. </param>
    ' <param name="value">The value to set. </param>
    ' <returns> true if succesful, or false otherwise. </returns>
    Public Function SetValuesFromPropertySetByName(ByVal pname As String, ByVal dbobj As AcadDb.DBObject, ByVal value As Object) As Boolean
        Dim findany As Boolean = False
        Dim setIds As ObjectIdCollection = AecPropDb.PropertyDataServices.GetPropertySets(dbobj)

        Dim db As Database = HostApplicationServices.WorkingDatabase
        Dim tm As AcadDb.TransactionManager = db.TransactionManager
        Dim trans As Transaction = tm.StartTransaction()

        Dim psId As ObjectId
        For Each psId In setIds
            Dim pset As AecPropDb.PropertySet = tm.GetObject(psId, OpenMode.ForWrite, False, False) ' As AecPropDb.PropertySet
            Dim pid As Integer
            Try
                pid = pset.PropertyNameToId(pname)
                If (pset.IsWriteEnabled) Then
                    pset.SetAt(pid, value)
                End If
                findany = True
            Catch e As Autodesk.AutoCAD.Runtime.Exception
                ' most likely eKeyNotfound
            End Try
        Next

        trans.Commit()
        trans.Dispose()

        Return findany
    End Function

#End Region

#Region "CreatePropSetOnDBObject"
    ' <summary>
    ' Creates a new property set on the given DBObject.
    ' </summary>
    ' <param name="pname">The property name to find on the object.</param>
    ' <param name="dbobj">The object to find the property on. </param>
    ' <param name="value">The value to set. </param>
    ' <returns> true is succesful, or false otherwise. </returns>
    Public Function CreatePropSetOnDBObject(ByVal dbobjId As ObjectId, ByVal propsetdefId As ObjectId) As Boolean
        Dim db As Database = HostApplicationServices.WorkingDatabase
        Dim tm As AcadDb.TransactionManager = db.TransactionManager
        Dim trans As Transaction = tm.StartTransaction()

        Try
            Dim dbobj As AcadDb.DBObject = tm.GetObject(dbobjId, OpenMode.ForWrite, False, False)
            AecPropDb.PropertyDataServices.AddPropertySet(dbobj, propsetdefId)
        Catch
            Return False ' failure
        End Try

        trans.Commit()
        trans.Dispose()

        Return True
    End Function
#End Region

#Region "CreatePropertySetDefinitionWithAutoData"
    ' <summary>
    ' Creates a new property set and adds all the available automatic source names as automatic data.
    ' </summary>
    ' <param name="propertySetDefName">The property set definition name to create.</param>
    ' <param name="className"> The class name used for the applies-to filter and the available source names. </param>
    ' <returns> true is succesful, or false otherwise. </returns>
    Public Function CreatePropertySetDefinitionWithAutoData(ByVal propertySetDefName As String, ByVal className As String) As Boolean
        Dim db As Database = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Database
        Dim tm As AcadDb.TransactionManager = db.TransactionManager
        Dim tx As AcadDb.Transaction = tm.StartTransaction()

        Try
            Dim setDef As AecPropDb.PropertySetDefinition = New AecPropDb.PropertySetDefinition()
            setDef.SetToStandard(db)
            setDef.SubSetDatabaseDefaults(db)
            Dim filters As System.Collections.Specialized.StringCollection = New System.Collections.Specialized.StringCollection()
            filters.Add(className)
            setDef.SetAppliesToFilter(filters, False)

            Dim defs As AecPropDb.PropertyDefinitionCollection = setDef.Definitions

            Dim sourceNames() As System.String = AecPropDb.PropertyDataServices.FindAutomaticSourceNames(className, db)
            Dim sourceName As String
            For Each sourceName In sourceNames
                Dim testAutoDef As AecPropDb.PropertyDefinition = New AecPropDb.PropertyDefinition()
                testAutoDef.SetToStandard(db)
                testAutoDef.SubSetDatabaseDefaults(db)
                testAutoDef.Name = "AutoSample " + sourceName
                testAutoDef.Automatic = True
                testAutoDef.Description = "Automatic data test:" + sourceName
                testAutoDef.IsVisible = True
                testAutoDef.IsReadOnly = True
                testAutoDef.SetAutomaticData(className, sourceName)
                defs.Add(testAutoDef)
            Next

            Dim propDefs As AecPropDb.DictionaryPropertySetDefinitions = New AecPropDb.DictionaryPropertySetDefinitions(db)
            propDefs.AddNewRecord(propertySetDefName, setDef)
            tm.AddNewlyCreatedDBObject(setDef, True)
        Catch
            Return False
        End Try

        tx.Commit()
        tx.Dispose()

        Return True
    End Function
#End Region

#Region "CreatePropertySetDefinitionWithAutoDataDup"
    ' <summary>
    ' Creates a new property set and adds the COMMON automatic source names as automatic data.
    ' </summary>
    ' <param name="propertySetDefName">The property set definition name to create.</param>
    ' <param name="className"> Input two class names which are used for the applies-to filter and the available common source names. </param>
    ' <returns> true is succesful, or false otherwise. </returns>
    Public Function CreatePropertySetDefinitionWithAutoDataDup(ByVal propertySetDefName As String, ByVal classNames As System.Collections.Specialized.StringCollection) As Boolean
        Dim db As Database = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Database
        Dim tm As AcadDb.TransactionManager = db.TransactionManager
        Dim tx As AcadDb.Transaction = tm.StartTransaction()

        Try
            Dim setDef As AecPropDb.PropertySetDefinition = New AecPropDb.PropertySetDefinition()
            setDef.SetToStandard(db)
            setDef.SubSetDatabaseDefaults(db)
            Dim filters As System.Collections.Specialized.StringCollection = New System.Collections.Specialized.StringCollection()
            Dim className As String
            For Each className In classNames
                filters.Add(className)
            Next
            setDef.SetAppliesToFilter(filters, False)

            Dim defs As AecPropDb.PropertyDefinitionCollection = setDef.Definitions

            Dim dupes As System.Collections.Specialized.StringCollection = New System.Collections.Specialized.StringCollection()
            Dim compare() As System.Collections.Specialized.StringCollection = New System.Collections.Specialized.StringCollection(classNames.Count) {}
            Dim i As Integer = 0
            Dim bFirst As Boolean = True

            Dim sourceName As String
            For Each className In classNames
                Dim sourceNames() As System.String = AecPropDb.PropertyDataServices.FindAutomaticSourceNames(className, db)
                compare(i) = New System.Collections.Specialized.StringCollection()
                compare(i).AddRange(sourceNames)

                If Not bFirst Then
                    For Each sourceName In sourceNames
                        If compare(i - 1).Contains(sourceName) Then
                            dupes.Add(sourceName)
                        End If
                    Next
                End If
                bFirst = False
                i = i + 1
            Next

            For Each sourceName In dupes
                Dim testAutoDef As AecPropDb.PropertyDefinition = New AecPropDb.PropertyDefinition()
                testAutoDef.SetToStandard(db)
                testAutoDef.SubSetDatabaseDefaults(db)
                testAutoDef.Name = sourceName
                testAutoDef.Automatic = True
                testAutoDef.Description = "Automatic data test"
                testAutoDef.IsVisible = True
                testAutoDef.IsReadOnly = True
                testAutoDef.SetAutomaticData(classNames(0), sourceName)
                testAutoDef.SetAutomaticData(classNames(1), sourceName)
                defs.Add(testAutoDef)

            Next

            Dim propDefs As AecPropDb.DictionaryPropertySetDefinitions = New AecPropDb.DictionaryPropertySetDefinitions(db)
            propDefs.AddNewRecord(propertySetDefName, setDef)
            tm.AddNewlyCreatedDBObject(setDef, True)
        Catch
            Return False
        End Try

        tx.Commit()
        tx.Dispose()

        Return True
    End Function
#End Region

#Region "GetScheduleDataProperty"
    ' <summary>
    ' 
    ' </summary>
    ' <param name="obj">The object. </param>
    ' <param name="blockReferencePath">The block reference path.</param>
    ' <param name="setName">The set name.</param>
    ' <param name="propertyName">The property name.</param>
    ' <returns></returns>
    Private Function GetScheduleDataProperty(ByVal obj As Autodesk.AutoCAD.DatabaseServices.DBObject, ByVal blockReferencePath As ObjectIdCollection, ByVal setName As String, ByVal propertyName As String) As Object
        Dim fullPropertyName As String = setName + ":" + propertyName
        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor
        Dim db As Database = obj.Database

        Dim tm As DBTransactionManager = db.TransactionManager
        Dim trans As Transaction = tm.StartTransaction()
        Dim result As PropertyValueUnitPair = Nothing

        Try
            Dim propertySetDictionary As DictionaryPropertySetDefinitions = New DictionaryPropertySetDefinitions(db)
            Dim dictId As ObjectId = propertySetDictionary.DictionaryId
            Dim dictDefinitions As Autodesk.AutoCAD.DatabaseServices.DBDictionary = trans.GetObject(dictId, OpenMode.ForRead)

            For Each entry As DBDictionaryEntry In dictDefinitions
                If setName = entry.Key Then
                    Dim propId As ObjectId = entry.Value
                    Dim nId As Int32 = 0
                    Dim existId As ObjectId = PropertyDataServices.PropertyExists(db, fullPropertyName, nId)

                    result = PropertyDataServices.GetPropertyValueUnitExt(obj, blockReferencePath, existId, nId)
                    Exit For
                End If
            Next

        Catch e As System.Exception
            ed.WriteMessage(e.Message)
            trans.Abort()
        Finally
            trans.Dispose()
        End Try

        If Nothing <> result Then
            Return result.Value
        Else
            Return Nothing
        End If
    End Function
#End Region

#Region "ShowPropertyData"
    ' <summary>
    ' 
    ' </summary>
    ' <param name="objId">The object id.</param>
    ' <param name="blockReferencePath">The block reference path.</param>
    Private Sub ShowPropertyData(ByVal objId As ObjectId, ByVal blockReferencePath As ObjectIdCollection, ByVal propertySetName As String, ByVal propertyName As String)

        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor
        Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database

        Dim tm As DBTransactionManager = db.TransactionManager
        Dim trans As Transaction = tm.StartTransaction()

        Try
            Dim geoObject As Autodesk.AutoCAD.DatabaseServices.DBObject = trans.GetObject(objId, OpenMode.ForWrite)

            Dim ret As Object = GetScheduleDataProperty(geoObject, blockReferencePath, propertySetName, propertyName)
            Dim retString As String = PropertyDataServices.DataToString(ret)
            ed.WriteMessage("Found Property: " + propertyName + " = " + retString & vbCrLf)
        Catch e As System.Exception
            ed.WriteMessage(e.Message)
            trans.Abort()
        Finally
            trans.Dispose()
        End Try
    End Sub
#End Region
    'End Implementations
#End Region

End Class
