#Region " Autodesk Copyright "
'      .NET Sample
'
'      Copyright (c) 2007 by Autodesk, Inc.
'
'      Permission to use, copy, modify, and distribute this software
'      for any purpose and without fee is hereby granted, provided
'      that the above copyright notice appears in all copies and
'      that both that copyright notice and the limited warranty and
'      restricted rights notice below appear in all supporting
'      documentation.
'
'      AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
'      AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
'      MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
'      DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
'      UNINTERRUPTED OR ERROR FREE.
'
'      Use, duplication, or disclosure by the U.S. Government is subject to
'      restrictions set forth in FAR 52.227-19 (Commercial Computer
'      Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
'      (Rights in Technical Data and Computer Software), as applicable.
'
#End Region

#Region " References "

Imports System
Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Diagnostics
Imports System.Drawing
Imports System.Reflection
Imports System.Text
Imports System.Windows.Forms
Imports Autodesk.Aec.Building.ApplicationServices
Imports Autodesk.Aec.Building.DatabaseServices

#End Region

Namespace CatalogExplorer

    Class PropertyUtils

        Public Shared Function Compare(ByVal a As Object, ByVal b As Object) As Integer

            Dim piA As PropertyInfo = DirectCast(a, PropertyInfo)
            Dim piB As PropertyInfo = DirectCast(b, PropertyInfo)

            Return piA.Name.CompareTo(piB.Name)

        End Function

        Public Shared Function AllowProperty(ByVal propInfo As PropertyInfo, ByVal type As System.Type) As Boolean

            Dim bAllow As Boolean = True

            If propInfo.Name.StartsWith("Value") OrElse propInfo.Name.StartsWith("Allow") OrElse propInfo.DeclaringType.FullName <> type.FullName Then
                bAllow = False
            End If
            Return bAllow

        End Function

    End Class

    Class DataFieldUtils

        Public Shared Function DataFieldValueToString(ByVal df As DataField) As String

            Dim strReturn As String

            Select Case df.Type
                Case CatalogDataType.Bool
                    strReturn = df.ValueBoolean.ToString()

                Case CatalogDataType.Double
                    strReturn = df.ValueDouble.ToString()

                Case CatalogDataType.Int
                    strReturn = df.ValueInteger.ToString()

                Case CatalogDataType.String
                    strReturn = df.ValueString.ToString()

                Case Else
                    strReturn = df.Type.ToString()
            End Select

            Return strReturn

        End Function

    End Class

    Class UiUitls

        Public Shared Sub ResizeColumnHeaders(ByVal lv As ListView)

            For Each ch As ColumnHeader In lv.Columns

                ch.AutoResize(ColumnHeaderAutoResizeStyle.ColumnContent)
                Dim columnWidth As Integer = ch.Width

                ch.AutoResize(ColumnHeaderAutoResizeStyle.HeaderSize)
                Dim headerWidth As Integer = ch.Width

                If columnWidth > headerWidth Then
                    ch.AutoResize(ColumnHeaderAutoResizeStyle.ColumnContent)
                End If

            Next

        End Sub


        Public Shared Sub PositionDialog(ByVal dlg As Form, ByVal startPoint As Point)

            Dim ptDialogLeft As Point = startPoint
            Dim newLeft As Integer = ptDialogLeft.X - dlg.Width

            If newLeft < 0 Then
                newLeft = 0
            End If

            dlg.Left = newLeft
            dlg.Top = ptDialogLeft.Y

            If dlg.Top < 0 OrElse dlg.Top > System.Windows.Forms.Screen.PrimaryScreen.Bounds.Bottom Then
                dlg.Top = System.Windows.Forms.Screen.PrimaryScreen.Bounds.Top
            End If

            Dim heightPos As Integer = dlg.Top + dlg.Height

            If heightPos > System.Windows.Forms.Screen.PrimaryScreen.Bounds.Bottom Then
                dlg.Top = Math.Abs(dlg.Top - dlg.Height)
            End If

        End Sub

    End Class

End Namespace

