#Region " Autodesk Copyright "
'      .NET Sample
'
'      Copyright (c) 2007 by Autodesk, Inc.
'
'      Permission to use, copy, modify, and distribute this software
'      for any purpose and without fee is hereby granted, provided
'      that the above copyright notice appears in all copies and
'      that both that copyright notice and the limited warranty and
'      restricted rights notice below appear in all supporting
'      documentation.
'
'      AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
'      AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
'      MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
'      DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
'      UNINTERRUPTED OR ERROR FREE.
'
'      Use, duplication, or disclosure by the U.S. Government is subject to
'      restrictions set forth in FAR 52.227-19 (Commercial Computer
'      Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
'      (Rights in Technical Data and Computer Software), as applicable.
'
#End Region

#Region " References "
Imports System
Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Data
Imports System.Drawing
Imports System.Text
Imports System.Windows.Forms
Imports Autodesk.Aec.Building.ApplicationServices
Imports Autodesk.Aec.Building.DatabaseServices
Imports Autodesk.Aec.Building.ApplicationServices.CatalogManager
Imports Autodesk.Aec.Building.ApplicationServices.PartManager
Imports System.Diagnostics
Imports System.Reflection
Imports System.IO
#End Region

Namespace CatalogExplorer
    Public Class CatalogExplorerDlg
        Inherits Form

        Private mProps As PropertyInfo()
        Private mExandedTable As DataExpandedTable
        Private mDataFields As DataFieldCollection
        Private mFirstPart As TreeNode
        Private mCatalog As CatalogManager

        Const ICON_SIZE = 16

        Public Sub New()

            Try

                mCatalog = New CatalogManager

                InitializeComponent()

            Catch ex As System.Exception

                MessageBox.Show(ex.Message)

            End Try

        End Sub

        Private Sub CatalogExplorerDlg_Load(ByVal sender As Object, ByVal e As EventArgs) Handles MyBase.Load

            LoadDomains()

        End Sub


        Private Sub LoadDomains()

            Try
                Dim ParentNode As TreeNode = New TreeNode("MEP Catalog", 0, 0)

                ParentNode.ImageIndex = 0
                ParentNode.SelectedImageIndex = 2

                tvCatalog.ImageList = imageList

                ' Get the list of domains from the catalog
                '
                Dim domains As Domain() = mCatalog.DomainList

                ' Loop through all the domain and get the local name 
                '
                For Each domain As Domain In domains
                    Dim formatter As EnumStringFormatter = New EnumStringFormatter
                    Dim strValue As String = String.Format(formatter, "{0:CatL}", domain)
                    Dim curNode As TreeNode = ParentNode.Nodes.Add(strValue, domain.ToString, 0, 0)
                    curNode.Tag = domain
                Next

                tvCatalog.Nodes.Add(ParentNode)

                FillTree(ParentNode)

                tvCatalog.SelectedNode = mFirstPart

            Catch ex As System.Exception

                MessageBox.Show(ex.Message)

            End Try

        End Sub


        Private Sub FillTree(ByVal parentNode As TreeNode)

            ' loop through each node and extract the domain from the tag
            ' then call AddParts for the domain
            '
            For Each domainNode As TreeNode In parentNode.Nodes
                AddParts(domainNode)
            Next

        End Sub


        Private Sub AddParts(ByVal domainNode As TreeNode)

            Try
                Dim query As DataQuery = New DataQuery
                query.Domain = CType(domainNode.Tag, Domain)

                Dim partlist As DataPartsList = mCatalog.GetPartsList(query)
                Dim parts As DataPartLookupCollection = partlist.DataParts

                Dim curNode As TreeNode = domainNode

                For Each partlookup As DataPartLookup In parts
                    curNode = domainNode
                    curNode = AddPaths(curNode, partlookup.Path)
                    Dim partNode As TreeNode = curNode.Nodes.Add(partlookup.Name)
                    If mFirstPart Is Nothing Then
                        mFirstPart = partNode
                    End If
                    partNode.Tag = partlookup
                Next

            Catch ex As System.Exception

                MessageBox.Show(ex.Message)

            End Try

        End Sub


        Function AddPaths(ByVal curNode As TreeNode, ByVal partPath As String) As TreeNode

            Try
                ' split the path into the folders and files
                '
                Dim pathParts As String() = partPath.Split("\"c)

                ' make sure length -1, dont add the actual xml file.
                '
                Dim i As Integer = 0
                While i < pathParts.Length - 1
                    If Not curNode.Nodes.ContainsKey(pathParts(i)) Then
                        curNode = curNode.Nodes.Add(pathParts(i), pathParts(i))
                    Else
                        Dim nodes As TreeNode() = curNode.Nodes.Find(pathParts(i), True)
                        curNode = nodes(0)
                    End If
                    i += 1
                End While

                Return curNode

            Catch ex As System.Exception

                MessageBox.Show(ex.Message)

                Return Nothing

            End Try

        End Function


        Private Sub tvCatalog_AfterSelect(ByVal sender As Object, ByVal e As TreeViewEventArgs) Handles tvCatalog.AfterSelect

            Try
                Dim curNode As TreeNode = e.Node

                If curNode.Nodes.Count = 0 Then

                    Dim dpl As DataPartLookup = CType(curNode.Tag, DataPartLookup)

                    Dim query As DataQuery = New DataQuery
                    query.Domain = dpl.Domain
                    query.PartGuid = dpl.Guid
                    ' only need the first record
                    mExandedTable = PartManager.GetPartTable(query, 1)
                    mDataFields = mExandedTable.DataRecords(0).DataFields
                    listView1.BeginUpdate()
                    listView1.Clear()
                    LoadPartInformation(dpl)
                    listView1.EndUpdate()

                End If

            Catch ex As System.Exception

                MessageBox.Show(ex.Message)

            End Try

        End Sub


        Public Sub LoadPartInformation(ByVal dplItem As DataPartLookup)

            Try

                ToolStripStatusLabel1.Text = "Description: " + dplItem.Description

                ToolStripStatusLabel2.Text = "Name: " + dplItem.Name

                Dim query As DataQuery = New DataQuery
                query.Domain = dplItem.Domain
                query.PartGuid = dplItem.Guid
                mExandedTable = PartManager.GetPartTable(query, 10)

                Dim partImg As Bitmap = New Bitmap(mExandedTable.ImageUrl)
                pbImage.Image = partImg

                LoadListView(mExandedTable.DataRecords(0).DataFields)

                Dim type As System.Type = dplItem.GetType

                Dim props As PropertyInfo() = type.GetProperties

                lvPartDetails.Items.Clear()
                lvPartDetails.Columns.Clear()
                lvPartDetails.Columns.Add("Name")
                lvPartDetails.Columns.Add("Value")

                Array.Reverse(props)

                Dim lvi As ListViewItem

                For Each pi As PropertyInfo In props

                    ' Check if this is a property we want to see
                    '
                    If PropertyUtils.AllowProperty(pi, type) Then

                        lvi = lvPartDetails.Items.Add(pi.Name)
                        lvi.SubItems.Add(pi.GetValue(dplItem, Nothing).ToString)

                    End If

                Next

                lvi = lvPartDetails.Items.Add("Connectors")
                lvi.SubItems.Add(mExandedTable.ConnectionCount.ToString)

                mExandedTable.ConnectionCount.ToString()

                lvi = lvPartDetails.Items.Add("Maximum number of part sizes")
                lvi.SubItems.Add(mExandedTable.DataRecords.FullyExpandedRecordCount.ToString)

                UiUitls.ResizeColumnHeaders(lvPartDetails)

            Catch ex As System.Exception

                MessageBox.Show(ex.Message)

            End Try

        End Sub

        Private Sub LoadListView(ByVal dfc As DataFieldCollection)

            Try
                ' use reflection to get the properties of the object
                ' only need the first as the are all the same
                '
                mProps = dfc(0).GetType.GetProperties

                ' Sort the properties
                '
                Array.Reverse(mProps)

                ' Fill in the column headers
                '
                LoadColumnHeaders()

                ' now load all the datfields
                '
                LoadDataFields()

                UiUitls.ResizeColumnHeaders(listView1)

                ' now we will iterate throught the column and Autosize the columns
                '
                For Each ch As ColumnHeader In listView1.Columns

                    ch.AutoResize(ColumnHeaderAutoResizeStyle.ColumnContent)
                    Dim columnWidth As Integer = ch.Width

                    ch.AutoResize(ColumnHeaderAutoResizeStyle.HeaderSize)
                    Dim headerWidth As Integer = ch.Width

                    If columnWidth > headerWidth Then
                        ch.AutoResize(ColumnHeaderAutoResizeStyle.ColumnContent)
                    End If

                Next

            Catch ex As System.Exception

                MessageBox.Show(ex.Message)

            End Try

        End Sub

        Private Sub LoadColumnHeaders()

            Try
                For Each pi As PropertyInfo In mProps

                    ' Check if this is a property we want to see
                    '
                    If PropertyUtils.AllowProperty(pi, mDataFields(0).GetType) Then
                        listView1.Columns.Add(pi.Name)
                    End If

                Next

            Catch ex As System.Exception

                MessageBox.Show(ex.Message)

            End Try

        End Sub


        Private Sub LoadDataFields()

            Try
                ' loop though all public properties on the object 
                ' and get the value then call internal sets
                '        
                For Each df As DataField In mDataFields

                    Dim lvItem As ListViewItem = Nothing
                    Dim i As Integer = 0

                    While i < mProps.Length

                        If PropertyUtils.AllowProperty(mProps(i), mDataFields(0).GetType) Then

                            If lvItem Is Nothing Then

                                lvItem = listView1.Items.Add(mProps(i).GetValue(df, Nothing).ToString)
                                lvItem.Tag = df

                            Else

                                lvItem.SubItems.Add(mProps(i).GetValue(df, Nothing).ToString)

                            End If

                        End If

                        i += 1

                    End While

                Next

            Catch ex As System.Exception

                MessageBox.Show(ex.Message)

            End Try

        End Sub

        Private Sub DrawSubItem(ByVal sender As Object, ByVal e As DrawListViewSubItemEventArgs) Handles listView1.DrawSubItem

            Try

                Dim flags As TextFormatFlags = TextFormatFlags.Left
                Dim sf As StringFormat = New StringFormat

                Try
                    ' If this a constList of Basic Table, display the icon
                    '
                    If e.Header.Text = "RawDataSource" Then

                        If e.SubItem.Text = "ConstantLists" OrElse e.SubItem.Text = "BasicTable" Then

                            Dim ico As Image = imageList.Images(3)
                            Dim icoRect As Rectangle = e.Bounds
                            icoRect.X = icoRect.Right - ICON_SIZE
                            icoRect.Width = ICON_SIZE
                            e.Graphics.DrawImage(ico, icoRect)

                        End If

                    End If

                    e.DrawText(flags)

                Catch

                Finally

                    CType(sf, IDisposable).Dispose()
                End Try

            Catch ex As System.Exception

                MessageBox.Show(ex.Message)

            End Try

        End Sub

        Private Sub DrawColumnHeader(ByVal sender As Object, ByVal e As DrawListViewColumnHeaderEventArgs) Handles listView1.DrawColumnHeader
            e.DrawDefault = True
        End Sub

        Private Sub listView1_MouseDoubleClick(ByVal sender As Object, ByVal e As MouseEventArgs) Handles listView1.MouseDoubleClick

            Try
                Dim info As ListViewHitTestInfo = listView1.HitTest(e.X, e.Y)

                Dim df As DataField = CType(info.Item.Tag, DataField)

                If info.SubItem.Text = "ConstantLists" Then

                    Dim icoRect As Rectangle = info.SubItem.Bounds
                    icoRect.X = icoRect.Right - ICON_SIZE
                    icoRect.Width = ICON_SIZE

                    If icoRect.Left < e.X AndAlso icoRect.Right > e.X AndAlso icoRect.Top < e.Y AndAlso icoRect.Bottom > e.Y Then

                        Dim curNode As TreeNode = tvCatalog.SelectedNode
                        Dim dpl As DataPartLookup = CType(curNode.Tag, DataPartLookup)
                        Dim query As DataQuery = New DataQuery
                        query.Domain = dpl.Domain
                        query.PartGuid = dpl.Guid

                        Dim rawTable As DataRawTable = PartManager.GetPartRawTable(query)

                        Dim dlc As DataListCollection = rawTable.ConstListSet

                        Dim dl As DataList = rawTable.FindConstListByContext(df.Context)
                        Dim dlg As ItemViewer = New ItemViewer(dl)
                        Dim r As Rectangle = listView1.RectangleToScreen(icoRect)
                        UiUitls.PositionDialog(dlg, New Point(r.Right, r.Bottom))

                        dlg.ShowDialog()

                    End If

                Else

                    If info.SubItem.Text = "BasicTable" Then

                        Dim icoRect As Rectangle = info.SubItem.Bounds
                        icoRect.X = icoRect.Right - ICON_SIZE
                        icoRect.Width = ICON_SIZE

                        If icoRect.Left < e.X AndAlso icoRect.Right > e.X AndAlso icoRect.Top < e.Y AndAlso icoRect.Bottom > e.Y Then

                            Dim curNode As TreeNode = tvCatalog.SelectedNode
                            Dim dpl As DataPartLookup = CType(curNode.Tag, DataPartLookup)
                            Dim query As DataQuery = New DataQuery
                            query.Domain = dpl.Domain
                            query.PartGuid = dpl.Guid

                            Dim rawTable As DataRawTable = PartManager.GetPartRawTable(query)
                            Dim dlg As ItemViewer = New ItemViewer(rawTable)

                            Dim r As Rectangle = listView1.RectangleToScreen(icoRect)
                            UiUitls.PositionDialog(dlg, New Point(r.Right, r.Bottom))
                            dlg.ShowDialog()

                        End If

                    Else

                        MessageBox.Show(DataFieldUtils.DataFieldValueToString(df))

                    End If

                End If

            Catch ex As System.Exception

                MessageBox.Show(ex.Message)

            End Try

        End Sub

        Private Sub InitializeComponent()
            Me.components = New System.ComponentModel.Container
            Dim resources As System.ComponentModel.ComponentResourceManager = New System.ComponentModel.ComponentResourceManager(GetType(CatalogExplorerDlg))
            Me.listView1 = New System.Windows.Forms.ListView
            Me.imageList = New System.Windows.Forms.ImageList(Me.components)
            Me.statusStrip1 = New System.Windows.Forms.StatusStrip
            Me.lvPartDetails = New System.Windows.Forms.ListView
            Me.splitContainer1 = New System.Windows.Forms.SplitContainer
            Me.tvCatalog = New System.Windows.Forms.TreeView
            Me.pbImage = New System.Windows.Forms.PictureBox
            Me.ToolStripStatusLabel1 = New System.Windows.Forms.ToolStripStatusLabel
            Me.ToolStripStatusLabel2 = New System.Windows.Forms.ToolStripStatusLabel
            Me.statusStrip1.SuspendLayout()
            Me.splitContainer1.Panel1.SuspendLayout()
            Me.splitContainer1.Panel2.SuspendLayout()
            Me.splitContainer1.SuspendLayout()
            CType(Me.pbImage, System.ComponentModel.ISupportInitialize).BeginInit()
            Me.SuspendLayout()
            '
            'listView1
            '
            Me.listView1.Anchor = CType((((System.Windows.Forms.AnchorStyles.Top Or System.Windows.Forms.AnchorStyles.Bottom) _
                        Or System.Windows.Forms.AnchorStyles.Left) _
                        Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
            Me.listView1.FullRowSelect = True
            Me.listView1.GridLines = True
            Me.listView1.HideSelection = False
            Me.listView1.Location = New System.Drawing.Point(3, 230)
            Me.listView1.MultiSelect = False
            Me.listView1.Name = "listView1"
            Me.listView1.OwnerDraw = True
            Me.listView1.ShowItemToolTips = True
            Me.listView1.Size = New System.Drawing.Size(546, 388)
            Me.listView1.TabIndex = 2
            Me.listView1.UseCompatibleStateImageBehavior = False
            Me.listView1.View = System.Windows.Forms.View.Details
            '
            'imageList
            '
            Me.imageList.ImageStream = CType(resources.GetObject("imageList.ImageStream"), System.Windows.Forms.ImageListStreamer)
            Me.imageList.TransparentColor = System.Drawing.Color.Transparent
            Me.imageList.Images.SetKeyName(0, "AecbCatalogEditorRC.ico")
            Me.imageList.Images.SetKeyName(1, "VSFolder_open.bmp")
            Me.imageList.Images.SetKeyName(2, "VSFolder_closed.bmp")
            Me.imageList.Images.SetKeyName(3, "search.ico")
            '
            'statusStrip1
            '
            Me.statusStrip1.Items.AddRange(New System.Windows.Forms.ToolStripItem() {Me.ToolStripStatusLabel1, Me.ToolStripStatusLabel2})
            Me.statusStrip1.Location = New System.Drawing.Point(0, 621)
            Me.statusStrip1.Name = "statusStrip1"
            Me.statusStrip1.Size = New System.Drawing.Size(822, 22)
            Me.statusStrip1.TabIndex = 5
            Me.statusStrip1.Text = "statusStrip1"
            '
            'lvPartDetails
            '
            Me.lvPartDetails.Anchor = CType(((System.Windows.Forms.AnchorStyles.Top Or System.Windows.Forms.AnchorStyles.Left) _
                        Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
            Me.lvPartDetails.FullRowSelect = True
            Me.lvPartDetails.GridLines = True
            Me.lvPartDetails.HeaderStyle = System.Windows.Forms.ColumnHeaderStyle.Nonclickable
            Me.lvPartDetails.Location = New System.Drawing.Point(225, 8)
            Me.lvPartDetails.Name = "lvPartDetails"
            Me.lvPartDetails.Size = New System.Drawing.Size(327, 216)
            Me.lvPartDetails.TabIndex = 27
            Me.lvPartDetails.UseCompatibleStateImageBehavior = False
            Me.lvPartDetails.View = System.Windows.Forms.View.Details
            '
            'splitContainer1
            '
            Me.splitContainer1.Dock = System.Windows.Forms.DockStyle.Fill
            Me.splitContainer1.Location = New System.Drawing.Point(0, 0)
            Me.splitContainer1.Name = "splitContainer1"
            '
            'splitContainer1.Panel1
            '
            Me.splitContainer1.Panel1.Controls.Add(Me.tvCatalog)
            '
            'splitContainer1.Panel2
            '
            Me.splitContainer1.Panel2.Controls.Add(Me.lvPartDetails)
            Me.splitContainer1.Panel2.Controls.Add(Me.pbImage)
            Me.splitContainer1.Panel2.Controls.Add(Me.listView1)
            Me.splitContainer1.Size = New System.Drawing.Size(822, 643)
            Me.splitContainer1.SplitterDistance = 266
            Me.splitContainer1.TabIndex = 4
            '
            'tvCatalog
            '
            Me.tvCatalog.Dock = System.Windows.Forms.DockStyle.Fill
            Me.tvCatalog.HideSelection = False
            Me.tvCatalog.HotTracking = True
            Me.tvCatalog.Location = New System.Drawing.Point(0, 0)
            Me.tvCatalog.Name = "tvCatalog"
            Me.tvCatalog.Size = New System.Drawing.Size(266, 643)
            Me.tvCatalog.TabIndex = 1
            '
            'pbImage
            '
            Me.pbImage.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D
            Me.pbImage.Location = New System.Drawing.Point(3, 8)
            Me.pbImage.Name = "pbImage"
            Me.pbImage.Size = New System.Drawing.Size(216, 216)
            Me.pbImage.TabIndex = 17
            Me.pbImage.TabStop = False
            '
            'ToolStripStatusLabel1
            '
            Me.ToolStripStatusLabel1.BorderSides = CType((((System.Windows.Forms.ToolStripStatusLabelBorderSides.Left Or System.Windows.Forms.ToolStripStatusLabelBorderSides.Top) _
                        Or System.Windows.Forms.ToolStripStatusLabelBorderSides.Right) _
                        Or System.Windows.Forms.ToolStripStatusLabelBorderSides.Bottom), System.Windows.Forms.ToolStripStatusLabelBorderSides)
            Me.ToolStripStatusLabel1.BorderStyle = System.Windows.Forms.Border3DStyle.SunkenInner
            Me.ToolStripStatusLabel1.DisplayStyle = System.Windows.Forms.ToolStripItemDisplayStyle.Text
            Me.ToolStripStatusLabel1.Name = "ToolStripStatusLabel1"
            Me.ToolStripStatusLabel1.Size = New System.Drawing.Size(115, 17)
            Me.ToolStripStatusLabel1.Text = "ToolStripStatusLabel1"
            '
            'ToolStripStatusLabel2
            '
            Me.ToolStripStatusLabel2.BorderSides = CType((((System.Windows.Forms.ToolStripStatusLabelBorderSides.Left Or System.Windows.Forms.ToolStripStatusLabelBorderSides.Top) _
                        Or System.Windows.Forms.ToolStripStatusLabelBorderSides.Right) _
                        Or System.Windows.Forms.ToolStripStatusLabelBorderSides.Bottom), System.Windows.Forms.ToolStripStatusLabelBorderSides)
            Me.ToolStripStatusLabel2.Name = "ToolStripStatusLabel2"
            Me.ToolStripStatusLabel2.Size = New System.Drawing.Size(115, 17)
            Me.ToolStripStatusLabel2.Text = "ToolStripStatusLabel2"
            '
            'CatalogExplorerDlg
            '
            Me.ClientSize = New System.Drawing.Size(822, 643)
            Me.Controls.Add(Me.statusStrip1)
            Me.Controls.Add(Me.splitContainer1)
            Me.Name = "CatalogExplorerDlg"
            Me.Text = "Catalog Explorer"
            Me.statusStrip1.ResumeLayout(False)
            Me.statusStrip1.PerformLayout()
            Me.splitContainer1.Panel1.ResumeLayout(False)
            Me.splitContainer1.Panel2.ResumeLayout(False)
            Me.splitContainer1.ResumeLayout(False)
            CType(Me.pbImage, System.ComponentModel.ISupportInitialize).EndInit()
            Me.ResumeLayout(False)
            Me.PerformLayout()

        End Sub

        Private WithEvents listView1 As System.Windows.Forms.ListView
        Private WithEvents imageList As System.Windows.Forms.ImageList
        Private components As System.ComponentModel.IContainer
        Private WithEvents statusStrip1 As System.Windows.Forms.StatusStrip
        Private WithEvents lvPartDetails As System.Windows.Forms.ListView
        Private WithEvents splitContainer1 As System.Windows.Forms.SplitContainer
        Private WithEvents tvCatalog As System.Windows.Forms.TreeView
        Private WithEvents pbImage As System.Windows.Forms.PictureBox
        Friend WithEvents ToolStripStatusLabel1 As System.Windows.Forms.ToolStripStatusLabel
        Friend WithEvents ToolStripStatusLabel2 As System.Windows.Forms.ToolStripStatusLabel

    End Class

End Namespace

