#Region " Autodesk Copyright"
'      .NET Sample
'
'      Copyright (c) 2007 by Autodesk, Inc.
'
'      Permission to use, copy, modify, and distribute this software
'      for any purpose and without fee is hereby granted, provided
'      that the above copyright notice appears in all copies and
'      that both that copyright notice and the limited warranty and
'      restricted rights notice below appear in all supporting
'      documentation.
'
'      AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
'      AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
'      MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
'      DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
'      UNINTERRUPTED OR ERROR FREE.
'
'      Use, duplication, or disclosure by the U.S. Government is subject to
'      restrictions set forth in FAR 52.227-19 (Commercial Computer
'      Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
'      (Rights in Technical Data and Computer Software), as applicable.
'
#End Region

#Region " References"

Imports System
Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Data
Imports System.Drawing
Imports System.Text
Imports System.Windows.Forms
Imports System.Reflection
Imports System.Diagnostics
Imports Autodesk.Aec.Building.ApplicationServices
Imports Autodesk.Aec.Building.DatabaseServices

#End Region

Namespace CatalogExplorer

    Public Class ItemViewer
        Inherits Form

        Private mDataRecords As DataRecordCollection
        Private mDataFields As DataFieldCollection

        Public Sub New(ByVal dataList As DataList)
            InitializeComponent()
            mDataFields = dataList.DataFields
            Me.DialogTitle = dataList.ParameterContext.ToString + " - " + mDataFields.Count.ToString + " Items"
            LoadConstList()
            UiUitls.ResizeColumnHeaders(Me.listView1)
        End Sub

        Public Sub New(ByVal rawTable As DataRawTable)
            InitializeComponent()
            mDataRecords = rawTable.BasicTable
            LoadRawTable()
            UiUitls.ResizeColumnHeaders(Me.listView1)
        End Sub


        Public WriteOnly Property DialogTitle() As String
            Set(ByVal value As String)
                Me.Text = value
            End Set
        End Property


        Public Sub LoadConstList()
            listView1.Columns.Add(mDataFields(0).ContextString)

            For Each df As DataField In mDataFields
                Dim item As String = DataFieldUtils.DataFieldValueToString(df) + " " + df.Units
                listView1.Items.Add(item)
            Next
        End Sub


        Sub LoadRawTable()
            LoadTableColumns()
            LoadTable()
        End Sub


        Sub LoadTableColumns()
            Dim dataRecord As DataRecord = mDataRecords(0)
            For Each df As DataField In dataRecord.DataFields
                If df.Name = "BTRowID" Then
                    Me.DialogTitle = "Basic Table - " + df.Name + ", contains " + mDataRecords.Count.ToString() + " rows"
                Else
                    listView1.Columns.Add(df.Name)
                End If
            Next
        End Sub

        Sub LoadTable()
            For Each dr As DataRecord In mDataRecords
                Dim lvItem As ListViewItem = Nothing
                Dim i As Integer = 1
                While i < dr.DataFields.Count
                    Dim df As DataField = dr.DataFields(i)
                    If i = 1 Then
                        lvItem = listView1.Items.Add(DataFieldUtils.DataFieldValueToString(df))
                    Else
                        lvItem.SubItems.Add((DataFieldUtils.DataFieldValueToString(df)))
                    End If
                    i += 1
                End While
            Next
        End Sub
    End Class
End Namespace

