#Region " ""Autodesk Copyright"

'      .NET Sample
'
'      Copyright (c) 2008 by Autodesk, Inc.
'
'      Permission to use, copy, modify, and distribute this software
'      for any purpose and without fee is hereby granted, provided
'      that the above copyright notice appears in all copies and
'      that both that copyright notice and the limited warranty and
'      restricted rights notice below appear in all supporting
'      documentation.
'
'      AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
'      AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
'      MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
'      DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
'      UNINTERRUPTED OR ERROR FREE.
'
'      Use, duplication, or disclosure by the U.S. Government is subject to
'      restrictions set forth in FAR 52.227-19 (Commercial Computer
'      Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
'      (Rights in Technical Data and Computer Software), as applicable.
'

#End Region

#Region "Namespaces References"

Imports System
Imports System.Collections.Generic
Imports System.Text

' AutoCAD MEP Specific
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.GeomeTry
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.EditorInput
Imports Autodesk.Aec.DatabaseServices
Imports Autodesk.Aec.Building.ApplicationServices
Imports Autodesk.Aec.Building.DatabaseServices

' create an alias to the AutoCAD Application object
Imports AcadApplication = Autodesk.AutoCAD.ApplicationServices.Application

#End Region

<Assembly: Autodesk.AutoCAD.Runtime.ExtensionApplication(GetType(GetEngineeringData))> 
<Assembly: Autodesk.AutoCAD.Runtime.CommandClass(GetType(GetEngineeringData))> 

Public Class GetEngineeringData
    Implements IExtensionApplication

#Region "IExtensionApplication"
    Public Sub Initialize() Implements IExtensionApplication.Initialize
        Dim ed As Editor = AcadApplication.DocumentManager.MdiActiveDocument.Editor
        ed.WriteMessage("Loading GetEngineeringData..." & vbCrLf)
    End Sub

    Public Sub Terminate() Implements IExtensionApplication.Terminate
    End Sub
#End Region

    ' <summary>
    ' A simple example to demonstrate how to get the engineering data by the context and index
    ' </summary>
    <Autodesk.AutoCAD.Runtime.CommandMethod("GetEngineeringData", "GetEngineeringData", Autodesk.AutoCAD.Runtime.CommandFlags.Modal)> _
    Public Sub NewCmd()

        Dim editor As Editor = AcadApplication.DocumentManager.MdiActiveDocument.Editor
        Dim db As Database = AcadApplication.DocumentManager.MdiActiveDocument.Database
        Dim transMgr As Autodesk.AutoCAD.DatabaseServices.TransactionManager = db.TransactionManager

        ' set up the entity prompt options
        Dim prOptions As PromptEntityOptions = New PromptEntityOptions(vbNewLine + "Select an MEP object: ")
        prOptions.SetRejectMessage(vbNewLine + "Object must an MEP object.")
        prOptions.AddAllowedClass(GetType(Member), False)

        Dim trans As Transaction = Nothing

        Try
            ' prompt the user to select an object
            Dim prResult As PromptEntityResult = editor.GetEntity(prOptions)

            If prResult.Status = PromptStatus.Cancel Then
                Return
            End If

            ' start a new transaction
            trans = transMgr.StartTransaction()

            ' get the ObjectId from the selected Duct 
            Dim oidSelected As Autodesk.AutoCAD.DatabaseServices.ObjectId = prResult.ObjectId

            ' open the Member for Read based on the objectId
            Dim member As Member = CType(transMgr.GetObject(oidSelected, OpenMode.ForRead, False), Member)

            '  now get the engineering data from the object
            If Not member Is Nothing Then
                Dim Newrecord As DataRecord = PartManager.GetEngineeringData(member)
                ' Get the engineering data by its context and index. The context are defined 
                ' in AecbContextConfig.xaml. You may also find the data by name. However, the 
                ' name may be changed in different cultures and languages, while the context
                ' is fixed with the application.
                Dim engField As DataField = Newrecord.DataFields.FindByContextAndIndex(Context.EngineeringPartId, 1)
                Dim str As String = engField.ValueString
                editor.WriteMessage(vbNewLine + "The engineering data: " + str)
            End If
        Catch be As Autodesk.Aec.Building.DatabaseServices.BuildingException
            editor.WriteMessage(be.Message)
            trans.Abort()
        Catch e As SystemException
            editor.WriteMessage(e.Message)
            trans.Abort()
        Catch ae As Autodesk.AutoCAD.Runtime.Exception
            editor.WriteMessage(ae.Message())
        Finally
            trans.Dispose()
        End Try
    End Sub
End Class
