﻿#Region " Autodesk Copyright "

'      .NET Sample
'
'      Copyright (c) 2008 by Autodesk, Inc.
'
'      Permission to use, copy, modify, and distribute this software
'      for any purpose and without fee is hereby granted, provided
'      that the above copyright notice appears in all copies and
'      that both that copyright notice and the limited warranty and
'      restricted rights notice below appear in all supporting
'      documentation.
'
'      AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
'      AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
'      MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
'      DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
'      UNINTERRUPTED OR ERROR FREE.
'
'      Use, duplication, or disclosure by the U.S. Government is subject to
'      restrictions set forth in FAR 52.227-19 (Commercial Computer
'      Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
'      (Rights in Technical Data and Computer Software), as applicable.
'

#End Region

#Region "Namespaces References"

Imports System
Imports System.Collections.Generic
Imports System.Text
Imports System.Diagnostics

' AutoCAD specific
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.Geometry
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.EditorInput
Imports Autodesk.Aec.DatabaseServices

' AutoCAD MEP Specific
Imports Autodesk.Aec.Building.ApplicationServices
Imports Autodesk.Aec.Building.DatabaseServices


' create an alias to the AutoCAD Application object
Imports AcadApplication = Autodesk.AutoCAD.ApplicationServices.Application
Imports ObjectId = Autodesk.AutoCAD.DatabaseServices.ObjectId

#End Region

<Assembly: Autodesk.AutoCAD.Runtime.ExtensionApplication(GetType(ModifyView.ModifyViewDefSample))> 
<Assembly: Autodesk.AutoCAD.Runtime.CommandClass(GetType(ModifyView.CmdModifyMvPartViewDef))> 

Namespace ModifyView
    Public Class ModifyViewDefSample
        Implements IExtensionApplication
#Region "IExtensionApplication"
        Public Sub Initialize() Implements IExtensionApplication.Initialize
            Dim ed As Editor = AcadApplication.DocumentManager.MdiActiveDocument.Editor
            ed.WriteMessage("Loading Modify View Sample...\r\n")
            ed.WriteMessage("Type ""ModifyMvPartView"" to run the sample..." + "\n")
        End Sub

        Public Sub Terminate() Implements IExtensionApplication.Terminate

        End Sub
#End Region
    End Class

    Public Class CmdModifyMvPartViewDef
        <Autodesk.AutoCAD.Runtime.CommandMethod("ModifyMvPartView", "ModifyMvPartView", Autodesk.AutoCAD.Runtime.CommandFlags.Modal)> _
        Public Sub ModifyMvPartView()
            ' Let's first select an MvPart
            Dim oidSelected As ObjectId = PromptingUtilites.PromptForObject(GetType(MultiViewPart), True)

            ' Setup a few things to work with
            Dim editor As Editor = AcadApplication.DocumentManager.MdiActiveDocument.Editor
            Dim db As Database = AcadApplication.DocumentManager.MdiActiveDocument.Database
            Dim transMgr As Autodesk.AutoCAD.DatabaseServices.TransactionManager = db.TransactionManager

            Dim trans As Transaction = Nothing
            Try
                ' Start the transaction
                '
                trans = transMgr.StartTransaction()

                ' We need to find out what the current display rep and view direction is.
                ' Instantiate the display rep manager for the database we are working with.
                '
                Dim dispRepMgr As DisplayRepresentationManager = New DisplayRepresentationManager(db)

                ' Get the display configuration id for the current view port
                '
                Dim dispConfId As ObjectId = dispRepMgr.DisplayConfigurationIdForCurrentViewport

                ' Open the display configuration
                '
                Dim dispConf As DisplayConfiguration = transMgr.GetObject(dispConfId, OpenMode.ForRead)

                ' Get the view direction
                '
                Dim viewDir As ViewDirection = dispConf.HardViewDirectionType

                ' Open the MultiViewPart
                Dim mvPart As MultiViewPart = transMgr.GetObject(oidSelected, OpenMode.ForRead)

                ' The current view port can have more than one display rep active for the object.  
                ' In this sample, we will use the first display rep in the collection.
                '
                Dim ids As Autodesk.AutoCAD.DatabaseServices.ObjectIdCollection = dispRepMgr.GetDisplayRepresentationIdsFromCurrentViewport(mvPart.GetRXClass())

                ' MultiViewParts can different view definitions setup based on display rep and/or view direction.  
                ' In this sample, we demonstrate two methods of finding the right view definition.
                ' We will first attempt to find the view definition based on the current display rep for the view port.
                ' This may not always work as some parts may have view definitions based solely on view direction.
                ' So, we will also find the right view direction based on the view direction of the display configuration.
                '
                ' To access the view definitions, we will be working with the style of the MultiViewPart.  When we make
                ' a change to the style, all MultiViewParts using this style will be affected.
                '
                Dim oidMvPartStyle As ObjectId = mvPart.StyleId
                Dim mvPartStyle As MultiViewPartStyle = transMgr.GetObject(oidMvPartStyle, OpenMode.ForRead)

                ' Let's select a block that we will use in the view definition
                '
                Dim oidBlockRef As ObjectId = PromptingUtilites.PromptForObject(GetType(Autodesk.AutoCAD.DatabaseServices.BlockReference), True)
                Dim blkRef As Autodesk.AutoCAD.DatabaseServices.BlockReference = transMgr.GetObject(oidBlockRef, OpenMode.ForRead)

                ' Get the block id and name
                '
                Dim blkId As ObjectId = blkRef.BlockTableRecord
                Dim blkName As String = blkRef.Name

                Dim bBlockReplaced As Boolean = False
                Dim viewDef As MultiViewPartViewDefinition
                For Each viewDef In mvPartStyle.MultiViewPartViewDefinitions
                    ' Iterate through the view definitions to find the one matching the current display rep.
                    ' There may be more than one view definition applicable to the current display rep.  
                    ' For the purposes of this sample, we will just change the block for the first view definition
                    ' we find that matches the current display rep.
                    '
                    If ids.Contains(viewDef.DisplayRepresentationId) Then
                        ' Upgrade the style since we are about to change view definitions that will require
                        ' write access to the style.
                        '
                        mvPartStyle.UpgradeOpen()
                        ' Set the block id
                        viewDef.BlockId = blkId
                        ' Set the block name
                        viewDef.Name = blkName
                        editor.WriteMessage("\nReplaced block in view definition.")
                        bBlockReplaced = True
                        Exit For
                    End If
                Next
                ' If no view definition is applicable for the current display rep, it is possible that the part
                ' has view definitions based on view direction instead.  Generally, these view definitions are 
                ' used for plan and 2 line display reps simultaneously.  Let's try to find one such view definition
                ' and change it's view block.
                '
                If bBlockReplaced = False Then

                    For Each viewDef In mvPartStyle.MultiViewPartViewDefinitions
                        ' Most MvParts will have view definitions setup for the 2 Line display rep that are also
                        ' used for plan.  
                        '
                        If viewDef.DisplayRepresentationName = "2 Line" And viewDef.IsViewOn(viewDir) Then
                            mvPartStyle.UpgradeOpen()
                            viewDef.BlockId = blkId
                            viewDef.Name = blkName
                            viewDef.DisplayRepresentationId = ids(0)
                            editor.WriteMessage("\nReplaced block in view definition.")
                            Exit For
                        End If
                    Next
                End If
                trans.Commit()
            Catch be As Autodesk.Aec.Building.DatabaseServices.BuildingException
                editor.WriteMessage(be.Message)
                trans.Abort()
            Catch ae As Autodesk.AutoCAD.Runtime.Exception
                editor.WriteMessage(ae.Message)
                trans.Abort()
            Catch e As SystemException
                editor.WriteMessage(e.Message)
                trans.Abort()
            Finally
                trans.Dispose()
            End Try
        End Sub
    End Class

    Public Class PromptingUtilites
#Region "PromptForObject"

        Public Shared Function PromptForObject(ByVal objType As System.Type, ByVal exactMatch As Boolean) As ObjectId
            Dim prOptions As PromptEntityOptions = New PromptEntityOptions("")

            prOptions.Message = "\nSelect a " + objType.ToString() + ": "
            prOptions.SetRejectMessage("\nObject must be of type " + objType.ToString())
            prOptions.AddAllowedClass(objType, exactMatch)

            Try
                ' prompt the user to select a member
                '
                Dim prResult As PromptEntityResult = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor.GetEntity(prOptions)

                If prResult.Status = PromptStatus.Cancel Then
                    Return Autodesk.AutoCAD.DatabaseServices.ObjectId.Null
                End If

                ' get the ObjectId from the selected member 
                '
                Return prResult.ObjectId
            Catch ex As SystemException
                Throw ex
            End Try
        End Function
#End Region
    End Class
End Namespace


