#Region " Autodesk Copyright "

'      .NET Sample
'
'      Copyright (c) 2008 by Autodesk, Inc.
'
'      Permission to use, copy, modify, and distribute this software
'      for any purpose and without fee is hereby granted, provided
'      that the above copyright notice appears in all copies and
'      that both that copyright notice and the limited warranty and
'      restricted rights notice below appear in all supporting
'      documentation.
'
'      AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
'      AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
'      MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
'      DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
'      UNINTERRUPTED OR ERROR FREE.
'
'      Use, duplication, or disclosure by the U.S. Government is subject to
'      restrictions set forth in FAR 52.227-19 (Commercial Computer
'      Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
'      (Rights in Technical Data and Computer Software), as applicable.
'

#End Region

#Region "Namespaces References"

Imports System
Imports System.Collections.Generic
Imports System.Text
Imports System.Diagnostics

' AutoCAD MEP Specific
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.GeomeTry
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.EditorInput
Imports Autodesk.Aec.Building.ApplicationServices
Imports Autodesk.Aec.Building.DatabaseServices

' create an alias to the AutoCAD Application object
Imports AcadApplication = Autodesk.AutoCAD.ApplicationServices.Application

#End Region

<Assembly: Autodesk.AutoCAD.Runtime.CommandClass(GetType(ShowConnectedParts.ShowConnectedParts))> 

Namespace ShowConnectedParts
    Public Class ShowConnectedParts
        Private mEditor As Editor
        Private mDatabase As Database
        Private mTransMgr As Autodesk.AutoCAD.DatabaseServices.TransactionManager
        Dim trans As Transaction

#Region "ShowConnectedParts Command"

        ' Simple sample to demonstrate how to modify a database resident part
        '

        ' Register command ShowConnectedParts
        '
        <Autodesk.AutoCAD.Runtime.CommandMethod("ShowConnectedParts", "ShowConnectedParts", Autodesk.AutoCAD.Runtime.CommandFlags.Modal)> _
        Public Sub ShowConnected()

            ' Get current database to start transaction
            '
            mEditor = AcadApplication.DocumentManager.MdiActiveDocument.Editor
            mDatabase = AcadApplication.DocumentManager.MdiActiveDocument.Database
            mTransMgr = mDatabase.TransactionManager

            ' need to start the trans action before calling getMemberObject
            trans = mTransMgr.StartTransaction()

            Try
                ' prompt for a member object
				Dim memberObject As Member = getMemberObject(vbNewLine + "Select object to highlight connected objects:")

                If Not memberObject Is Nothing Then
                    ' Call function getConnectedObjects to get  a collection of connected objects
                    Dim oIds As ObjectIdCollection = getConnectedObjects(memberObject)

                    Dim id As ObjectId
                    ' Iterate all connected objects
                    For Each id In oIds
                        ' Get an object in the collection
                        Dim memHighLight As Member = CType(mTransMgr.GetObject(id, OpenMode.ForRead, False), Member)
                        ' Highlight the object
                        memHighLight.Highlight()
                        memHighLight.Dispose()
                    Next

                    ' Pause and wait for user input to continue.
                    mEditor.GetString(vbNewLine + "Press enter to continue")

                    ' Iterate all connected objects again

                    For Each id In oIds
                        ' Get an object in the collection
                        Dim memHighLight As Member = CType(mTransMgr.GetObject(id, OpenMode.ForRead, False), Member)
                        ' now unhighlight 
                        memHighLight.Unhighlight()
                        memHighLight.Dispose()
                    Next
                    ' Make sure the transaction is committed
                    trans.Commit()
                Else
                    ' If something goes wrong the transaction should be aborted.
                    trans.Abort()
                End If

                'Catch any errors found
            Catch e As System.Exception
                Debug.WriteLine(e.Message)
                trans.Abort()
            Finally
                trans.Dispose()
            End Try
        End Sub

#End Region

#Region "ListConnectedParts Command"

        ' Simple sample to demonstrate how to modify a database resident part
        '
        <Autodesk.AutoCAD.Runtime.CommandMethod("ShowConnectedParts", "ListConnectedParts", Autodesk.AutoCAD.Runtime.CommandFlags.Modal)> _
        Public Sub ListConnected()
            Try
                mEditor = AcadApplication.DocumentManager.MdiActiveDocument.Editor
                mDatabase = AcadApplication.DocumentManager.MdiActiveDocument.Database
                mTransMgr = mDataBase.TransactionManager


                ' need to start the trans action before calling getMemberObject
                trans = mTransMgr.StartTransaction()

                ' prompt for a member object
				Dim memberObject As Member = getMemberObject(vbNewLine + "Select object to list connected objects:")

                If Not memberObject Is Nothing Then
                    Dim oIds As ObjectIdCollection = getConnectedObjects(memberObject)

                    Dim id As ObjectId
                    For Each id In oIds
                        Dim memList As Member = CType(mTransMgr.GetObject(id, OpenMode.ForRead, False), Member)
                        memList.List()
                        memList.Dispose()
                    Next
                    trans.Commit()
                Else
                    trans.Abort()
                End If
            Catch e As SystemException
                Debug.WriteLine(e.Message)
                trans.Abort()
            Finally
                trans.Dispose()
            End Try
        End Sub

#End Region

        Private Function getMemberObject(ByVal prompt As String) As Member

            ' Function that asks the user to pick an object
            '
            Dim member As Member = Nothing
            Try
                'Get current database to start transaction
                mEditor = AcadApplication.DocumentManager.MdiActiveDocument.Editor
                mDatabase = AcadApplication.DocumentManager.MdiActiveDocument.Database
                mTransMgr = mDatabase.TransactionManager

                ' set up the entity prompt options, note that we only want a member object
                Dim prOptions As PromptEntityOptions = New PromptEntityOptions(prompt)
                prOptions.SetRejectMessage("\nObject must be of type Member.")
                prOptions.AddAllowedClass(GetType(Member), False)

                ' prompt the user to select a member
                Dim prResult As PromptEntityResult = mEditor.GetEntity(prOptions)

                ' check if user cancelled the prompt
                If prResult.Status = PromptStatus.Cancel Then
                    Return Nothing
                End If

                ' get the ObjectId from the selected object 
                Dim oidSelected As Autodesk.AutoCAD.DatabaseServices.ObjectId = prResult.ObjectId

                ' open the Block table and get the BlockTableRecord object from the table
                Dim bt As BlockTable = CType(mTransMgr.GetObject(mDatabase.BlockTableId, OpenMode.ForRead, False), BlockTable)
                Dim btr As BlockTableRecord = CType(mTransMgr.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForRead, False), BlockTableRecord)

                ' open the member for Read based on the objectId
                member = CType(mTransMgr.GetObject(oidSelected, OpenMode.ForRead, False), Member)

                ' Catch any errors found
            Catch e As System.Exception
                Debug.Write(e.ToString())
                Throw e
            End Try

            ' Return the object
            Return member
        End Function

        Private Function getConnectedObjects(ByVal member As Member) As ObjectIdCollection
            ' Function that prints information about connectors on the Member object
            ' It also returns a collection of ObjectId's for objects that are connected to the Member object

            ' Create a new collection of ObjectId's
            Dim oIdsConn As ObjectIdCollection = New ObjectIdCollection()

            ' Get all connectors on Member
            Dim connComps As ConnectionComponentMemberCollection = member.ConnectionComponentMembers

            Dim ccm As ConnectionComponentMember
            ' Iterate all connectors found on Member
            For Each ccm In connComps
                ' Get all other objects connected to Member in this connector
                Dim oids As Autodesk.AutoCAD.DatabaseServices.ObjectIdCollection = member.GetObjectsAtConnectionComponent(ccm)

                Dim newLineTab As String = vbNewLine + vbTab

                mEditor.WriteMessage(vbNewLine + "Connection name: " + ccm.Name)

                ' print out info about the connectionComponent
                '
                mEditor.WriteMessage(newLineTab + "Connection domain: " + ccm.Domain.ToString())
                mEditor.WriteMessage(newLineTab + "Connection style id: " + ccm.StyleId.ToString())
                mEditor.WriteMessage(newLineTab + "Connection type: " + ccm.ConnectionType.ToString())

                ' Check if the connector has a valid Id
                If ccm.NodeId > 0 Then

                    Dim vdir As Vector3d = member.GetLayoutNodeDirectionAtPoint(ccm.Point)

                    If TypeOf member Is MultiViewPart Then
                        mEditor.WriteMessage(newLineTab + "Direction: " + ccm.Direction.ToString())
                    Else
                        mEditor.WriteMessage(newLineTab + "Direction: " + vdir.ToString())
                    End If

                End If

                ' Print out more information about the connector
                mEditor.WriteMessage(newLineTab + "Flow type: " + ccm.FlowType.ToString())
                mEditor.WriteMessage(newLineTab + "Gender: " + ccm.Gender.ToString())
                mEditor.WriteMessage(newLineTab + "Is Curve connector: " + ccm.IsCurveConnector.ToString())
                mEditor.WriteMessage(newLineTab + "System type: " + ccm.SystemType.ToString())
                mEditor.WriteMessage(newLineTab + "Connected objects: " + oids.Count.ToString())

                Dim id As Autodesk.AutoCAD.DatabaseServices.ObjectId
                ' Finally add all ObjectId's for connected objects into the collection
                For Each id In oids
                    If Not oIdsConn.Contains(id) Then
                        oIdsConn.Add(id)
                    End If
                Next
            Next
            ' return the collection of connected objects
            Return oIdsConn
        End Function
    End Class
End Namespace


